/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

/*
 * Code to store information about the type of argument
 * that is allowed in a certain situation.
 */

#ifndef __ARG_HINTS_H_
#define __ARG_HINTS_H_

typedef struct _ArgHints ArgHints;
typedef struct _ArgHintsDomainTable ArgHintsDomainTable;
typedef struct _ArgHintsTable ArgHintsTable;

#include <glib.h>
#include <gdam/common/gdamarg.h>

struct _ArgHints {
	/* map from strings to GINT_TO_POINTER(GdamArgType) */
	GHashTable*	type_table;

	gboolean	tried_parent;
	ArgHints*	parent_hints;
};

GdamArgType    arg_hints_lookup(ArgHints*       arg_hints,
                                const char*     arg_name);


struct _ArgHintsTable {

	/* map from class_name => arg_type */
	GHashTable*	arg_table;

	/* map from (class_name) => (ghashtable: function_name => arg_type) */
	GHashTable*	function_arg_table;

	/* map from (class_name) => (GSList which include the name,
	                             and continues up through the parents */
	GHashTable*     name_to_branch;
};

ArgHintsTable* arg_hints_table_builtin    ();
ArgHintsTable* arg_hints_table_load       (const char*        filename);

ArgHints*      arg_hints_table_lookup_arg (ArgHintsTable*     table,
                                           const char*        class_name);
ArgHints*      arg_hints_table_lookup_func(ArgHintsTable*     table,
                                           const char*        class_name,
					   const char*        function_name);
const char*    arg_hints_table_lookup_parent(ArgHintsTable*   table,
                                           const char*        class_name);

void           arg_hints_table_destroy    (ArgHintsTable*     table);



/* private */
ArgHintsTable* arg_hints_table_new();
void           arg_hints_table_add_base_class (ArgHintsTable* table,
                                           const char*    base_class);
                      
void           arg_hints_table_add_parent (ArgHintsTable* table,
                                           const char*    base_class,
					   const char*    derived_class);

void           arg_hints_table_add_func   (ArgHintsTable* table,
					   const char*    class_name,
					   const char*    func_name,
                                           const char*    arg_name,
					   GdamArgType    arg_type);
void           arg_hints_table_add_arg    (ArgHintsTable* table,
					   const char*    class_name,
                                           const char*    arg_name,
					   GdamArgType    arg_type);
ArgHints*      arg_hints_table_force_arg  (ArgHintsTable* table,
                                           const char*    class_name);
ArgHints*      arg_hints_table_force_func (ArgHintsTable* table,
                                           const char*    class_name,
					   const char*    function_name);
#endif
