#! /usr/bin/perl -w

if (scalar(@ARGV) == 0) {
	print <<"EOF";
usage: $0 arg-table-file

Generates C code to make the builtin arg-hints-table.
EOF
	exit 1;
}

open INPUT, "$ARGV[0]" or die "couldn't open $ARGV[0]";

print <<"EOF";

/*
 * This file is machine-generated by
 * $0 [with $ARGV[0] as input].
 *
 * See COPYRIGHT, it may as well apply to this file.
 */

#include "arghints.h"

ArgHintsTable* arg_hints_table_builtin()
{
	ArgHintsTable* table;
	table = arg_hints_table_new();

EOF

while (<INPUT>) {
	# skip comments.
	next if (/^\s*$/ || /^\s*\#/);

	if (/^BASE\s+(\w+)\s+$/) {
		my $base_class = $1;
		print "arg_hints_table_add_base_class(table, ";
		print "\"$base_class\");\n";
		next;
	}

	if (/^PARENT\s+(\w+)\s+(\w+)\s+$/) {
		my ($base_class, $derived_class) = ($1, $2);
		print "arg_hints_table_add_parent(table, ";
		print "\"$base_class\", \"$derived_class\");\n";
		next;
	}

	if (s/^FUNC\s+(\w+)\s+(\w+)\s+//) {
		my ($class, $func_name) = ($1, $2);
		while (s/(\w+)\s+(\w+)\s+//) {
			my ($arg_name, $type) = ($1, $2);
			print "arg_hints_table_add_func(table, ";
			print "\"$class\", \"$func_name\", ";
			print "\"$arg_name\", GDAM_ARG_TYPE_", uc($type), ");\n";
		}
		next;
	}

	if (/^ARG\s+(\w+)\s+(\w+)\s+(\w+)\s+$/) {
		my ($class, $arg_name, $type) = ($1, $2, $3);
		print "arg_hints_table_add_arg(table, ";
		print "\"$class\", \"$arg_name\", ";
		print "GDAM_ARG_TYPE_", uc($type), ");\n";
		next;
	}

	chomp;
	die "unparseable line: $_";
}

print <<"EOF";

	return table;
}
EOF
