/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamprinter.h"
#include <stdlib.h>

struct _GdamPrinter {
	int indent;
	FILE* fp;
};

GdamPrinter*   gdam_printer_new(FILE* out)
{
  GdamPrinter *printer = g_new (GdamPrinter, 1);
  printer->indent = 0;
  printer->fp = out;
  return printer;
}

void           gdam_printer_indent(GdamPrinter* printer)
{
  printer->indent++;
}
void           gdam_printer_unindent(GdamPrinter* printer)
{
  printer->indent--;
  g_assert (printer->indent >= 0);
}

static const char* spaces(int num_spaces)
{
  static int spaces_len = -1;
  static char *rv = NULL;
  if (num_spaces > spaces_len)
    {
      char *at;
      rv = (char *) realloc (rv, num_spaces + 1);
      at = rv;
      if (spaces_len > 0)
	at += spaces_len;
      while (at < rv + num_spaces)
	*at++ = ' ';
      *at = 0;
      num_spaces = spaces_len;
    }
  return rv + (spaces_len - num_spaces);
}

void gdam_printer_write_line(GdamPrinter* printer, const char* fmt, ...)
{
  va_list args;
  fprintf (printer->fp, spaces (printer->indent));
  va_start (args, fmt);
  vfprintf (printer->fp, fmt, args);
  va_end (args);
  fputc ('\n', printer->fp);
}

void           gdam_printer_destroy(GdamPrinter* printer)
{
  if (printer->fp)
    fclose (printer->fp);
  g_free (printer);
}
void           gdam_printer_destroy_without_fclose(GdamPrinter* printer)
{
  g_free (printer);
}


