#include "gdam-protocol.h"
#include <glib.h>
#include <string.h>

typedef struct _NamedEnum NamedEnum;
struct _NamedEnum
{
  int value;
  const char* name;
};
#define NAMED_ENUM(word)    { (int) word, #word }

static NamedEnum profile_enums[] =
{
  NAMED_ENUM (GDAM_PROTOCOL_PROFILE_AVERAGE),
  NAMED_ENUM (GDAM_PROTOCOL_PROFILE_LEFT),
  NAMED_ENUM (GDAM_PROTOCOL_PROFILE_RIGHT),
  NAMED_ENUM (GDAM_PROTOCOL_PROFILE_STEREO),
  NAMED_ENUM (GDAM_PROTOCOL_PROFILE_FFT),
  NAMED_ENUM (GDAM_PROTOCOL_PROFILE_FFT+1),
  NAMED_ENUM (GDAM_PROTOCOL_PROFILE_FFT+2),
  NAMED_ENUM (GDAM_PROTOCOL_PROFILE_FFT+3),
  NAMED_ENUM (GDAM_PROTOCOL_PROFILE_FFT+4),
  NAMED_ENUM (GDAM_PROTOCOL_PROFILE_FFT+5),
  NAMED_ENUM (GDAM_PROTOCOL_PROFILE_FFT+6),
  NAMED_ENUM (GDAM_PROTOCOL_PROFILE_FFT_LAST),
  NAMED_ENUM (GDAM_PROTOCOL_PROFILE_MIN_MAX),
};

static NamedEnum arg_enums[] =
{
  NAMED_ENUM (GDAM_PROTOCOL_SOURCE_ARG),
  NAMED_ENUM (GDAM_PROTOCOL_BUFFER_ARG),
};

static NamedEnum message_enums[] =
{
  NAMED_ENUM (GDAM_MESSAGE_FIRST_MESSAGE),
  NAMED_ENUM (GDAM_MESSAGE_OPEN_DEVICE),
  NAMED_ENUM (GDAM_MESSAGE_FEED_DEVICE),
  NAMED_ENUM (GDAM_MESSAGE_CLOSE_DEVICE),
  NAMED_ENUM (GDAM_MESSAGE_CREATE),
  NAMED_ENUM (GDAM_MESSAGE_DETACH_SOURCE),
  NAMED_ENUM (GDAM_MESSAGE_SUBSCRIBE_SOURCE),
  NAMED_ENUM (GDAM_MESSAGE_UNSUBSCRIBE),
  NAMED_ENUM (GDAM_MESSAGE_CONFIGURE_SOURCE),
  NAMED_ENUM (GDAM_MESSAGE_GET_SERVER_INT),
  NAMED_ENUM (GDAM_MESSAGE_START_CACHING),
  NAMED_ENUM (GDAM_MESSAGE_UNCACHE),
  NAMED_ENUM (GDAM_MESSAGE_GET_CACHE_PROFILE),
  NAMED_ENUM (GDAM_MESSAGE_SYNCHRONIZE),
  NAMED_ENUM (GDAM_MESSAGE_SYNC_PREVIEW),
  NAMED_ENUM (GDAM_MESSAGE_SOURCE),
  NAMED_ENUM (GDAM_MESSAGE_SET_ARGS),
  NAMED_ENUM (GDAM_MESSAGE_GET_ARGS),
  NAMED_ENUM (GDAM_MESSAGE_GET_ALL_ARGS),
  NAMED_ENUM (GDAM_MESSAGE_REOPEN_DEVICE),
  NAMED_ENUM (GDAM_MESSAGE_CALL_FUNCTION),
  NAMED_ENUM (GDAM_MESSAGE_REF),
  NAMED_ENUM (GDAM_MESSAGE_UNREF),
  NAMED_ENUM (GDAM_MESSAGE_PACKAGE),
  NAMED_ENUM (GDAM_MESSAGE_GET_NAMED_BUFFERS),
  NAMED_ENUM (GDAM_RESPONSE_FAILED),
  NAMED_ENUM (GDAM_RESPONSE_TIME_SUCCESS),
  NAMED_ENUM (GDAM_RESPONSE_INT_SUCCESS),
  NAMED_ENUM (GDAM_RESPONSE_2INT_SUCCESS),
  NAMED_ENUM (GDAM_RESPONSE_DATA_SUCCESS),
  NAMED_ENUM (GDAM_RESPONSE_SUCCESS),
  NAMED_ENUM (GDAM_RESPONSE_ARG_SUCCESS),
  NAMED_ENUM (GDAM_RESPONSE_CACHING_STARTED),
  NAMED_ENUM (GDAM_RESPONSE_PACKAGE),
  NAMED_ENUM (GDAM_EVENT_FIRST),
  NAMED_ENUM (GDAM_EVENT_SOURCE_ENDED),
  NAMED_ENUM (GDAM_EVENT_SOURCE_TIMER),
  NAMED_ENUM (GDAM_EVENT_CACHE_BUFFER_FINISH),
  NAMED_ENUM (GDAM_EVENT_ERROR_MESSAGE)
};

static NamedEnum configure_enums[] =
{
  NAMED_ENUM (GDAM_CFG_MIX_ADD),
  NAMED_ENUM (GDAM_CFG_GET_TIME),
  NAMED_ENUM (GDAM_CFG_SOURCE_SEEK_RELATIVE),
  NAMED_ENUM (GDAM_CFG_SEQUENCE_CHANGE)
};

static NamedEnum subscription_enums[] =
{
  NAMED_ENUM (GDAM_SUBSCRIPTION_ENDED),
  NAMED_ENUM (GDAM_SUBSCRIPTION_TIMER)
};

static const char*
get_enum_by_name(NamedEnum *array,
		 int        length,
		 int        v)
{
  int i;
  for (i = 0; i < length; i++)
    if (array[i].value == v)
      return array[i].name;
  return NULL;
}

#define GET_ENUM_BY_NAME(array)						\
    (get_enum_by_name (array, sizeof (array) / sizeof ((array)[0]), v))
			

const char*
gdam_protocol_get_enum_name(const char *family,
			    int         v)
{
  if (strcmp (family, "profile") == 0)
    return GET_ENUM_BY_NAME (profile_enums);
  if (strcmp (family, "arg") == 0)
    return GET_ENUM_BY_NAME (arg_enums);
  if (strcmp (family, "message") == 0)
    return GET_ENUM_BY_NAME (message_enums);
  if (strcmp (family, "configure") == 0)
    return GET_ENUM_BY_NAME (configure_enums);
  if (strcmp (family, "subscription") == 0)
    return GET_ENUM_BY_NAME (subscription_enums);
  return "unknown enum family";
}
