#include "gdammathfunction.h"
#include <math.h>

static inline gdouble evaluate(GdamMathFunction* function, gdouble* input)
{
  return (*function->any.evaluator) (function, input);
}

static gdouble product_evaluator(GdamMathFunction*     function,
			         gdouble*              input_vector)
{
  gdouble eval = 1.0;
  int i;
  GdamMathFunctionOperator *func_operator;

  func_operator = (GdamMathFunctionOperator *) function;
  for (i = 0; i < func_operator->num_args; i++)
    {
      if (func_operator->whether_inverted[i])
	eval /= evaluate (func_operator->args[i], input_vector);
      else
	eval *= evaluate (func_operator->args[i], input_vector);
    }
  return eval;
}

static gdouble sum_evaluator(GdamMathFunction*     function,
			     gdouble*              input_vector)
{
  gdouble eval = 0.0;
  int i;
  GdamMathFunctionOperator *func_operator;

  func_operator = (GdamMathFunctionOperator *) function;
  for (i = 0; i < func_operator->num_args; i++)
    {
      if (func_operator->whether_inverted[i])
	eval -= evaluate (func_operator->args[i], input_vector);
      else
	eval += evaluate (func_operator->args[i], input_vector);
    }
  return eval;
}

static inline gdouble arctan(gdouble v)
{
  return atan2 (v, 1);
}


/* 
 * blech.  ugly macros to map
 *
 *         c-library math functions  ==> gdammathfunctionclass::evaluate
 */
#define MAKE_SIMPLE_EVALUATOR(func)					\
static gdouble func ## _evaluator(GdamMathFunction*     function,	\
			          gdouble*              input_vector)	\
{                                                                       \
  GdamMathFunction * sub = function->named.argument;                    \
  return func (evaluate (sub, input_vector));                           \
}
MAKE_SIMPLE_EVALUATOR(sin)
MAKE_SIMPLE_EVALUATOR(cos)
MAKE_SIMPLE_EVALUATOR(tanh)
MAKE_SIMPLE_EVALUATOR(arctan)
MAKE_SIMPLE_EVALUATOR(log)
MAKE_SIMPLE_EVALUATOR(exp)
MAKE_SIMPLE_EVALUATOR(sinh)
MAKE_SIMPLE_EVALUATOR(cosh)
MAKE_SIMPLE_EVALUATOR(sqrt)

void gdam_math_function_init()
{
  static gboolean inited = FALSE;

  if (inited)
    return;
  inited = TRUE;

  gdam_math_function_class_new_binary ("*", "/", 80, product_evaluator);
  gdam_math_function_class_new_binary ("+", "-", 50, sum_evaluator);

#define MAKE_SIMPLE_CLASS_NEW(name)					\
	gdam_math_function_class_new_named_function(#name,		\
	                                            name##_evaluator);

  MAKE_SIMPLE_CLASS_NEW (sin)
  MAKE_SIMPLE_CLASS_NEW (cos)
  MAKE_SIMPLE_CLASS_NEW (tanh)
  MAKE_SIMPLE_CLASS_NEW (arctan)
  MAKE_SIMPLE_CLASS_NEW (log)
  MAKE_SIMPLE_CLASS_NEW (exp)
  MAKE_SIMPLE_CLASS_NEW (sinh)
  MAKE_SIMPLE_CLASS_NEW (cosh)
  MAKE_SIMPLE_CLASS_NEW (sqrt)
}
