/*
 * This code was translated from the perl
 * module by Chris Nandor, pudge@pobox.com, http://pudge.net/.
 *
 * I believe that originally it came from C code
 * written by Johann Lindvall, but I wasn't able to find that. :(
 *
 * This code written by Dave Benson. 2000.
 */

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <string.h>
#include <ctype.h>
#include <gdam/common/gdamdebug.h>
#include "gdammp3info.h"
//#include "../gdamskinutil.h"

typedef struct _MapName MapName;

struct _MapName {
	const char* id;
	const char* fullname;
};

static MapName v2_names[] = {
	{ "TIT2", "TITLE" },
	{ "TPE1", "ARTIST" },
	{ "TALB", "ALBUM" },
	{ "TYER", "YEAR" },
	{ "COMM", "COMMENT" },
	{ "TRCK", "TRACKNUM" },
	{ "TCON", "GENRE" },

	{ "AENC", "Audio encryption" },
	{ "APIC", "Attached picture" },
	{ "COMR", "Commercial frame" },
	{ "ENCR", "Encryption method registration" },
	{ "EQUA", "Equalization" },
	{ "ETCO", "Event timing codes" },
	{ "GEOB", "General encapsulated object" },
	{ "GRID", "Group identification registration" },
	{ "IPLS", "Involved people list" },
	{ "LINK", "Linked information" },
	{ "MCDI", "Music CD identifier" },
	{ "MLLT", "MPEG location lookup table" },
	{ "OWNE", "Ownership frame" },
	{ "PCNT", "Play counter" },
	{ "POPM", "Popularimeter" },
	{ "POSS", "Position synchronisation frame" },
	{ "PRIV", "Private frame" },
	{ "RBUF", "Recommended buffer size" },
	{ "RVAD", "Relative volume adjustment" },
	{ "RVRB", "Reverb" },
	{ "SYLT", "Synchronized lyric/text" },
	{ "SYTC", "Synchronized tempo codes" },
	{ "TBPM", "BPM (beats per minute)" },
	{ "TCOM", "Composer" },
	{ "TCOP", "Copyright message" },
	{ "TDAT", "Date" },
	{ "TDLY", "Playlist delay" },
	{ "TENC", "Encoded by" },
	{ "TEXT", "Lyricist/Text writer" },
	{ "TFLT", "File type" },
	{ "TIME", "Time" },
	{ "TIT1", "Content group description" },
	{ "TIT3", "Subtitle/Description refinement" },
	{ "TKEY", "Initial key" },
	{ "TLAN", "Language(s)" },
	{ "TLEN", "Length" },
	{ "TMED", "Media type" },
	{ "TOAL", "Original album/movie/show title" },
	{ "TOFN", "Original filename" },
	{ "TOLY", "Original lyricist(s)/text writer(s)" },
	{ "TOPE", "Original artist(s)/performer(s)" },
	{ "TORY", "Original release year" },
	{ "TOWN", "File owner/licensee" },
	{ "TPE2", "Band/orchestra/accompaniment" },
	{ "TPE3", "Conductor/performer refinement" },
	{ "TPE4", "Interpreted, remixed, or otherwise modified by" },
	{ "TPOS", "Part of a set" },
	{ "TPUB", "Publisher" },
	{ "TRDA", "Recording dates" },
	{ "TRSN", "Internet radio station name" },
	{ "TRSO", "Internet radio station owner" },
	{ "TSIZ", "Size" },
	{ "TSRC", "ISRC (international standard recording code)" },
	{ "TSSE", "Software/Hardware and settings used for encoding" },
	{ "TXXX", "User defined text information frame" },
	{ "UFID", "Unique file identifier" },
	{ "USER", "Terms of use" },
	{ "USLT", "Unsychronized lyric/text transcription" },
	{ "WCOM", "Commercial information" },
	{ "WCOP", "Copyright/Legal information" },
	{ "WOAF", "Official audio file webpage" },
	{ "WOAR", "Official artist/performer webpage" },
	{ "WOAS", "Official audio source webpage" },
	{ "WORS", "Official internet radio station homepage" },
	{ "WPAY", "Payment" },
	{ "WPUB", "Publishers official webpage" },
	{ "WXXX", "User defined URL link frame" },

#if 0
	{ "TIT2", "Title/songname/content description" },
	{ "TPE1", "Lead performer(s)/Soloist(s)" },
	{ "TALB", "Album/Movie/Show title" },
	{ "TYER", "Year" },
	{ "COMM", "Comments" },
	{ "TRCK", "Track number/Position in set" },
	{ "TCON", "Content type" },
#endif
};

static gboolean map_names_inited = FALSE;

static const char* genre_table[] = {
        "Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk",
        "Grunge", "Hip-Hop", "Jazz", "Metal", "New Age", "Oldies",
        "Other", "Pop", "R&B", "Rap", "Reggae", "Rock", "Techno",
        "Industrial", "Alternative", "Ska", "Death Metal", "Pranks",
        "Soundtrack", "Euro-Techno", "Ambient", "Trip-Hop", "Vocal",
        "Jazz+Funk", "Fusion", "Trance", "Classical", "Instrumental",
        "Acid", "House", "Game", "Sound Clip", "Gospel", "Noise",
        "AlternRock", "Bass", "Soul", "Punk", "Space", "Meditative",
        "Instrumental Pop", "Instrumental Rock", "Ethnic", "Gothic",
        "Darkwave", "Techno-Industrial", "Electronic", "Pop-Folk",
        "Eurodance", "Dream", "Southern Rock", "Comedy", "Cult",
        "Gangsta", "Top 40", "Christian Rap", "Pop/Funk", "Jungle",
        "Native American", "Cabaret", "New Wave", "Psychadelic",
        "Rave", "Showtunes", "Trailer", "Lo-Fi", "Tribal", "Acid Punk",
        "Acid Jazz", "Polka", "Retro", "Musical", "Rock & Roll",
        "Hard Rock",
};
static const char* find_genre(int genre)
{
  if (genre < 0 || genre >= G_N_ELEMENTS (genre_table))
    return "Unknown Genre";
  return genre_table[genre];
}
static void
add_hash_ref    (GdamMp3Info* mp3,
                 const char*  key,
		 char*        val)
{
  char *end = strchr (val, '\0');

  /* Chomp off trailing whitespace in all cases. */
  while (end > val && isspace (end[-1]))
    end--;
  *end = '\0';
  if (g_hash_table_lookup (mp3->extra_values, key) == NULL)
    {
      g_hash_table_insert (mp3->extra_values, g_strdup (key), val);
      return;
    }
  g_hash_table_insert (mp3->extra_values, (char *) key, val);
}
static void
add_hash_val    (GdamMp3Info* mp3,
                 const char*  key,
		 const char*  val,
		 int          value_length)
{
  GHashTable *extra;
  char *value = NULL;
  if (mp3->extra_values == NULL)
    mp3->extra_values = g_hash_table_new (g_str_hash, g_str_equal);
  extra = mp3->extra_values;
  if (memchr (val, '\0', value_length) == NULL)
    {
      value = g_new (char, value_length + 1);
      memcpy (value, val, value_length);
      value[value_length] = '\0';
    }
  else
    {
      value = g_strdup (val);
    }
  add_hash_ref (mp3, key, value);
}
static int compare_map_names(const void* a, const void* b)
{
  const char *key_a = ((MapName *) a)->id;
  const char *key_b = ((MapName *) b)->id;
  return strcmp (key_a, key_b);
}
static void initialize_map_names()
{
  qsort (v2_names,
	 G_N_ELEMENTS (v2_names), sizeof (MapName), compare_map_names);
  map_names_inited = TRUE;
}

static const char* lookup_v2_name(const char* in)
{
  MapName *map_name;
  MapName saught = { in, NULL };
  if (!map_names_inited)
    initialize_map_names ();
  map_name = bsearch (&saught,
		      v2_names,
		      G_N_ELEMENTS (v2_names),
		      sizeof (MapName), compare_map_names);
  return map_name == NULL ? NULL : map_name->id;
}

static void
add_hash_v2_val(GdamMp3Info* mp3,
                const char*  v2_key,
		const char*  val,
		int          value_length)
{
  const char *key;
  if (!map_names_inited)
    initialize_map_names ();
  key = lookup_v2_name (v2_key);
  add_hash_val (mp3, key, val, value_length);
}

static void
add_hash_str    (GdamMp3Info* mp3,
                 const char*  key,
		 const char*  val)
{
  add_hash_ref (mp3, key, g_strdup (val));
}

static void
add_hash_num    (GdamMp3Info* mp3,
                 const char*  key,
                 int          num)
{
  add_hash_ref (mp3, key, g_strdup_printf ("%d", num));
}


static gboolean get_v2_head(FILE* fp,
                            GdamMp3Info* mp3_info,
			    int header_size,
			    char* header)
{
  char *buf = header;
  if (header_size < 3)
    {
      g_warning ("file way too short");
      return FALSE;
    }

  /* check first three bytes for 'ID3' */
  if (memcmp (header, "ID3", 3) != 0)
    {
      return FALSE;
    }

  mp3_info->has_id3_data = TRUE;

  /* get version */
  mp3_info->major_version = (int) (guint8) buf[3];
  mp3_info->minor_version = (int) (guint8) buf[4];

  /* get flags: TODO: check these against the perl module,
     realworld examples, and the standard. */
  mp3_info->unsync = (buf[5] & 0x80) ? 1 : 0;
  mp3_info->extended_header = (buf[5] & 0x40) ? 1 : 0;
  mp3_info->experimental = (buf[5] & 0x20) ? 1 : 0;

  /* get ID3v2 tag length from bytes 7-10 */
  mp3_info->tag_size = 10
    + (((int) (guint8) buf[6]) << (7 * 3))
    + (((int) (guint8) buf[7]) << (7 * 2))
    + (((int) (guint8) buf[8]) << (7 * 1))
    + (((int) (guint8) buf[9]) << (7 * 0));

  if (mp3_info->tag_size > 65536)
    {
      g_warning ("mp3 tag too large (%d) ", mp3_info->tag_size);
      return FALSE;
    }

  /* get extended header size, if needed. */
  if (mp3_info->extended_header)
    {
      int offset;
      int max_read_size;
      mp3_info->ext_header_size = 10
	+ (((int) (guint8) buf[10]) << (7 * 3))
	+ (((int) (guint8) buf[11]) << (7 * 2))
	+ (((int) (guint8) buf[12]) << (7 * 1))
	+ (((int) (guint8) buf[13]) << (7 * 0));
      offset = mp3_info->ext_header_size + 10;

      gdam_debug ("parsing ID3v2 tags");

      while (offset < mp3_info->tag_size)
	{
	  char id[5];
	  int size;
	  char buf[10];
	  char tmpbuf[2048];
	  if (fseek (fp, offset, 0) < 0)
	    break;
	  if (fread (buf, sizeof (buf), 1, fp) != 1)
	    break;
	  if (!isalnum (buf[0]) || !isalnum (buf[1])
	      || !isalnum (buf[2]) || !isalnum (buf[3]))
	    break;
	  size = GUINT32_FROM_LE (*(guint32 *) buf);
	  memcpy (id, buf, 4);
	  id[4] = '\0';

	  /* `id' and `size' */
	  max_read_size = MIN (size, (int) sizeof (tmpbuf));
	  if (fread (tmpbuf, max_read_size, 1, fp) == 1)
	    {
	      add_hash_v2_val (mp3_info, id, tmpbuf, max_read_size);
	    }
	}
    }

  return TRUE;
}

static unsigned bitrate_table[2][3][16] = {
	{
		{0,32,48,56,64,80,96,112,128,144,160,176,192,224,256},
		{0,8,16,24,32,40,48,56,64,80,96,112,128,144,160},
		{0,8,16,24,32,40,48,56,64,80,96,112,128,144,160}
	},{
		{0,32,64,96,128,160,192,224,256,288,320,352,384,416,448},
		{0,32,48,56,64,80,96,112,128,160,192,224,256,320,384},
		{0,32,40,48,56,64,80,96,112,128,160,192,224,256,320}
	}
};
static unsigned sampling_frequency_table[2][3] = {
        {22050, 24000, 16000},
        {44100, 48000, 32000}
};


/* Fill in the mp3_info from the 4 `header_bytes'.
 *
 * The filename is only used to print better error messages :)
 */
static gboolean get_mp3_header(const char* filename,
                               GdamMp3Info* mp3_info,
			       char* header_bytes)
{
  guint32 header_in_little_endian = *(guint32 *) header_bytes;
  guint32 header = GUINT32_FROM_BE (header_in_little_endian);
  int sampling_freq_index, bitrate_index, frequency_index;
  mp3_info->id = (header >> 19) & 1;
  mp3_info->layer = (header >> 17) & 3;
  mp3_info->protection_bit = (header >> 16) & 1;
  bitrate_index = (header >> 12) & 15;
  sampling_freq_index = (header >> 10) & 3;
  mp3_info->padding_bit = (header >> 9) & 1;
  mp3_info->private_bit = (header >> 8) & 1;
  mp3_info->mode = (header >> 6) & 3;
  mp3_info->mode_extension = (header >> 4) & 3;
  mp3_info->copyright = (header >> 3) & 1;
  mp3_info->original = (header >> 2) & 1;
  mp3_info->emphasis = (header >> 0) & 3;
  mp3_info->version_index = (header >> 19) & 3;

#if 0
  fprintf (stderr,
	   "get_mp3_header [debug]:\n"
	   "   id=%d, layer=%d, protection_bit=%d, bitrate_index=%d\n"
	   "   sampling_freq_index=%d, padding_bit=%d, private_bit=%d\n"
	   "   mode=%d, mode_extension=%d, copyright=%d, original=%d\n"
	   "   emphasis=%d, version_index=%d, bytes=%d.\n",
	   mp3_info->id, mp3_info->layer, mp3_info->protection_bit,
	   bitrate_index, sampling_freq_index, mp3_info->padding_bit,
	   mp3_info->private_bit, mp3_info->mode,
	   mp3_info->mode_extension, mp3_info->copyright,
	   mp3_info->original, mp3_info->emphasis,
	   mp3_info->version_index, header);
#endif
  if (bitrate_index == 0)
    {
      //g_warning("%s: error parsing bitrate from mp3 header",
      //filename);
      return FALSE;
    }
  frequency_index = 3 * mp3_info->id + sampling_freq_index;
  mp3_info->bitrate
    = bitrate_table[mp3_info->id][3 - mp3_info->layer][bitrate_index];
  mp3_info->sampling_frequency
    = sampling_frequency_table[mp3_info->id][sampling_freq_index];

  if (mp3_info->sampling_frequency == 0)
    {
      //g_warning("%s: sampling_frequency was 0", filename);
      return FALSE;
    }
  if ((header & 0xffe00000) != 0xffe00000)
    {
      //g_warning("%s: header didn't contain mp3 identifier",
      // filename);
      return FALSE;
    }
  if (mp3_info->version_index == 1)
    {
      //g_warning("%s: version_index was 1", filename);
      return FALSE;
    }
  if (frequency_index >= 0)
    {
      int id_index = frequency_index / 3;
      if (id_index < 2)
	{
	  mp3_info->frequency = sampling_frequency_table
	    [id_index][frequency_index % 3];
	}
      else
	{
	  //g_warning("%s: id_index was too big "
	  //"computing frequency", filename);
	  return FALSE;
	}
    }
  return TRUE;
}

GdamMp3Info*   gdam_mp3_info_new(const char* filename)
{
  FILE *fp = fopen (filename, "r");
  char buf[4096];
  int num_read;
  int offset = 0;
  char tail_buf[128];
  gint64 file_size;
  GdamMp3Info *rv;
  gboolean v1 = FALSE, v2 = FALSE;
  if (fp == NULL)
    {
      g_warning ("error opening %s to determine how long it is", filename);
      return NULL;
    }
  num_read = fread (buf, 1, sizeof (buf), fp);

  rv = g_new0 (GdamMp3Info, 1);
  if (offset == 0 && get_v2_head (fp, rv, num_read, buf))
    {
      offset = rv->tag_size;
      v2 = TRUE;
    }


  /* get_mp3_header is `_get_head' with a return value of `_is_mp3' */
  while (offset < num_read - 4
	 && !get_mp3_header (filename, rv, buf + offset))
    {
      offset++;
    }

  if (offset >= num_read - 4)
    {
      fclose (fp);
      g_free (rv);
      return NULL;
    }

  fseek (fp, 0, SEEK_END);
  file_size = ftell (fp);

  /* From mp3_info:  parse ID3v1 tag (which come at the end of file) */
  fseek (fp, -128, SEEK_END);
  fread (tail_buf, 128, 1, fp);
  if (memcmp (tail_buf, "TAG", 3) == 0)
    {
      //g_message("had id3 tag");
      offset += 128;
    }
  fclose (fp);

  if (v2 == FALSE
      && tail_buf[0] == 'T' && tail_buf[1] == 'A' && tail_buf[2] == 'G')
    {
      v1 = TRUE;
      gdam_debug ("parsing ID3v1 style tags");
      if (tail_buf[126] == '\0' && tail_buf[127] != '\0')
	{
	  const char *genre;
	  add_hash_val (rv, "TITLE", tail_buf + 3, 30);
	  add_hash_val (rv, "ARTIST", tail_buf + 33, 30);
	  add_hash_val (rv, "ALBUM", tail_buf + 63, 30);
	  add_hash_val (rv, "YEAR", tail_buf + 93, 4);
	  add_hash_val (rv, "COMMENT", tail_buf + 97, 28);
	  add_hash_num (rv, "TRACKNUM", (guint8) (tail_buf[126]));
	  genre = find_genre ((guint8) tail_buf[127]);
	  add_hash_str (rv, "GENRE", genre);
	}
      else
	{
	  const char *genre;
	  add_hash_val (rv, "TITLE", tail_buf + 3, 30);
	  add_hash_val (rv, "ARTIST", tail_buf + 33, 30);
	  add_hash_val (rv, "ALBUM", tail_buf + 63, 30);
	  add_hash_val (rv, "YEAR", tail_buf + 93, 4);
	  add_hash_val (rv, "COMMENT", tail_buf + 97, 30);
	  genre = find_genre ((guint8) tail_buf[127]);
	  add_hash_str (rv, "GENRE", genre);
	}
    }

  /* get frame size */
  {
    int mean_frame_size;
    int bitrate = rv->bitrate;
    int fs = rv->sampling_frequency;
    if (rv->id)
      {
	mean_frame_size = 144000 * bitrate / fs;
      }
    else
      {
	mean_frame_size = 72000 * bitrate / fs;
      }
    rv->mean_frame_size = mean_frame_size;
  }

  /* get the length in seconds. */
  rv->length = (gdouble) (file_size - offset)
    / rv->mean_frame_size
    * ((115200.0 / 2.0) * (1.0 + rv->id)) / rv->sampling_frequency / 100.0;

  /* Is this correct? 
   * I'm almost sure it's what the perl code does... */
  rv->stereo = (rv->mode == 3 ? 0 : 1);
  if (rv->mode > 0)
    rv->layer = (rv->mode == 3 ? 2 : 3);
  else
    rv->layer = 0;

  return rv;
}

void gdam_mp3_info_destroy(GdamMp3Info* mp3_info)
{
  if (mp3_info->extra_values != NULL)
    {
      g_hash_table_destroy (mp3_info->extra_values);
      mp3_info->extra_values = NULL;
    }
  g_free (mp3_info);
}

