#include <gdam/common/gdammp3info.h>
#include <stdio.h>

void usage(const char* progname)
{
  fprintf (stderr, "usage: %s files...\n\nfind mp3info.\n", progname);
  exit (1);
}

static void print_extra_value(const char* key, const char* value, FILE* fp)
{
  fprintf (fp, "   %s => \"%s\"\n", key, value);
}

gboolean gdam_debugging_on = FALSE;
void gdam_debug(const char* str, ...)
{
  va_list args;

  if (gdam_debugging_on)
    {
      va_start (args, str);
      vfprintf (stderr, str, args);
      fprintf (stderr, ".\n");
      va_end (args);
    }
}


int main(int argc, char** argv)
{
  int i;
  gdam_debugging_on = TRUE;
  if (argc == 1)
    usage (argv[0]);
  for (i = 1; i < argc; i++)
    {
      GdamMp3Info *info;
      info = gdam_mp3_info_new (argv[i]);
      if (info == NULL)
	{
	  g_warning ("no info for %s", argv[i]);
	  continue;
	}
      fprintf (stderr,
	       "%s: %4.4gsec, %dbps, %d samples/sec.\n",
	       argv[i],
	       info->length, info->bitrate, info->sampling_frequency);
      if (info->extra_values != NULL)
	g_hash_table_foreach (info->extra_values,
			      (GHFunc) print_extra_value, stderr);
    }
  return 0;
}
