/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include <gmodule.h>
#include <gtk/gtkobject.h>
#include <gdam/common/gdamdebug.h>
#include "gdamplugin.h"
#include "gdammodule.h"
#include <ctype.h>

static char* get_new_type_function_name(const char* name)
{
  const char *at = name;
  int count = 0;
  char *rv;
  while (*at)
    {
      if ('a' <= *at && *at <= 'z')
	{
	  count++;
	  at++;
	}
      else if ('A' <= *at && *at <= 'Z')
	{
	  count += 2;
	  at++;
	}
      else if ('0' <= *at && *at <= '9')
	{
	  count++;
	  at++;
	}
      else
	{
	  g_warning ("type `%s' contained unallowed character", name);
	  return NULL;
	}
    }
  rv = g_new (char, count + 1 + strlen ("_get_type"));

  at = name;
  count = 0;
  while (*at)
    {
      if ('a' <= *at && *at <= 'z')
	{
	  rv[count++] = *at;
	}
      else if ('A' <= *at && *at <= 'Z')
	{
	  if (at > name)
	    rv[count++] = '_';
	  rv[count++] = tolower (*at);
	}
      else if ('0' <= *at && *at <= '9')
	{
	  rv[count++] = *at;
	}
      else
	{
	  g_warning ("type `%s' contained unallowed character "
		     "`%c'", name, *at);
	  return NULL;
	}
      at++;
    }
  strcpy (rv + count, "_get_type");
  return rv;
}

GtkType gdam_plugin_get_type_by_name(const char* family,
                                     const char* type)
{
  GtkType t;
  t = gtk_type_from_name (type);
  if (!t)
    {
      GModule *module = gdam_module_self ();
      GtkType (*type_getter) ();
      char *fct_name = get_new_type_function_name (type);
      /* XXX: is fct_name a mem leak? */
      if (g_module_symbol (module, fct_name,
			   (gpointer *) & type_getter) && type_getter != 0)
	{
	  t = (*type_getter) ();
	  g_free (fct_name);
	  return t;
	}

      module = gdam_module_by_name (family, type);
      if (!module)
	{
	  g_warning ("no module or builtin for %s", type);
	  return 0;
	}
      if (g_module_symbol (module,
			   fct_name,
			   (gpointer *) & type_getter) && type_getter != 0)
	{
	  t = (*type_getter) ();
	  g_free (fct_name);
	  return t;
	}
      else
	{
	  g_warning ("module for %s missing func '%s'", type, fct_name);
	  g_module_close (module);
	}

    }
  return t;
}

void gdam_plugin_check_version(const char* filename, 
                               const char* version_string)
{
  if (strcmp (version_string, GDAM_VERSION) != 0)
    {
      g_warning ("the module %s is compiled for version %s; "
		 "but %s is compiled for %s",
		 filename, version_string, g_get_prgname (), GDAM_VERSION);
    }
}
