#include "gdamscandir.h"
#include <sys/types.h>
#include <dirent.h>
#include <string.h>

GSList* gdam_scandir                (const char*           path,
                                     GdamScandirCheckFunc  is_valid,
		                     gpointer              user_data)
{
  DIR *dir;
  struct dirent *entry;
  GSList *list = NULL;
  dir = opendir (path);
  if (dir == NULL)
    return NULL;
  while ((entry = readdir (dir)) != NULL)
    {
      if ((*is_valid) (path, entry->d_name, user_data))
	{
	  char *fullpath;
	  fullpath = g_strjoin (G_DIR_SEPARATOR_S, path, entry->d_name, NULL);
	  list = g_slist_prepend (list, fullpath);
	}
    }
  closedir (dir);
  return g_slist_reverse (list);
}

static gboolean check_extension(const char* dir,
                                const char* path,
				gpointer    extension_ptr)
{
  const char *dot;
  dot = strrchr (path, '.');
  if (dot == NULL)
    return FALSE;
  return strcmp (dot + 1, (char *) extension_ptr) == 0;
}

GSList* gdam_scandir_for_extension  (const char*           path,
                                     const char*           extension)
{
  return gdam_scandir (path, check_extension, (gpointer) extension);
}
	
GSList* gdam_scandir_for_shared_libs(const char*           path)
{
  return gdam_scandir_for_extension (path, "so");
}
