#include "gdamunixsignal.h"
#include <string.h>
#include <signal.h>

#define MAX_SIGNAL	64

static guint8 signal_markers[MAX_SIGNAL] = { 0 };
static guint8 signals_handled[MAX_SIGNAL] = { 0 };

typedef struct _SourceData SourceData;
struct _SourceData
{
  guint sig_no;
  GdamUnixSignalFunc func;
};

static gboolean 
gdam_unix_signal_prepare  (gpointer  source_data, 
		           GTimeVal *current_time,
		           gint     *timeout,
		           gpointer  user_data)
{
  SourceData *source = source_data;
  guint sig_no = source->sig_no;
  if (signal_markers[sig_no])
    return TRUE;
  return FALSE;
}

static gboolean 
gdam_unix_signal_check    (gpointer  source_data,
		           GTimeVal *current_time,
		           gpointer  user_data)
{
  SourceData *source = source_data;
  guint sig_no = source->sig_no;
  if (signal_markers[sig_no])
    return TRUE;
  return FALSE;
}

static gboolean 
gdam_unix_signal_dispatch (gpointer  source_data, 
		           GTimeVal *dispatch_time,
		           gpointer  user_data)
{
  SourceData *source = source_data;
  guint sig_no = source->sig_no;
  signal_markers[sig_no] = 0;
  (*source->func) (sig_no, user_data);
  return TRUE;
}

static GSourceFuncs
gdam_unix_signal_source_funcs =
{
  gdam_unix_signal_prepare,
  gdam_unix_signal_check,
  gdam_unix_signal_dispatch,
  g_free
};

/* this is the asynchronous signal handling function */
static void
handle_raw_signal (int id)
{
  signal_markers[id] = 1;
}

guint
gdam_unix_signal_connect (guint                sig_no,
			  GdamUnixSignalFunc   func,
			  gpointer             data,
			  GDestroyNotify       destroy)
{
  SourceData *source_data;
  g_return_val_if_fail (sig_no >= MAX_SIGNAL, 0);
  if (!signals_handled[sig_no])
    {
      struct sigaction sigaction_info;
      bzero (&sigaction_info, sizeof (sigaction_info));
      sigaction_info.sa_handler = handle_raw_signal;
      sigaction (sig_no, &sigaction_info, NULL);
      signals_handled[sig_no] = 1;
    }
  source_data = g_new (SourceData, 1);
  source_data->func = func;
  source_data->sig_no = sig_no;
  return g_source_add (G_PRIORITY_DEFAULT,
		       FALSE,
		       &gdam_unix_signal_source_funcs,
		       source_data,
		       data,
		       destroy);
}
