#include "gdammathfunction.h"

static void
test_function(const char* string,
              int         num_test_vals,
              ...)
{
  gdouble input;
  gdouble output;
  gdouble desired_output;
  GdamMathFunction *function;
  va_list args;

  va_start (args, num_test_vals);

  function = gdam_math_function_parse (string, NULL);
  if (function == NULL)
    g_error ("FAILED TEST: couldn't parse %s", string);

  while (num_test_vals-- > 0)
    {
      gdouble distance;
      input = va_arg (args, gdouble);
      desired_output = va_arg (args, gdouble);
      output = gdam_math_function_eval (function, &input);
      distance = output - desired_output;
      if (distance < 0.0)
	distance = (-distance);
      if (distance > 0.0001)
	g_error ("MISMATCH: f(%.8g) = %.8g (should be %.8g) "
		 "[f = %s]", input, output, desired_output, string);
    }
  gdam_math_function_destroy (function);
  va_end (args);
}

int main(int argc, char** argv)
{
  GdamMathFunction *function;
  char *string;
  double input;

  gdam_math_function_init ();

  test_function ("x*x", 2,	/* number of subtests */
		 2.0, 4.0, -1.0, 1.0);
  test_function ("x+x", 2,	/* number of subtests */
		 2.0, 4.0, -1.0, -2.0);
  test_function ("2+x*x", 2,	/* number of subtests */
		 4.0, 18.0, -1.0, 3.0);
  test_function ("2*x+x", 2,	/* number of subtests */
		 4.0, 12.0, -1.0, -3.0);
  test_function ("2/x+x", 4,	/* number of subtests */
		 1.0, 3.0, -1.0, -3.0, -2.0, -3.0, -4.0, -4.5);
  test_function ("2/x*x", 4,	/* number of subtests */
		 1.0, 2.0, -1.0, 2.0, -2.0, 2.0, -4.0, 2.0);
  test_function ("2/x+3", 4,	/* number of subtests */
		 1.0, 5.0, -1.0, 1.0, -2.0, 2.0, 4.0, 3.5);
  test_function ("2/x-3", 4,	/* number of subtests */
		 1.0, -1.0, -1.0, -5.0, -2.0, -4.0, 4.0, -2.5);

  /* test variables */
  {
    char *names[2] = { "hello", "world" };
    gdouble input[2];
    GdamMathParseContext context = { TRUE, 2, names };
    GdamMathFunction *function = gdam_math_function_parse ("hello*hello + world", &context);
    gdouble desired_output;
    gdouble output;
    gdouble distance;

    input[0] = 2; input[1] = 3;
    desired_output = 7;
    output = gdam_math_function_eval (function, input);
    distance = output - desired_output;
    if (distance < 0.0)
      distance = (-distance);
    if (distance > 0.0001)
      g_error ("MISMATCH: f(%.8g,%.8g) = %.8g (should be %.8g) "
		 , input[0],input[1], output, desired_output);
    gdam_math_function_destroy (function);
  }

  return 0;
}


