/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdampitch.h"

gdouble gdam_pitch_get_magic_rate (gdouble rate)
{
  /* ratios for intervals in extended just tuning system. */
  gdouble ratios[12] = {1.0416666, 1.0546875, 1.066666, 1.111111,
                        1.125,     1.2,       1.25,     1.333333,
			1.5,       1.6666666, 1.875,    2.0};
  gdouble best_ratio = 1.0;
  int sign_factor = 1;
  int i;
  int num_ratios = 12;

  if (rate < 0)
    {
      sign_factor = -1;
      rate *= sign_factor;
    }

  if (rate > 1.0)
    for (i = 0; ratios[i] < rate && i < num_ratios; i++)
      best_ratio = ratios[i];
  else
    for (i = 0; (1.0 / ratios[i]) > rate && i < num_ratios; i++)
      best_ratio = 1.0 / ratios[i];

  g_message ("converted rate %f to magic ratio %f", rate * sign_factor, 
             best_ratio * sign_factor);
  return best_ratio * sign_factor;
}

