/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamrequestbuilder.h"

GdamRequestBuilder* gdam_request_builder_new    ()
{
  return g_new0 (GdamRequestBuilder, 1);
}

int gdam_request_builder_append_create (GdamRequestBuilder* builder,
                                        GdamRequest*        request)
{
  gdam_request_builder_append_generic (builder, request);
  g_return_val_if_fail (request->local_id < 0, 0);
  return request->local_id;
}

void gdam_request_builder_append_generic(GdamRequestBuilder* builder,
                                         GdamRequest*        request)
{
  if (builder->first_request == NULL)
    {
      builder->first_request = builder->last_request
	= g_slist_append (NULL, request);
    }
  else
    {
      g_slist_append (builder->last_request, request);
      builder->last_request = builder->last_request->next;
    }
}

GdamRequest* gdam_request_builder_package(GdamRequestBuilder* builder)
{
  int num_sub = g_slist_length (builder->first_request);
  GdamRequest **requests;
  GSList *at;
  int i = 0;

  if (num_sub == 0)
    return NULL;

  requests = g_new (GdamRequest *, num_sub);
  for (at = builder->first_request; at; at = at->next)
    requests[i++] = (GdamRequest *) (at->data);

  g_slist_free (builder->first_request);
  builder->first_request = builder->last_request = NULL;

  return gdam_request_package_v (num_sub, requests, NULL, NULL, NULL);
}

void gdam_request_builder_destroy(GdamRequestBuilder* builder)
{
  g_return_if_fail (builder != NULL);
  g_slist_foreach (builder->first_request,
		   (GFunc) gdam_request_destroy, NULL);
  g_slist_free (builder->first_request);
  builder->first_request = builder->last_request = NULL;
  g_free (builder);
}
