/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include <sys/stat.h>
#include <math.h>
#include "gdamutil.h"

gboolean gdam_file_exists(const char* filename)
{
  struct stat sbuf;
  if (stat (filename, &sbuf) < 0)
    return 0;
  if (S_ISREG (sbuf.st_mode))
    return 1;
  return 0;
}

typedef void (*GtkSignal_NONE__DOUBLE) (GtkObject * object,
					gdouble  arg1,
					gpointer user_data);

void
gtk_marshal_NONE__DOUBLE (GtkObject * object,
                           GtkSignalFunc func,
                           gpointer func_data, GtkArg * args)
{
  GtkSignal_NONE__DOUBLE rfunc;
  rfunc = (GtkSignal_NONE__DOUBLE) func;
  (*rfunc) (object, GTK_VALUE_DOUBLE (args[0]), func_data);
}

typedef void (*GtkSignal_NONE__DOUBLE_DOUBLE) (GtkObject * object,
					       gdouble  arg1,
					       gdouble  arg2,
					       gpointer user_data);

void
gtk_marshal_NONE__DOUBLE_DOUBLE (GtkObject * object,
                           GtkSignalFunc func,
                           gpointer func_data, GtkArg * args)
{
  GtkSignal_NONE__DOUBLE_DOUBLE rfunc;
  rfunc = (GtkSignal_NONE__DOUBLE_DOUBLE) func;
  (*rfunc) (object, GTK_VALUE_DOUBLE (args[0]),
	    GTK_VALUE_DOUBLE (args[1]), func_data);
}

typedef gdouble (*GtkSignal_DOUBLE__NONE) (GtkObject * object,
					   gpointer user_data);

void
gtk_marshal_DOUBLE__NONE (GtkObject * object,
                           GtkSignalFunc func,
                           gpointer func_data, GtkArg * args)
{
  GtkSignal_DOUBLE__NONE rfunc;
  gdouble *return_val;
  return_val = GTK_RETLOC_DOUBLE (args[0]);
  rfunc = (GtkSignal_DOUBLE__NONE) func;
  *return_val = (*rfunc) (object, func_data);
}


typedef void (*GtkSignal_NONE__DOUBLE_POINTER) 
					   (GtkObject * object,
					    gdouble  arg1,
					    gpointer  arg2,
					    gpointer user_data);

typedef void (*GtkSignal_NONE__DOUBLE_INT) (GtkObject * object,
					    gdouble  arg1,
					    gint  arg2,
					    gpointer user_data);

void
gtk_marshal_NONE__DOUBLE_POINTER 
			     (GtkObject * object,
                              GtkSignalFunc func,
                              gpointer func_data,
                              GtkArg * args)
{
  GtkSignal_NONE__DOUBLE_POINTER rfunc;
  rfunc = (GtkSignal_NONE__DOUBLE_POINTER) func;
  (*rfunc) (object,
	    GTK_VALUE_DOUBLE (args[0]),
	    GTK_VALUE_POINTER (args[1]), func_data);
}


void
gtk_marshal_NONE__DOUBLE_INT (GtkObject * object,
                              GtkSignalFunc func,
                              gpointer func_data,
                              GtkArg * args)
{
  GtkSignal_NONE__DOUBLE_INT rfunc;
  rfunc = (GtkSignal_NONE__DOUBLE_INT) func;
  (*rfunc) (object,
	    GTK_VALUE_DOUBLE (args[0]), GTK_VALUE_INT (args[1]), func_data);
}

void gdam_do_nothing()
{
}

gdouble
gdam_hz_from_midi_note_idx (int midi_index)
{
    return 440.0 * pow (2.0, (gdouble) (midi_index - 57.0) / 12.0);
}


/*   gdam_monitor_object_destroy (GtkObject **obj_ptr)
 *
 * Watch a member object for destruction, NULL the pointer
 * and unref.  Object should have been reffed when obj_ptr
 * was assigned, and should be unreffed whenever the pointer
 * is NULLed.   Assumes the host struct will be valid until
 * after the object has been destroyed.
 *
 * EXAMPLE:
 *
 *   GtkWidget *some_widget;
 *
 *   gtk_object_ref (some_widget);
 *   my_struct->some_widget = some_widget; 
 *   gdam_monitor_object_destroy (&my_struct->some_widget);
 *
 *     ...
 *
 *   if (my_struct->some_widget) {  do something with some_widget }
 *
 *     ...
 *
 *   if (my_struct->some_widget) {
 *     gtk_object_unref (my_struct->some_widget);
 *     my_struct->some_widget = NULL;
 *   }
 */

static void 
monitored_object_destroyed (GtkObject **obj_ptr)
{
  g_return_if_fail (*obj_ptr != NULL);
  gtk_object_unref (GTK_OBJECT (*obj_ptr));
  *obj_ptr = NULL;
}

void 
gdam_monitor_object_destroy (GtkObject **obj_ptr)
{
  g_return_if_fail (obj_ptr != NULL);
  g_return_if_fail (*obj_ptr != NULL);
  gtk_signal_connect_object (GTK_OBJECT (*obj_ptr),
			     "destroy",
			     GTK_SIGNAL_FUNC (monitored_object_destroyed),
			     (GtkObject *) obj_ptr);
}


