/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamdistortion.h"
#include <gdam/common/gdamsample.h>
#include <math.h>

static void gdam_distortion_init(GdamDistortion* distortion)
{
  distortion->saturation = 0.1;
}

static inline void maybe_dispense_request(GdamDistortion* distortion)
{
  GdamArg arg;

  arg.name = "saturation";
  arg.type = GDAM_ARG_TYPE_DOUBLE;
  GDAM_ARG_DOUBLE (&arg) = distortion->saturation;

  gdam_model_set_args (GDAM_MODEL (distortion), 0, 1, &arg);
}

static int gdam_distortion_append_start_request(GdamFilter* filter,
				                int new_source_id,
				                GdamRequestBuilder* builder)
{
  GdamDistortion *distortion = GDAM_DISTORTION (filter);
  GdamModel *model = GDAM_MODEL (filter);
  GdamRequest *request;
  gtk_object_ref (GTK_OBJECT (distortion));
  model->source_id = gdam_channel_new_local_id (model->channel);
  request = gdam_request_create_source ("GdasDistortion",
					model->source_id,
					gdam_model_got_subsource,
					gdam_model_didnt_get_subsource,
					distortion,
					2,
					"saturation",
					GDAM_ARG_TYPE_DOUBLE,
					distortion->saturation,
					"filtered",
					GDAM_ARG_TYPE_INT, new_source_id);
  return gdam_request_builder_append_create (builder, request);
}

static void gdam_distortion_arg_set_saturation(GtkObject* object, GdamArg* arg)
{
  GdamDistortion *distortion;
  distortion = GDAM_DISTORTION (object);
  distortion->saturation = GDAM_ARG_DOUBLE (arg);
  maybe_dispense_request (distortion);
}
static void gdam_distortion_arg_get_saturation(GtkObject* object, GdamArg* arg)
{
  GdamDistortion *distortion;
  distortion = GDAM_DISTORTION (object);
  GDAM_ARG_DOUBLE (arg) = (gdouble) distortion->saturation;
}

static void gdam_distortion_class_init(GdamModelClass* model_class)
{
  GdamFilterClass *filter_class;
  GtkObjectClass *object_class;
  filter_class = GDAM_FILTER_CLASS (model_class);
  object_class = GTK_OBJECT_CLASS (model_class);
  model_class->num_arg_sets = 1;
  filter_class->append_start_request = gdam_distortion_append_start_request;
  gdam_arg_register (object_class,
		     "saturation",
		     GDAM_ARG_TYPE_DOUBLE,
		     gdam_distortion_arg_set_saturation,
		     gdam_distortion_arg_get_saturation,
		     NULL, "0,100", GDAM_ARG_VITAL);
}

GtkType gdam_distortion_get_type()
{
  static GtkType distortion_type = 0;
  if (!distortion_type)
    {
      static const GtkTypeInfo distortion_info = {
	"GdamDistortion",
	sizeof (GdamDistortion),
	sizeof (GdamDistortionClass),
	(GtkClassInitFunc) gdam_distortion_class_init,
	(GtkObjectInitFunc) gdam_distortion_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_FILTER;
      distortion_type = gtk_type_unique (parent, &distortion_info);
    }
  return distortion_type;
}
