/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __COMPONENT_FILTER_EQUALIZER_H_
#define __COMPONENT_FILTER_EQUALIZER_H_

#include <gdam/client/gdamchannel.h>
#include "../gdamfilter.h"
#include "../gdaminfomanager.h"

typedef struct _GdamEqualizer GdamEqualizer;
typedef struct _GdamEqualizerClass GdamEqualizerClass;

GtkType gdam_equalizer_get_type();
#define GDAM_TYPE_EQUALIZER			(gdam_equalizer_get_type ())
#define GDAM_EQUALIZER(obj)			(GTK_CHECK_CAST ((obj), GDAM_TYPE_EQUALIZER, GdamEqualizer))
#define GDAM_EQUALIZER_CLASS(klass)		(GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_EQUALIZER, GdamEqualizerClass))
#define GDAM_IS_EQUALIZER(obj)		(GTK_CHECK_TYPE ((obj), GDAM_TYPE_EQUALIZER))
#define GDAM_IS_EQUALIZER_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_EQUALIZER))

struct _GdamEqualizerClass {
	GdamFilterClass	filter_class;
	void		(*update_frequency)    (GdamEqualizer*	equalizer,
						gdouble		frequency);
	void            (*update_mode)         (GdamEqualizer*  equalizer,
	                                        const char* 	mode);
	void            (*update_db_gain)       (GdamEqualizer*  equalizer,
	                                        gdouble 	gain);
	void            (*update_bandwidth)    (GdamEqualizer*  equalizer,
	                                        gdouble 	bandwidth);
	void            (*update_effect)       (GdamEqualizer*  equalizer,
	                                        gdouble 	effect);
};

struct _GdamEqualizer {
	GdamFilter		filter;
	GtkWidget*		equalizer_frame;
	guint 			equalizer_id;
	double			frequency;
	double			bandwidth;
	double			effect;
	double			db_gain;
	const char*		mode;
};

GdamEqualizer*	gdam_equalizer_new		();
void		gdam_equalizer_free		(GdamEqualizer*	equalizer);
void		gdam_equalizer_set_frequency    (GdamEqualizer*	equalizer,
					 	 double 	frequency);
void		gdam_equalizer_set_mode		(GdamEqualizer*	equalizer,
					 	 const char* 	mode);
void		gdam_equalizer_set_bandwidth    (GdamEqualizer*	equalizer,
					 	 double 	frequency);
void		gdam_equalizer_set_db_gain      (GdamEqualizer*	equalizer,
					 	 double 	frequency);
void		gdam_equalizer_set_effect       (GdamEqualizer*	equalizer,
					 	 double 	effect);

#endif
