/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamfestival.h"

static int gdam_festival_play_request     (GdamProducer*	producer,
					   GdamPlayInfo*	play_info,
					   int                  subsource_id,
					   GdamRequestBuilder*  builder)
{
  GdamRequest *request;
  GdamArg args[1];
  GdamFestival *festival;
  GdamModel *model = GDAM_MODEL (producer);
  festival = GDAM_FESTIVAL (producer);
  g_return_val_if_fail (festival->text != NULL, 0);
  args[0].name = "text";
  args[0].type = GDAM_ARG_TYPE_STRING;
  GDAM_ARG_STRING (args + 0) = festival->text;
  gtk_object_ref (GTK_OBJECT (festival));
  model->source_id = gdam_channel_new_local_id (model->channel);
  request = gdam_request_create_source_v ("GdasFestival",
					  model->source_id,
					  1,
					  args,
					  gdam_model_got_subsource,
					  gdam_model_didnt_get_subsource,
					  festival);
  return gdam_request_builder_append_create (builder, request);
}

static void gdam_festival_arg_set_text(GtkObject* object, GdamArg* arg)
{
  GdamFestival *festival;
  festival = GDAM_FESTIVAL (object);
  g_free (festival->text);
  festival->text = g_strdup (GDAM_ARG_STRING (arg));
}
static void gdam_festival_arg_get_text(GtkObject* object, GdamArg* arg)
{
  GdamFestival *festival;
  festival = GDAM_FESTIVAL (object);
  GDAM_ARG_STRING (arg) = g_strdup (festival->text);
}

static void gdam_festival_init(GdamFestival* festival)
{
}
static void gdam_festival_class_init(GdamFestivalClass* festival_class)
{
  GdamProducerClass *producer_class;
  GtkObjectClass *object_class;
  producer_class = GDAM_PRODUCER_CLASS (festival_class);
  object_class = GTK_OBJECT_CLASS (festival_class);
  producer_class->play_request = gdam_festival_play_request;
  gdam_arg_register (object_class,
		     "text",
		     GDAM_ARG_TYPE_STRING,
		     gdam_festival_arg_set_text,
		     gdam_festival_arg_get_text, NULL, NULL, GDAM_ARG_VITAL);
}

GtkType gdam_festival_get_type()
{
  static GtkType festival_type = 0;
  if (!festival_type)
    {
      static const GtkTypeInfo festival_info = {
	"GdamFestival",
	sizeof (GdamFestival),
	sizeof (GdamFestivalClass),
	(GtkClassInitFunc) gdam_festival_class_init,
	(GtkObjectInitFunc) gdam_festival_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_PRODUCER;
      festival_type = gtk_type_unique (parent, &festival_info);
    }
  return festival_type;
}
