/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_FILTER_WAVETABLE_H_
#define __GDAM_FILTER_WAVETABLE_H_


typedef struct _GdamFilterWavetable GdamFilterWavetable;
typedef struct _GdamFilterWavetableClass GdamFilterWavetableClass;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "../gdamfilter.h"

GtkType gdam_filter_wavetable_get_type();
#define GDAM_TYPE_FILTER_WAVETABLE			(gdam_filter_wavetable_get_type ())
#define GDAM_FILTER_WAVETABLE(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_FILTER_WAVETABLE, GdamFilterWavetable))
#define GDAM_FILTER_WAVETABLE_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_FILTER_WAVETABLE, GdamFilterWavetableClass))
#define GDAM_IS_FILTER_WAVETABLE(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_FILTER_WAVETABLE))
#define GDAM_IS_FILTER_WAVETABLE_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_FILTER_WAVETABLE))

struct _GdamFilterWavetableClass {
	GdamFilterClass		filter_class;
	void                  (*handle_new_modifier)(GdamFilterWavetable*,
	                                             int wavetable_local_id,
						     GdamRequestBuilder*);
};
struct _GdamFilterWavetable {
	GdamFilter		filter;

	guint			buffer_id;
	gboolean                must_dealloc_buffer_id;

	gdouble			hertz;
	int			note_octave;  

	gint			wavetable_source_id;
};

gint gdam_filter_wavetable_make_wavetable_request(GdamFilterWavetable*,
                                                  GdamRequestBuilder*);

void gdam_filter_wavetable_set_hertz(GdamFilterWavetable*, gdouble hertz);
void gdam_filter_wavetable_set_buffer_id     (GdamFilterWavetable*,
                                              int buffer_id,
					      gboolean must_dealloc);
void gdam_filter_wavetable_set_default_buffer(GdamFilterWavetable*);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
