/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamgdamdsp.h"

GSList* first_accepted = NULL;
GSList* last_accepted = NULL;

void   gdam_gdam_dsp_enqueue_source_id(int         source_id)
{
  if (first_accepted != NULL)
    {
      GdamModel *gdam_dsp = first_accepted->data;
      first_accepted = g_slist_remove (first_accepted, gdam_dsp);
      gdam_model_set_source_id (GDAM_MODEL (gdam_dsp), source_id);
      if (first_accepted == NULL)
	last_accepted = NULL;
    }
  else
    {
      g_warning ("gdam_gdam_dsp_enqueue_source_id called before "
		 "GdamGdamDsp was created");
    }
}

static void gdam_gdam_dsp_init(GdamGdamDsp* gdam_dsp)
{
  if (first_accepted == NULL)
    {
      first_accepted = last_accepted = g_slist_prepend (NULL, gdam_dsp);
    }
  else
    {
      last_accepted = g_slist_append (last_accepted, gdam_dsp);
      last_accepted = last_accepted->next;
    }
}

static void gdam_gdam_dsp_class_init(GdamGdamDspClass* gdam_dsp_class)
{
}

GtkType gdam_gdam_dsp_get_type()
{
  static GtkType gdam_dsp_type = 0;
  if (!gdam_dsp_type)
    {
      static const GtkTypeInfo gdam_dsp_info = {
	"GdamGdamDsp",
	sizeof (GdamGdamDsp),
	sizeof (GdamGdamDspClass),
	(GtkClassInitFunc) gdam_gdam_dsp_class_init,
	(GtkObjectInitFunc) gdam_gdam_dsp_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_MODEL;
      gdam_dsp_type = gtk_type_unique (parent, &gdam_dsp_info);
    }
  return gdam_dsp_type;
}
