/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_GONG_H_
#define __GDAM_GONG_H_


typedef struct _GdamGong GdamGong;
typedef struct _GdamGongClass GdamGongClass;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "../gdamproducer.h"

GtkType gdam_gong_get_type();
#define GDAM_TYPE_GONG			(gdam_gong_get_type ())
#define GDAM_GONG(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_GONG, GdamGong))
#define GDAM_GONG_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_GONG, GdamGongClass))
#define GDAM_IS_GONG(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_GONG))
#define GDAM_IS_GONG_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_GONG))

typedef enum _GdamGongType GdamGongType;
enum _GdamGongType {
	GDAM_GONG_FREE_OSCILLATION,
	GDAM_GONG_CLAMPED_CIRCULAR_PLATE,
	GDAM_GONG_NUM_STYLES
};

typedef struct _GdamGongStyle GdamGongStyle;
struct _GdamGongStyle {
	char*			description;
	int			num_coefficients;
	const gdouble*		coefficients;

	gdouble			overall_volume;
	gdouble			halflife_scale;
	gdouble			hertz;

	GdamGong*		gong;
};

struct _GdamGongClass {
	GdamProducerClass	producer_class;
};
struct _GdamGong {
	GdamProducer		producer;
	int			last_style;
	GSList*			style_list;
};

GdamGongStyle* gdam_gong_add     (GdamGong*        gong,
                                  GdamGongType     gong_type);

void           gdam_gong_strike  (GdamGong*        gong,
                                  GdamGongStyle*   style);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
