/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamiokithid.h"

#if HAVE_IOKIT_HID_IOHIDLIB_H

#include <gdam/common/gdamplugin.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <mach/mach.h>
#include <mach/mach_error.h>
#include <IOKit/IOKitLib.h>
#include <IOKit/IOCFPlugIn.h>
#include <IOKit/hid/IOHIDKeys.h>
#include <CoreFoundation/CoreFoundation.h>
#include <Carbon/Carbon.h>

typedef struct IOHIDEventStruct usb_input_event;

static void gdam_usbevent_init(GdamIokitHid* usbevent)
{
}

#if 0
/* XXX: figure out how not to leak memory here */
static char *
translate_usb_input_range_to_name (GdamIokitHidEventRange *evt)
{
  //if (evt->type  == EV_ABS)
  //  return g_strdup_printf ("c:%d", evt->code);
  return g_strdup_printf ("t:%d c:%d v:%d (unknown range_type %x)",
                          evt->type, evt->code, evt->value, evt->type);
}
#endif

/* XXX: rename variable to relative */

#if 0
/* XXX: figure out how not to leak memory here */
static char *
translate_usb_input_to_name (GdamIokitHidEvent *evt)
{
  //if (evt->type  == EV_RST)
  //  return g_strdup_printf ("c:%d v:%d rst", evt->code, evt->value);
  return g_strdup_printf ("t:%d c:%d v:%d (unknown type %x)", 
                          evt->type, evt->code, evt->value, evt->type);
}
#endif

#if 0
/* Event Ranges are a different type, and one EventRange is defined
 * for each code,type pair, and covers all values */
static void 
gdam_usbevent_parse_event_range (GdamIokitHid* hid, usb_input_event* evt)
{
  GdamIokitHidEventRange *event;
  event = g_new0 (GdamIokitHidEventRange, 1);
  event->event.input_device = (GdamInputDevice *) (hid);
  event->event.event_type = gdam_iokithid_event_absolute_get_type ();
  //event->code = evt->code;
  event->type = evt->type;
  event->value = evt->value;
  event->event.value = evt->value;
  /* XXX: are these always the correct values?? */
  event->event.min_value = 0;
  event->event.max_value = 256;
  event->name = translate_usb_input_range_to_name (event);
  //g_message ("parsed event range, code %d type %d value %d", 
             //evt->code, evt->type, evt->value);
  gdam_input_trap_emit (NULL, (GdamInputEvent *) event);
}


/* On usb data input, we take an IOHIDEventStruct from the 
 * OS X HID Manager and create a GdamIokitHidEvent.  We then emit the
 * event to anyone who had trapped it. */
static void 
gdam_usbevent_parse_event(GdamIokitHid* hid, usb_input_event* evt)
{
  GdamIokitHidEvent *event;

#if 0
  if (evt->type  == EV_ABS) 
    {
      gdam_usbevent_parse_event_range (hid, evt);
      //return;  /* not returning -> also emit iokithidevent */
    }
#endif

  event = g_new0 (GdamIokitHidEvent, 1);
  event->event.input_device = (GdamInputDevice *) (hid);
  event->event.event_type = gdam_iokithid_event_get_type ();
#if 0
  if (evt->type  == EV_REL)
    event->event.event_type = gdam_iokithid_event_variable_get_type ();
  if (evt->type  == EV_KEY)
    event->event.event_type = gdam_iokithid_event_key_get_type ();
  //event->code = evt->code;
#endif
  event->type = evt->type;
  event->value = evt->value;
  event->event.number = evt->value;
  event->name = translate_usb_input_to_name (event);
  //g_message ("parsed hid event, code %d type %d value %d", 
             //evt->code, evt->type, evt->value);
  gdam_input_trap_emit (NULL, (GdamInputEvent *) event);
}

static gboolean gdam_usbevent_process  (GdamInputDeviceFile *device_file,
                                            guint8              *hid_buf,
					    int                  num_read)
{
  GdamIokitHid *hid = GDAM_IOKIT_HID (device_file);
  int index;
  int num_to_process = num_read;
  guint8* buf_ptr = hid_buf;

  if (num_read <= 0)
    {
      g_warning ("usb device closed...");
      return FALSE;
    }
  while (num_to_process > 0) {
    int evtsize = sizeof (usb_input_event);
    if (num_read >= evtsize) 
    {
      usb_input_event* evt;
      evt = (usb_input_event*)buf_ptr;
      gdam_usbevent_parse_event (hid, evt);
      num_to_process -= evtsize;
      buf_ptr += evtsize;
    }
    else 
      {
	g_warning ("iokithid, had only %d bytes of %d", num_to_process, evtsize);
	/* TODO: does this close device?  we probably should... */
	return FALSE;
      }
  }
  return TRUE;
}
#endif

static CFIndex element_index;
static int nesting_level = 0;

static CFIndex 
get_element_index()
{
  return element_index;
}

static void
set_element_index (CFIndex new_index)
{
  element_index = new_index;
}

static void
increment_element_index (CFIndex increment)
{
  element_index += increment;
}

static void 
my_indent ()
{
  int i;
  for (i=0; i<nesting_level; i++)
    fprintf (stderr, "    ");
}

static void
my_start_block (gboolean print_index)
{
  //g_message ("START BLOCK");
  if (print_index)
    {
      my_indent();
      fprintf (stderr, "[%ld]\n", get_element_index());
    }
  nesting_level++;
}

static void
my_end_block ()
{
  nesting_level--;
}

static void 
dump_string (CFTypeRef object)
{
  const char *c = CFStringGetCStringPtr (object, CFStringGetSystemEncoding ());
  if (c)
    {
      fprintf (stderr, "%s", c);
    }
  else
    {
      CFIndex buffersize = CFStringGetLength(object) + 1;
      char *buffer = (char *) malloc (buffersize);
      if (buffer)
        {
          if (CFStringGetCString (object, buffer, buffersize,
              CFStringGetSystemEncoding()))
            fprintf (stderr, "%s", buffer);
          free (buffer);
        }
    }
}

static void dump_object (CFTypeRef object, GdamIokitHid *iokithid);
static void dump_element (CFDictionaryRef dictionary, CFStringRef key, GdamIokitHid *iokithid);

static void
array_show_applier (const void *value, void *parameter)
{
  GdamIokitHid *iokithid = parameter;
  if (CFGetTypeID (value) != CFDictionaryGetTypeID()) return;
  dump_object (value, iokithid);
}

static void
dump_array (CFArrayRef object, GdamIokitHid *iokithid)
{
  CFRange range = {0, CFArrayGetCount (object)};
  CFIndex saved_index = get_element_index();
  set_element_index (0);
  fprintf (stderr, "\n");
  my_start_block (FALSE);
  CFArrayApplyFunction (object, range, array_show_applier, iokithid);
  my_end_block ();
  set_element_index (saved_index);
}

static void
dump_boolean (CFBooleanRef object)
{
  fprintf (stderr, CFBooleanGetValue (object) ? "true" : "false");
}

IOHIDElementCookie gXAxisCookie = 0;
IOHIDElementCookie gButton1Cookie = 0;
IOHIDElementCookie gButton2Cookie = 0;
IOHIDElementCookie gButton3Cookie = 0;

static void
store_important_cookies (CFDictionaryRef element, GdamIokitHid *iokithid)
{
  CFTypeRef  object;
  long number = 0;
  IOHIDElementCookie cookie = 0;
  long usage = 0;
  long usagePage = 0;
  HRESULT result;

  /* Get Cookie */
  object = CFDictionaryGetValue (element, CFSTR (kIOHIDElementCookieKey));
  g_return_if_fail (object != 0 && CFGetTypeID (object) == CFNumberGetTypeID());
  g_return_if_fail (CFNumberGetValue((CFNumberRef) object, kCFNumberLongType, &number));
  cookie = (IOHIDElementCookie) number;

  /* Get Usage */
  object = CFDictionaryGetValue (element, CFSTR (kIOHIDElementUsageKey));
  g_return_if_fail (object != 0 && CFGetTypeID (object) == CFNumberGetTypeID());
  g_return_if_fail (CFNumberGetValue((CFNumberRef) object, kCFNumberLongType, &number));
  usage = number;

  /* Get Usage Page */
  object = CFDictionaryGetValue (element, CFSTR (kIOHIDElementUsagePageKey));
  g_return_if_fail (object != 0 && CFGetTypeID (object) == CFNumberGetTypeID());
  g_return_if_fail (CFNumberGetValue((CFNumberRef) object, kCFNumberLongType, &number));
  usagePage = number;

  result = (*iokithid->queue)->addElement (iokithid->queue, cookie, 0);
  g_message ("ADD COOKIE u=%ld up=%ld", usage, usagePage);

  /* x axis */
  if (usage == 0x30 && usagePage == 0x01)
    {
      gXAxisCookie = cookie;
      g_message ("MATCHED XAXIS COOKIE");
    }
  else if (usage == 0x01 && usagePage == 0x09)
    {
      gButton1Cookie = cookie;
      g_message ("MATCHED BUTTON1 COOKIE");
    }
  else if (usage == 0x02 && usagePage == 0x09)
    {
      gButton2Cookie = cookie;
      g_message ("MATCHED BUTTON2 COOKIE");
    }
  else if (usage == 0x03 && usagePage == 0x09)
    {
      gButton3Cookie = cookie;
      g_message ("MATCHED BUTTON3 COOKIE");
    }
}

static void
dump_dictionary (CFDictionaryRef object, GdamIokitHid *iokithid)
{
  my_start_block (TRUE);

  dump_element (object, CFSTR (kIOHIDElementCookieKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementTypeKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementCollectionTypeKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementUsageKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementUsagePageKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementMinKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementMaxKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementScaledMinKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementScaledMaxKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementSizeKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementIsRelativeKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementIsWrappingKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementIsNonLinearKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementHasPreferredStateKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementHasNullStateKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementVendorSpecificKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementUnitKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementUnitExponentKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementNameKey), iokithid);
  dump_element (object, CFSTR (kIOHIDElementKey), iokithid);

  /* TODO: store important cookies */
  store_important_cookies (object, iokithid);

  my_end_block();
  increment_element_index (1);
}

static void
dump_number (CFNumberRef object)
{
  long number;
  if (CFNumberGetValue (object, kCFNumberLongType, &number))
    fprintf (stderr, "0x%1x (%ld)", (guint)number, number);
}

static void 
dump_object (CFTypeRef object, GdamIokitHid *iokithid)
{
  CFTypeID type = CFGetTypeID (object);
  //g_message ("DUMP OBJ");
  if      (type == CFArrayGetTypeID())      dump_array      (object, iokithid);
  else if (type == CFBooleanGetTypeID())    dump_boolean    (object);
  else if (type == CFDictionaryGetTypeID()) dump_dictionary (object, iokithid);
  else if (type == CFNumberGetTypeID())     dump_number     (object);
  else if (type == CFStringGetTypeID())     dump_string     (object);
  else
    g_warning ("unknown object type");
}

static void
dump_element (CFDictionaryRef dictionary, CFStringRef key, 
              GdamIokitHid *iokithid)
{
  CFTypeRef object = CFDictionaryGetValue (dictionary, key);
  //g_message ("DUMP EL");
  if (object)
    {
      my_indent();
      //fprintf (stderr, "    ");
      dump_string (key);
      fprintf (stderr, " -- ");
      dump_object (object, iokithid);
      fprintf (stderr, "\n");
    }
}

static void
removal_callback (void *target, IOReturn result, void *refcon, void *sender)
{
  IOHIDDeviceInterface **hid_device_interface = target;
  g_message ("REMOVAL CALLBACK %p", hid_device_interface);
}

static gboolean
create_hid_device_interface (io_object_t hid_device,
                             IOHIDDeviceInterface ***hid_device_interface)
{
  io_name_t  class_name;
  IOCFPlugInInterface **plugin_interface = NULL;
  HRESULT plugin_result = S_OK;
  SInt32 score = 0;
  IOReturn io_rv = kIOReturnSuccess;
  IOHIDCallbackFunction callback;
  callback = removal_callback;

  io_rv = IOObjectGetClass (hid_device, class_name);
  g_return_val_if_fail (io_rv == kIOReturnSuccess, FALSE);

  g_message ("found device type %s, creating plugin interface\n", class_name);

  /* We create a plugin interface solely so that it can in turn
   * create a hid device interface for us.  once this is done,
   * we free the plugin interface. */
  io_rv = IOCreatePlugInInterfaceForService (hid_device,
                   kIOHIDDeviceUserClientTypeID,
                   kIOCFPlugInInterfaceID,
                   &plugin_interface, &score);
  if (io_rv == kIOReturnSuccess)
    {
      g_message ("created plugin interface for iokit hid device, opening device interface (score %d)", (int)score);
      /* Free plugin */
      plugin_result = (*plugin_interface)->QueryInterface (plugin_interface,
				CFUUIDGetUUIDBytes (kIOHIDDeviceInterfaceID),
                		(LPVOID) hid_device_interface);
      g_return_val_if_fail (plugin_result == S_OK, FALSE); 
      //plugin_result = (*plugin_interface)->Release (plugin_interface);
      //g_return_val_if_fail (plugin_result == S_OK, FALSE); 

      /* set removal callback */
      io_rv = (**hid_device_interface)->setRemovalCallback 
        (*hid_device_interface, 
	 &callback,
	 *hid_device_interface,
	 *hid_device_interface);
	 //iokithid);
      if (io_rv != kIOReturnSuccess);
	g_warning ("error setting removal callback: %s (%x,%d)", 
		   mach_error_string (io_rv), io_rv, io_rv & 0xffffff);
      return TRUE;
    }
  else
    {
      g_warning ("failed to create plugin interface for iokit hid device");
      return FALSE;
    }
}

static void
hid_callback (void *target, IOReturn result, void *refcon, void *sender)
{
  IOHIDQueueInterface **queue = target;
  IOHIDEventStruct event;
  AbsoluteTime maxtime = {0,0};
/* XXX: check if queue is empty and handle while */
  g_message ("about to get next hid event");
  (*queue)->getNextEvent (queue, &event, maxtime, 0);
  g_message ("got hid event type %d val %d", (int)event.type, (int)event.value);
}

static void
create_queue (IOHIDDeviceInterface **hid_device_interface,
              GdamIokitHid *iokithid)
{
  IOHIDQueueInterface **queue;
  HRESULT result;

  g_message ("creating queue for iokit hid");

  queue = (*hid_device_interface)->allocQueue (hid_device_interface);
  g_return_if_fail (queue != NULL);
  result = (*queue)->create (queue, 0, 32); /* 3rd arg is max events in queue */
  g_return_if_fail (result == S_OK);
  g_message ("allocated and created queue");

  iokithid->queue = queue;
}

static void
test_queue (IOHIDDeviceInterface **hid_device_interface,
              GdamIokitHid *iokithid)
{
  HRESULT result;
  IOHIDQueueInterface **queue = iokithid->queue;
  int i;

  g_message ("testing iokit hid device");

  sleep (1);

  for (i=0; i< 10; i++)
    {
      AbsoluteTime zero_time = {0,0};
      IOHIDEventStruct event;

      /* Check queue */
      result = (*queue)->getNextEvent (queue, &event, zero_time, 0);
      if (result)
        g_message ("couldn't get event (???) result = %lx", result);
      else
        g_message ("EVENT: %lx %ld", (unsigned long) event.elementCookie,
                   event.value);

      /* Check Cookies */
      result = (*hid_device_interface)->getElementValue (hid_device_interface,
						       gXAxisCookie, &event);
      if (result) 
        g_message ("couldn't get x axis result = %lx", result);
      else
	g_message ("X AXIS (cookie=%lx): %ld", 
		   (unsigned long) event.elementCookie, event.value);
      result = (*hid_device_interface)->getElementValue (hid_device_interface,
						       gButton1Cookie, &event);
      if (result) 
        g_message ("couldn't get button 1 result = %lx", result);
      else
	g_message ("BUTTON 1 (cookie=%lx): %ld", 
		   (unsigned long) event.elementCookie, event.value);
      result = (*hid_device_interface)->getElementValue (hid_device_interface,
						       gButton2Cookie, &event);
      if (result) 
        g_message ("couldn't get button 2 result = %lx", result);
      else
	g_message ("BUTTON 2 (cookie=%lx): %ld", 
		   (unsigned long) event.elementCookie, event.value);
      result = (*hid_device_interface)->getElementValue (hid_device_interface,
						       gButton3Cookie, &event);
      if (result) 
        g_message ("couldn't get button 3 result = %lx", result);
      else
	g_message ("BUTTON 3 (cookie=%lx): %ld", 
		   (unsigned long) event.elementCookie, event.value);

      sleep (1);
    }
}


static void
start_queue (IOHIDDeviceInterface **hid_device_interface,
              GdamIokitHid *iokithid)
{
  //HRESULT result;
  IOReturn io_rv;
  IOHIDQueueInterface **queue = iokithid->queue;
  IOHIDCallbackFunction callback;

  callback = hid_callback;

  g_message ("testing device interface for iokit hid");

  io_rv = (*queue)->setEventCallout (queue, &callback, queue, hid_device_interface);
  if (io_rv != kIOReturnSuccess);
    g_warning ("error setting event callout: %s (%x,%d)", 
	       mach_error_string (io_rv), io_rv, io_rv & 0xffffff);

  io_rv = (*queue)->start (queue);
  if (io_rv != kIOReturnSuccess);
    g_warning ("error starting event queue: %s (%x,%d)", 
	       mach_error_string (io_rv), io_rv, io_rv & 0xffffff);

  g_message ("STARTED QUEUE");

#if 0
  sleep (1);

  for (i=0; i< 10; i++)
    {
      AbsoluteTime zero_time = {0,0};
      IOHIDEventStruct event;
      result = (*queue)->getNextEvent (queue, &event, zero_time, 0);
      if (result)
        g_message ("couldn't get event (???) result = %lx", result);
      else
        g_message ("event: %lx %ld", (unsigned long) event.elementCookie,
                   event.value);
      sleep (1);
    }
#endif

#if 0
/* XXX: when? */
  result = (*queue)->stop (queue);
  result = (*queue)->dispose (queue);
  (*queue)->Release (queue);
  g_message ("stopped, disposed, and released event queue");
#endif
}

static void
walk_device_elements (io_object_t *hidDevice, GdamIokitHid *iokithid)
{
  /* show properties */
  kern_return_t result;
  CFMutableDictionaryRef properties = 0;
  char path[512];

  result = IORegistryEntryGetPath (*hidDevice, kIOServicePlane, path);
  g_return_if_fail (result == KERN_SUCCESS);
  g_message ("HID DEVICE PATH: %s \n", path);
  result = IORegistryEntryCreateCFProperties (*hidDevice, &properties,
					      kCFAllocatorDefault,
					      kNilOptions);
  if ((result == KERN_SUCCESS) && properties)
    {
      dump_element (properties, CFSTR (kIOHIDTransportKey), iokithid);
      dump_element (properties, CFSTR (kIOHIDVendorIDKey), iokithid);
      dump_element (properties, CFSTR (kIOHIDProductIDKey), iokithid);
      dump_element (properties, CFSTR (kIOHIDVersionNumberKey), iokithid);
      dump_element (properties, CFSTR (kIOHIDManufacturerKey), iokithid);
      dump_element (properties, CFSTR (kIOHIDProductKey), iokithid);
      dump_element (properties, CFSTR (kIOHIDSerialNumberKey), iokithid);
      dump_element (properties, CFSTR (kIOHIDLocationIDKey), iokithid);
      dump_element (properties, CFSTR (kIOHIDPrimaryUsageKey), iokithid);
      dump_element (properties, CFSTR (kIOHIDPrimaryUsagePageKey), iokithid);
      dump_element (properties, CFSTR (kIOHIDElementKey), iokithid);
      CFRelease (properties);
    }
} 

static gboolean   
gdam_iokit_hid_open (GdamInputDevice *input_device)
{
/* why dont definitions match sample code?? */
  mach_port_t masterPort = 0;
  io_iterator_t hidObjectIterator = 0;
  IOReturn ioReturnValue = kIOReturnSuccess;
  gboolean rv = FALSE;
  CFMutableDictionaryRef hidMatchDictionary = NULL;
  gboolean noMatchingDevices = FALSE;
  GdamIokitHid *iokithid = GDAM_IOKIT_HID (input_device);

  ioReturnValue = IOMasterPort (bootstrap_port, &masterPort);
  g_return_val_if_fail (ioReturnValue == kIOReturnSuccess, FALSE);

  /* find hid devices */
  hidMatchDictionary = IOServiceMatching (kIOHIDDeviceKey);
  ioReturnValue = IOServiceGetMatchingServices (masterPort, hidMatchDictionary, &hidObjectIterator);
  noMatchingDevices = ((ioReturnValue != kIOReturnSuccess) 
		       | (hidObjectIterator == 0));
  if (noMatchingDevices)
    {
      g_warning ("no hid devices attached");
      return FALSE;
    }
   /* GetMatchingServices unreffed the dictionary for us */
   hidMatchDictionary = NULL;

  /* test hid devices */
  if (hidObjectIterator)
    {
      io_object_t hidDevice = 0;
      IOHIDDeviceInterface **hid_device_interface = NULL;

      while ((hidDevice = (IOIteratorNext (hidObjectIterator))))
	{
          /* show properties */
          kern_return_t result;
          //CFMutableDictionaryRef properties = 0;
          char path[512];

          result = IORegistryEntryGetPath (hidDevice, kIOServicePlane, path);
	  g_return_val_if_fail (result == KERN_SUCCESS, FALSE);
          g_message ("HID DEVICE PATH: %s \n", path);
#if 0
          result = IORegistryEntryCreateCFProperties (hidDevice, &properties,
						      kCFAllocatorDefault,
						      kNilOptions);
          if ((result == KERN_SUCCESS) && properties)
	    {
	      dump_element (properties, CFSTR (kIOHIDTransportKey), iokithid);
	      dump_element (properties, CFSTR (kIOHIDVendorIDKey), iokithid);
	      dump_element (properties, CFSTR (kIOHIDProductIDKey), iokithid);
	      dump_element (properties, CFSTR (kIOHIDVersionNumberKey), iokithid);
	      dump_element (properties, CFSTR (kIOHIDManufacturerKey), iokithid);
	      dump_element (properties, CFSTR (kIOHIDProductKey), iokithid);
	      dump_element (properties, CFSTR (kIOHIDSerialNumberKey), iokithid);
	      dump_element (properties, CFSTR (kIOHIDLocationIDKey), iokithid);
	      dump_element (properties, CFSTR (kIOHIDPrimaryUsageKey), iokithid);
	      dump_element (properties, CFSTR (kIOHIDPrimaryUsagePageKey), iokithid);
	      dump_element (properties, CFSTR (kIOHIDElementKey), iokithid);
              CFRelease (properties);
	    }
          g_message ("done dumping hid device info\n\n");
#endif

          /* create interface */
          create_hid_device_interface (hidDevice, &hid_device_interface);
          iokithid->hid_device_interface = hid_device_interface;

          /* we can now release the device for which we have an interface */
          ioReturnValue = IOObjectRelease (hidDevice);
          g_return_val_if_fail (ioReturnValue == kIOReturnSuccess, FALSE);

          /* if interface creation was successful, test interface */
          if (hid_device_interface != NULL)
            {
              IOReturn io_rv = kIOReturnSuccess;
	      io_rv = (*hid_device_interface)->open (hid_device_interface, 0);
	      g_return_val_if_fail (io_rv == kIOReturnSuccess, FALSE);
              g_message ("created and opened device interface for iokit hid");

	      /* test interface */
              create_queue (hid_device_interface, iokithid);

              /* add all cookies to queue */
              walk_device_elements (&hidDevice, iokithid);

              start_queue (hid_device_interface, iokithid);

              test_queue (hid_device_interface, iokithid);

#if 0
/* XXX: put in close() */
	      /* Close the device and release the interface */
	      io_rv = (*hid_device_interface)->close (hid_device_interface);
	      (*hid_device_interface)->Release (hid_device_interface);
	      g_message ("closed and released hid device interface");
#endif
	    }
          else
            g_warning ("failed to create device interface for iokit hid");
	}
       IOObjectRelease (hidObjectIterator);
    }

#if 0
  if (masterPort)
    mach_port_deallocate (mach_task_self(), masterPort);
#endif

  /* XXX: what does rv mean?? */
  rv = TRUE;

  return rv;
}


static void 
gdam_iokit_hid_close (GdamInputDevice *input_device)
{
}

static void gdam_usbevent_class_init(GdamInputDeviceClass* klass)
{
  //klass->process = gdam_usbevent_process;
  klass->open = gdam_iokit_hid_open;
  klass->close = gdam_iokit_hid_close;
}

/* A device file which gets hid events from the linux kernel input core */
GtkType gdam_iokit_hid_get_type()
{
  static GtkType usbevent_type = 0;
  if (!usbevent_type)
    {
      static const GtkTypeInfo usbevent_info = {
	"GdamIokitHid",
	sizeof (GdamIokitHid),
	sizeof (GdamIokitHidClass),
	(GtkClassInitFunc) gdam_usbevent_class_init,
	(GtkObjectInitFunc) gdam_usbevent_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_INPUT_DEVICE;
      usbevent_type = gtk_type_unique (parent, &usbevent_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
      //(void) gdam_iokithid_event_get_type ();
      //(void) gdam_iokithid_event_variable_get_type ();
      //(void) gdam_iokithid_event_absolute_get_type ();
      //(void) gdam_iokithid_event_key_get_type ();
    }
  return usbevent_type;
}

#if 0
static char* 
gdam_usb_event_stringify (GdamInputEvent *event)
{
  GdamIokitHidEvent *hid_event = (GdamIokitHidEvent *) event;
  return g_strdup_printf ("IokitHidEvent: %s", hid_event->name);
}

static char *
gdam_usb_event_key_stringify (GdamInputEvent *event)
{
  GdamIokitHidEvent *hid_event = (GdamIokitHidEvent *) event;
  return g_strdup_printf ("IokitHidEventKey: %s", hid_event->name);
}

static char *
gdam_usb_event_variable_stringify (GdamInputEvent *event)
{
  GdamIokitHidEvent *hid_event = (GdamIokitHidEvent *) event;
  return g_strdup_printf ("IokitHidEventVariable: %s", hid_event->name);
}

static char *
gdam_usb_event_absolute_stringify (GdamInputEvent *event)
{
  GdamIokitHidEventRange *hid_event = (GdamIokitHidEventRange *) event;
  return g_strdup_printf ("IokitHidEventAbsolute: %s", hid_event->name);
}

static unsigned  
gdam_usb_event_variable_hash (GdamInputEvent *event)
{
  return GPOINTER_TO_UINT (event->input_device)
    ^ (((GdamIokitHidEvent *) event)->code << 16)
    ^ (((GdamIokitHidEvent *) event)->type << 6)
    ^ (event->event_type << 8);
}

static unsigned 
gdam_usb_event_range_hash (GdamInputEvent *event)
{
  return GPOINTER_TO_UINT (event->input_device)
    ^ (((GdamIokitHidEventRange *) event)->code << 16)
    ^ (((GdamIokitHidEventRange *) event)->type << 6)
    ^ (event->event_type << 8);
}

static unsigned gdam_usb_event_hash(GdamInputEvent* event)
{
  return GPOINTER_TO_UINT (event->input_device)
    ^ (((GdamIokitHidEvent *) event)->code << 16)
    ^ (((GdamIokitHidEvent *) event)->type << 6)
    ^ (((GdamIokitHidEvent *) event)->value << 4)
    ^ (event->event_type << 8);
}

static GHashTable* gdam_usb_event_to_keyvalues(GdamInputEvent* event)
{
  GHashTable *table = g_hash_table_new (g_str_hash, g_str_equal);
  const char *device_name;
  GdamIokitHidEvent *usb_event = (GdamIokitHidEvent *) event;
  table = g_hash_table_new (g_str_hash, g_str_equal);

  device_name = event->input_device->device_name;
  gdam_input_store_string (table, "device", device_name);

  gdam_input_store_int (table, "type", usb_event->type);
  gdam_input_store_int (table, "code", usb_event->code);
  gdam_input_store_int (table, "value", usb_event->value);
  gdam_input_store_string (table, "name", usb_event->name);
  return table;
}

static GdamInputEvent * 
gdam_usb_event_from_keyvalues	(int         type,
                                 GHashTable *table)
{

  GdamIokitHidEvent *event;
  char *device_name;
  int htype, code, value;

  event = g_new0 (GdamIokitHidEvent, 1);
  event->event.event_type = type;
  event->event.input_id = 0;
  device_name = g_hash_table_lookup (table, "device");
  gdam_input_event_parse_device ((GdamInputEvent *) event, device_name);
  htype = gdam_input_fetch_int (table, "type");
  code = gdam_input_fetch_int (table, "code");
  value = gdam_input_fetch_int (table, "value");
#if 0
//  if (type == gdam_iokithid_event_key_get_type ())
//    event->type = EV_KEY;
  else if (htype == -1)
    {
      g_warning ("no 'type' in usb event");
      g_free (event);
      return NULL;
    }
  else
    event->type = htype;
#endif
  if (code == -1)
    {
      g_warning ("no 'code' in usb event");
      g_free (event);
      return NULL;
    }
  event->code = code;
  event->value = value;
  event->event.number = value;
  event->name = translate_usb_input_to_name (event);
  return (GdamInputEvent *) event;
}

static GHashTable * 
gdam_usb_event_range_to_keyvalues (GdamInputEvent *event)
{
  GHashTable *table = g_hash_table_new (g_str_hash, g_str_equal);
  const char *device_name;
  GdamIokitHidEventRange *usb_event = (GdamIokitHidEventRange *) event;
  table = g_hash_table_new (g_str_hash, g_str_equal);

  device_name = event->input_device->device_name;
  gdam_input_store_string (table, "device", device_name);

  gdam_input_store_int (table, "type", usb_event->type);
  gdam_input_store_int (table, "code", usb_event->code);
  gdam_input_store_int (table, "value", usb_event->event.value);
  gdam_input_store_string (table, "name", usb_event->name);
  return table;
}

static GdamInputEvent *
gdam_usb_event_range_from_keyvalues     (int 	      type,  
                                         GHashTable   *table)
{

  GdamIokitHidEventRange *event;
  char *device_name;
  int htype, code, value;

  event = g_new0 (GdamIokitHidEventRange, 1);
  event->event.event_type = type;
  event->event.input_id = 0;
  device_name = g_hash_table_lookup (table, "device");
  gdam_input_event_parse_device ((GdamInputEvent *) event, device_name);
  htype = gdam_input_fetch_int (table, "type");
  code = gdam_input_fetch_int (table, "code");
  value = gdam_input_fetch_int (table, "value");
  if (htype == -1)
    {
      g_warning ("no 'type' in usb event");
      g_free (event);
      return NULL;
    }
  if (code == -1)
    {
      g_warning ("no 'code' in usb event");
      g_free (event);
      return NULL;
    }
  event->type = htype;
  event->code = code;
  event->value = value;
  event->event.value = value;
  event->event.min_value = 0;
  event->event.max_value = 256;
  event->name = translate_usb_input_range_to_name (event);
  return (GdamInputEvent *) event;
}

/* Events for devices which emit variable values.  Because 'value' can
 * cover a wide range, we don't match against it.
 */
static gboolean 
gdam_usb_event_variable_equal (GdamInputEvent *event1,
                               GdamInputEvent *event2)
{
  GdamIokitHidEvent *kevent1 = (GdamIokitHidEvent *) event1;
  GdamIokitHidEvent *kevent2 = (GdamIokitHidEvent *) event2;
  return event1->input_device == event2->input_device
    && event1->event_type == event2->event_type
    && kevent1->type == kevent2->type
    && kevent1->code == kevent2->code;
}

static gboolean  
gdam_usb_event_range_equal (GdamInputEvent *event1,
			    GdamInputEvent *event2)
{
  GdamIokitHidEventRange *kevent1 = (GdamIokitHidEventRange *) event1;
  GdamIokitHidEventRange *kevent2 = (GdamIokitHidEventRange *) event2;
  return event1->input_device == event2->input_device
    && event1->event_type == event2->event_type
    && kevent1->type == kevent2->type
    && kevent1->code == kevent2->code;
}

static gboolean 
gdam_usb_event_equal (GdamInputEvent *event1,
                      GdamInputEvent *event2)
{
  GdamIokitHidEvent *kevent1 = (GdamIokitHidEvent *) event1;
  GdamIokitHidEvent *kevent2 = (GdamIokitHidEvent *) event2;
  return event1->input_device == event2->input_device
    && event1->event_type == event2->event_type
    && kevent1->type == kevent2->type
    && kevent1->code == kevent2->code
    && kevent1->value == kevent2->value;
}

static GdamInputEvent* gdam_usb_event_canonicalize(GdamInputEvent* event)
{
  return g_memdup (event, sizeof (GdamIokitHidEvent));
}

static GdamInputEvent* gdam_usb_event_range_canonicalize(GdamInputEvent* event)
{
  return g_memdup (event, sizeof (GdamIokitHidEventRange));
}

/* The generic usb hid event, as presented by the linux kernel input core.
 * This is used for keys and other event which emit one or more constant values.
 * An event is defined for each value emitted.  This type is used for any
 * HID event type which isn't being parsed yet. */
int gdam_iokithid_event_get_type()
{
  static GdamInputEventTypeInfo type_info = {
    "GdamIokitHidEvent",
    gdam_usb_event_stringify,
    gdam_usb_event_hash,
    gdam_usb_event_equal,
    gdam_usb_event_to_keyvalues,
    gdam_usb_event_from_keyvalues,
    gdam_usb_event_canonicalize,
    NULL
  };
  static guint type = 0;
  if (type == 0)
    type = gdam_input_event_get_unique
      (gdam_input_event_int_get_type (), GDAM_TYPE_IOKIT_HID, &type_info);
  return type;
}

/* An HID event which includes a variable value, such as a relative position
 * event (mice, trackball, touchpad)  In contrast to a normal HidEvent,
 * there is only one event for all values. */
int gdam_iokithid_event_variable_get_type()
{
  static GdamInputEventTypeInfo type_info = {
    "GdamIokitHidEventVariable",
    gdam_usb_event_stringify,
    gdam_usb_event_variable_hash,
    gdam_usb_event_variable_equal,
    gdam_usb_event_to_keyvalues,
    gdam_usb_event_from_keyvalues,
    gdam_usb_event_canonicalize,
    NULL
  };
  static guint type = 0;
  if (type == 0)
    type = gdam_input_event_get_unique
      (gdam_input_event_int_get_type (), GDAM_TYPE_IOKIT_HID, &type_info);
  return type;
}

/* XXX: don't parse type from events whose type is implicit from name */

/* An HID event which includes an absolute value, such as a joystick axis
 * or midi slider. */
int gdam_iokithid_event_absolute_get_type()
{
  static GdamInputEventTypeInfo type_info = {
    "GdamIokitHidEventAbsolute",
    gdam_usb_event_absolute_stringify,
    gdam_usb_event_range_hash,
    gdam_usb_event_range_equal,
    gdam_usb_event_range_to_keyvalues,
    gdam_usb_event_range_from_keyvalues,
    gdam_usb_event_range_canonicalize,
    NULL
  };
  static guint type = 0;
  if (type == 0)
    type = gdam_input_event_get_unique
      (gdam_input_event_range_get_type (), GDAM_TYPE_IOKIT_HID, &type_info);
  return type;
}

/* Key press... generally 1 for press, 2 for release. */
int 
gdam_iokithid_event_key_get_type()
{
  static GdamInputEventTypeInfo type_info = {
    "GdamIokitHidEventKey",
    gdam_usb_event_key_stringify,
    gdam_usb_event_hash,
    gdam_usb_event_equal,
    gdam_usb_event_to_keyvalues,
    gdam_usb_event_from_keyvalues,
    gdam_usb_event_canonicalize,
    NULL
  };
  static guint type = 0;
  if (type == 0)
    type = gdam_input_event_get_unique
      (gdam_input_event_int_get_type (), GDAM_TYPE_IOKIT_HID, &type_info);
  return type;
}
#endif


#endif     /*  HAVE_IOKIT_HID_IOHIDLIB_H */
