/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_LADSPA_FILTER_H_
#define __GDAM_LADSPA_FILTER_H_


typedef struct _GdamLadspaFilter GdamLadspaFilter;
typedef struct _GdamLadspaFilterClass GdamLadspaFilterClass;
typedef struct _GdamLadspaControl GdamLadspaControl;
typedef struct _GdamLadspaPluginType GdamLadspaPluginType;
typedef struct _GdamLadspaParameterType GdamLadspaParameterType;

#include "../gdamfilter.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _GdamLadspaParameterType {
	char*			name;
	gdouble			min_value;
	gdouble			max_value;
	unsigned		is_toggle : 1,
				has_min_value : 1,
				has_max_value : 1,
				is_integer : 1,
				is_logarithmic : 1,
				is_sample_rate_ratio : 1;
};

struct _GdamLadspaPluginType {
	char*			database_name;
	char*			module_name;
	char*			author;
	int			module_index;
	guint32			unique_id;
	int			num_controls;
	GdamLadspaParameterType* controls;
	int			num_meters;
	GdamLadspaParameterType* meters;
	int			num_inputs;
	int			num_outputs;
};

GdamLadspaPluginType*	gdam_ladspa_plugin_type_get_by_name(const char* name);

/* This just returns a reference -- do not free it. */
char**                  gdam_ladspa_plugin_type_get_names  ();

GtkType gdam_ladspa_filter_get_type();
#define GDAM_TYPE_LADSPA_FILTER			(gdam_ladspa_filter_get_type ())
#define GDAM_LADSPA_FILTER(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_LADSPA_FILTER, GdamLadspaFilter))
#define GDAM_LADSPA_FILTER_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_LADSPA_FILTER, GdamLadspaFilterClass))
#define GDAM_IS_LADSPA_FILTER(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_LADSPA_FILTER))
#define GDAM_IS_LADSPA_FILTER_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_LADSPA_FILTER))

struct _GdamLadspaFilterClass {
	GdamFilterClass		filter_class;

	void                  (*meters_updated)(GdamLadspaFilter* filter);
	void                  (*control_updated)(GdamLadspaFilter* filter, 
	                                         int control_index);
};

struct _GdamLadspaFilter {
	GdamFilter		filter;

	GdamLadspaPluginType*	plugin_type;

	/* Left/right value pairs. */
	gdouble*		controls;
	gdouble*		meters;

	int			timer_id;
	gboolean*		which_meters_to_query;
	int			controls_pending;
	int			making_arg_request;
};


GdamLadspaFilter* gdam_ladspa_filter_new(const char*       database_name);

/* Whether to emit ``update'' signal periodically for that meter.
 * (It will be emitted whenever the `which_meters_to_query' set is nonempty.
 * You must know the status of the flag you are checking.) */
void gdam_ladspa_filter_set_meter_status(GdamLadspaFilter* filter,
                                         int               which_meter,
					 gboolean          new_query_status);

void gdam_ladspa_filter_set_control     (GdamLadspaFilter* filter,
                                         int               control_index,
                                         gdouble           left_value,
                                         gdouble           right_value);

void gdam_ladspa_filter_rescan_database();
void gdam_ladspa_filter_save_preset     (GdamLadspaFilter* filter, 
					 const char*       preset_name);
void gdam_ladspa_filter_load_preset     (GdamLadspaFilter* filter, 
					 const char*       preset_name);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
