/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdammeter.h"

#include <gdam/common/gdamplugin.h>
static int gdam_meter_append_start_request(GdamFilter* filter,
				           int new_source_id,
				           GdamRequestBuilder* builder);

static void gdam_meter_init(GdamMeter* meter)
{
}

static void gdam_meter_arg_set_level(GtkObject* object, GdamArg* arg)
{
  //g_message("meter level set to %f", GDAM_ARG_DOUBLE(arg));
}
static void gdam_meter_arg_get_level(GtkObject* object, GdamArg* arg)
{
  GDAM_ARG_DOUBLE (arg) = 0;
}

static void gdam_meter_class_init(GdamMeterClass* meter_class)
{
  GdamFilterClass *filter_class = GDAM_FILTER_CLASS (meter_class);
  GtkObjectClass *object_class = GTK_OBJECT_CLASS (meter_class);
  meter_class->server_class_name = "GdasVolumeMeter";
  filter_class->append_start_request = gdam_meter_append_start_request;
  gdam_arg_register (object_class,
		     "meterlevel",
		     GDAM_ARG_TYPE_DOUBLE,
		     gdam_meter_arg_set_level,
		     gdam_meter_arg_get_level, NULL, ",", 0);
}

static int gdam_meter_append_start_request(GdamFilter* filter,
				           int new_source_id,
				           GdamRequestBuilder* builder)
{
  GdamMeter *meter = GDAM_METER (filter);
  GdamRequest *request;
  GdamMeterClass *meter_class;
  GdamModel *model = GDAM_MODEL (filter);
  meter_class = GDAM_METER_CLASS (GTK_OBJECT (filter)->klass);
  gtk_object_ref (GTK_OBJECT (meter));
  model->source_id = gdam_channel_new_local_id (model->channel);
  request = gdam_request_create_source (meter_class->server_class_name,
					model->source_id,
					gdam_model_got_subsource,
					gdam_model_didnt_get_subsource,
					meter,
					1,
					"filtered",
					GDAM_ARG_TYPE_INT, new_source_id);
  return gdam_request_builder_append_create (builder, request);
}


GtkType gdam_meter_get_type()
{
  static GtkType meter_type = 0;
  if (!meter_type)
    {
      static const GtkTypeInfo meter_info = {
	"GdamMeter",
	sizeof (GdamMeter),
	sizeof (GdamMeterClass),
	(GtkClassInitFunc) gdam_meter_class_init,
	(GtkObjectInitFunc) gdam_meter_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_FILTER;
      meter_type = gtk_type_unique (parent, &meter_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return meter_type;
}
