/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamproducercache.h"
#include <gdam/common/gdamplugin.h>

static GtkObjectClass* parent_class = NULL;

static int gdam_producer_cache_play_request(GdamProducer*       producer,
					    GdamPlayInfo*       play_info,
					    int                 source_id,
					    GdamRequestBuilder* builder);

static void gdam_producer_cache_init(GdamProducerCache* producer_cache)
{
  producer_cache->length = -1;
}
static void gdam_producer_cache_class_init(GdamProducerClass* producer_class)
{
  //GtkObjectClass* object_class = GTK_OBJECT_CLASS(producer_class);
  producer_class->play_request = gdam_producer_cache_play_request;
  //object_class->destroy = gdam_producer_cache_destroy;
  parent_class = gtk_type_class (GDAM_TYPE_PRODUCER);
}

GtkType gdam_producer_cache_get_type()
{
  static GtkType producer_cache_type = 0;
  if (!producer_cache_type)
    {
      static const GtkTypeInfo producer_cache_info = {
	"GdamProducerCache",
	sizeof (GdamProducerCache),
	sizeof (GdamProducerCacheClass),
	(GtkClassInitFunc) gdam_producer_cache_class_init,
	(GtkObjectInitFunc) gdam_producer_cache_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_PRODUCER;
      producer_cache_type = gtk_type_unique (parent, &producer_cache_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return producer_cache_type;
}
static int gdam_producer_cache_play_request(GdamProducer*       producer,
					    GdamPlayInfo*       play_info,
					    int                 source_id,
					    GdamRequestBuilder* builder)
{
  GdamProducerCache *cache = GDAM_PRODUCER_CACHE (producer);
  GdamModel *model = GDAM_MODEL (cache);
  int local_source_id;
  gtk_object_ref (GTK_OBJECT (cache));
  model->source_id = gdam_channel_new_local_id (model->channel);
  local_source_id = gdam_request_builder_append_create (builder,
							gdam_request_create_source
							("GdasCache",
							 model->source_id,
							 gdam_model_got_subsource,
							 gdam_model_didnt_get_subsource,
							 cache, 3,
							 "buffer_id",
							 GDAM_ARG_TYPE_INT,
							 cache->buffer_id,
							 "position",
							 GDAM_ARG_TYPE_INT64,
							 (gint64)
							 cache->start,
							 "length",
							 GDAM_ARG_TYPE_INT,
							 cache->length));
  return local_source_id;
}
