/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamproducermicrophone.h"
#include <gdam/common/gdamplugin.h>
#include <gdam/component/gdamtoplevel.h>
#include "../gdambuffermanager.h"
#include <gtk/gtksignal.h>
#include <gdam/common/gdamplugin.h>
#include "../gdamutil.h"

static int gdam_producer_microphone_play_request(GdamProducer*       producer,
					        GdamPlayInfo*       play_info,
					        int                 source_id,
					        GdamRequestBuilder* builder);

static void 
gdam_producer_microphone_init	(GdamProducerMicrophone* producer_microphone)
{
  producer_microphone->device_name = g_strdup ("/dev/dsp1");
}

static void gdam_producer_microphone_arg_set_device_name
				(GtkObject* object, GdamArg* arg)
{
  GdamProducerMicrophone *mic = GDAM_PRODUCER_MICROPHONE (object);
  g_free (mic->device_name);
  mic->device_name = g_strdup (GDAM_ARG_STRING (arg));
}
static void gdam_producer_microphone_arg_get_device_name
				(GtkObject* object, GdamArg* arg)
{
  GdamProducerMicrophone *mic = GDAM_PRODUCER_MICROPHONE (object);
  GDAM_ARG_STRING (arg) = g_strdup (mic->device_name);
}

static void 
gdam_producer_microphone_class_init	(GdamProducerMicrophoneClass* klass)
{
  GdamProducerClass *producer_class = GDAM_PRODUCER_CLASS (klass);
  GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);
  producer_class->play_request = gdam_producer_microphone_play_request;
  gdam_arg_register (object_class,
		     "device_name",
		     GDAM_ARG_TYPE_STRING,
		     gdam_producer_microphone_arg_set_device_name,
		     gdam_producer_microphone_arg_get_device_name,
		     NULL, NULL, GDAM_ARG_VITAL);
}

GtkType gdam_producer_microphone_get_type()
{
  static GtkType producer_microphone_type = 0;
  if (!producer_microphone_type)
    {
      static const GtkTypeInfo producer_microphone_info = {
	"GdamProducerMicrophone",
	sizeof (GdamProducerMicrophone),
	sizeof (GdamProducerMicrophoneClass),
	(GtkClassInitFunc) gdam_producer_microphone_class_init,
	(GtkObjectInitFunc) gdam_producer_microphone_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = gdam_plugin_get_type_by_name ("model",
						     "GdamProducer");
      producer_microphone_type = gtk_type_unique (parent,
						  &producer_microphone_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return producer_microphone_type;
}

static int gdam_producer_microphone_play_request(GdamProducer*       producer,
					        GdamPlayInfo*       play_info,
					        int                 source_id,
					        GdamRequestBuilder* builder)
{
  GdamProducerMicrophone *microphone = GDAM_PRODUCER_MICROPHONE (producer);
  GdamChannel *channel;
  GdamModel *model;
  GdamRequest *request;
  gdouble position;
  const char *server_class;
  const char *device_name;

  g_return_val_if_fail (microphone != NULL, 0);

  model = GDAM_MODEL (microphone);
  channel = model->channel;
  g_return_val_if_fail (channel != NULL, 0);
  device_name = microphone->device_name;

  position = play_info->position;

  /* HACK:  we need some table of generic method 
     for finding out how to open the microphone.

     For now, if the device is open, assume it
     is a duplex device and use GdamMicrophoneOss.

     Otherwise, just use the generic microphone class. */
  {
    GdamToplevel *toplevel;
    toplevel = gdam_toplevel_get_global (channel);
    if (gdam_toplevel_is_device_open (toplevel, device_name))
      {
	//server_class = "GdasMicrophoneOss";
	g_message ("sorry, duplex sound support not supported and %s is opened",
		   device_name);
	return 0;
      }
    else
      {
	server_class = "GdasMicrophone";
	g_message ("Open record-only sound device %s", device_name);
      }
  }
  gdam_debug ("Creating producing microphone with position %f", position);
  gtk_object_ref (GTK_OBJECT (microphone));
  model->source_id = gdam_channel_new_local_id (model->channel);
  request = gdam_request_create_source (server_class,
					model->source_id,
					gdam_model_got_subsource,
					gdam_model_didnt_get_subsource,
					microphone,
					2,
					"use_helper",
					GDAM_ARG_TYPE_INT,
					1,
					"device_name",
					GDAM_ARG_TYPE_STRING, device_name);

  return gdam_request_builder_append_create (builder, request);
}
