/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_PRODUCER_MP3_CACHEABLE_H_
#define __GDAM_PRODUCER_MP3_CACHEABLE_H_

typedef struct _GdamProducerMp3CacheableClass GdamProducerMp3CacheableClass;
typedef struct _GdamProducerMp3Cacheable GdamProducerMp3Cacheable;

#include "../gdamsongdatabase.h"
#include "../gdamproducermp3.h"

GtkType gdam_producer_mp3_cacheable_get_type();
#define GDAM_TYPE_PRODUCER_MP3_CACHEABLE              (gdam_producer_mp3_cacheable_get_type ())
#define GDAM_PRODUCER_MP3_CACHEABLE(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_PRODUCER_MP3_CACHEABLE, GdamProducerMp3Cacheable))
#define GDAM_PRODUCER_MP3_CACHEABLE_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_PRODUCER_MP3_CACHEABLE, GdamProducerMp3CacheableClass))
#define GDAM_IS_PRODUCER_MP3_CACHEABLE(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_PRODUCER_MP3_CACHEABLE))
#define GDAM_IS_PRODUCER_MP3_CACHEABLE_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_PRODUCER_MP3_CACHEABLE))


struct _GdamProducerMp3CacheableClass {
	GdamProducerMp3Class 	mp3_class;
};

struct _GdamProducerMp3Cacheable {
	GdamProducerMp3		mp3;
	double			cache_start;
	double			cache_beats;
	  /*  this producer will do double duty for now, playing mp3 chunks. */
	int 			loop_index;  /* for loading predefined loops */
	char*			loop_name;
	int			buffer_id;
	int			buffer_length;
	/* Whether or not to play a cached buffer rather than the mp3 */
	gboolean		play_buffer;
        int			fname_arg_connect_id;
};

GdamProducerMp3Cacheable* gdam_producer_mp3_cacheable_new(GdamChannel*	channel);

void gdam_producer_mp3_cacheable_set_start(GdamProducerMp3Cacheable* mp3);
void gdam_producer_mp3_cacheable_set_beats(GdamProducerMp3Cacheable* mp3,
					   float 		     num_beats);
void gdam_producer_mp3_cacheable_export(GdamProducerMp3Cacheable* mp3);
void 
gdam_producer_mp3_cacheable_write_to_database(GdamProducerMp3Cacheable* mp3);
void 
gdam_producer_mp3_cacheable_read_from_database(GdamProducerMp3Cacheable* mp3);


#endif
