/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_RECORDER_FILTER_H_
#define __GDAM_RECORDER_FILTER_H_


typedef struct _GdamRecorderFilter GdamRecorderFilter;
typedef struct _GdamRecorderFilterClass GdamRecorderFilterClass;

#include "../gdamfilter.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

GtkType gdam_recorder_filter_get_type();
#define GDAM_TYPE_RECORDER_FILTER			(gdam_recorder_filter_get_type ())
#define GDAM_RECORDER_FILTER(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_RECORDER_FILTER, GdamRecorderFilter))
#define GDAM_RECORDER_FILTER_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_RECORDER_FILTER, GdamRecorderFilterClass))
#define GDAM_IS_RECORDER_FILTER(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_RECORDER_FILTER))
#define GDAM_IS_RECORDER_FILTER_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_RECORDER_FILTER))

struct _GdamRecorderFilterClass {
	GdamFilterClass		filter_class;

	/* Called to set whether the recording is recording. */
	void                  (*set_recording)(GdamRecorderFilter*,
	                                       gboolean       is_recording);

	/* Called to indicate whether a buffer should be written
	 * at source eof. */
	void                  (*set_save_end_buffer)(GdamRecorderFilter*,
	                                             gboolean  should_save);

	void                  (*new_length)(GdamRecorderFilter*,
	                                    int length);
};
struct _GdamRecorderFilter {
	GdamFilter			filter;

	/* If we are capturing the buffer on source-eof. */
	guint                           eof_buffer_connect_id;

	/* A thread updating the length. */
	guint                           new_length_thread;

	char*                           eof_description;

	unsigned			make_terminal : 1,
	                                is_recording : 1;
};

void gdam_recorder_filter_set_recording      (GdamRecorderFilter* filter, 
                                              gboolean            is_rec);
void gdam_recorder_filter_set_save_end_buffer(GdamRecorderFilter* filter,
	                                      gboolean            should_save);
void gdam_recorder_filter_capture_buffer     (GdamRecorderFilter* recorder,
                                              gboolean            drain);


/* Internal use only. */
void gdam_recorder_filter_new_length         (GdamRecorderFilter* recorder,
                                              int                 length);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
