/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamsplinemap.h"
static int gdam_spline_map_append_start_request(GdamFilter* filter,
				                int new_source_id,
				                GdamRequestBuilder* builder);

static int gdam_spline_map_append_start_request(GdamFilter* filter,
				                int new_source_id,
				                GdamRequestBuilder* builder)
{
  GdamSplineMap *map = GDAM_SPLINE_MAP (filter);
  GdamModel *model = GDAM_MODEL (filter);
  GdamRequest *request;
  gtk_object_ref (GTK_OBJECT (model));
  model->source_id = gdam_channel_new_local_id (model->channel);
  request = gdam_request_create_source ("GdasSplineMap",
					model->source_id,
					gdam_model_got_subsource,
					gdam_model_didnt_get_subsource,
					model,
					2,
					"spline",
					GDAM_ARG_TYPE_DOUBLE_ARRAY,
					map->num_points * 2,
					map->points,
					"filtered",
					GDAM_ARG_TYPE_INT, new_source_id);
  return gdam_request_builder_append_create (builder, request);
}

static void gdam_spline_map_init(GdamSplineMap* spline_map)
{
  spline_map->num_points = 2;
  spline_map->points = g_new (gdouble, 2 * 2);
  spline_map->points[0] = -1.0;
  spline_map->points[1] = +1.0;
  spline_map->points[2] = -1.0;
  spline_map->points[3] = +1.0;
}

static void gdam_spline_map_arg_set_spline(GtkObject* object, GdamArg* arg)
{
  GdamSplineMap *spline_map = GDAM_SPLINE_MAP (object);
  int source_id = GDAM_MODEL (object)->source_id;
  int length = GDAM_ARG_DOUBLE_ARRAY (arg).length;
  if (length % 2 != 0)
    {
      g_warning ("gdam_spline_map_arg_set_spline: "
		 "point array must have even length");
      return;
    }
  spline_map->points = g_memdup (GDAM_ARG_DOUBLE_ARRAY (arg).array,
				 sizeof (gdouble) * length);
  spline_map->num_points = length / 2;

  if (source_id != 0)
    {
      GdamRequest *request;
      GdamArg args[1];
      args[0].type = GDAM_ARG_TYPE_DOUBLE_ARRAY;
      args[0].name = "spline";
      GDAM_ARG_DOUBLE_ARRAY (args + 0).length = length;
      GDAM_ARG_DOUBLE_ARRAY (args + 0).array = spline_map->points;
      request = gdam_request_set_args (source_id, 1, args, NULL, NULL, NULL);
      gdam_channel_append_request (GDAM_MODEL (object)->channel, request);
    }
}
static void gdam_spline_map_arg_get_spline(GtkObject* object, GdamArg* arg)
{
  GdamSplineMap *map = GDAM_SPLINE_MAP (object);
  gdouble *array;
  int length = map->num_points * 2;
  GDAM_ARG_DOUBLE_ARRAY (arg).length = map->num_points * 2;
  array = g_memdup (map->points, sizeof (gdouble) * 2 * map->num_points);
  GDAM_ARG_DOUBLE_ARRAY (arg).array = array;
}

static void gdam_spline_map_class_init(GdamSplineMapClass* spline_map_class)
{
  GtkObjectClass *object_class;
  object_class = GTK_OBJECT_CLASS (spline_map_class);
  gdam_arg_register (object_class, "spline", GDAM_ARG_TYPE_DOUBLE_ARRAY,
		     gdam_spline_map_arg_set_spline,
		     gdam_spline_map_arg_get_spline, NULL, NULL,	/* bounds ? */
		     GDAM_ARG_VITAL);
}

GtkType gdam_spline_map_get_type()
{
  static GtkType spline_map_type = 0;
  if (!spline_map_type)
    {
      static const GtkTypeInfo spline_map_info = {
	"GdamSplineMap",
	sizeof (GdamSplineMap),
	sizeof (GdamSplineMapClass),
	(GtkClassInitFunc) gdam_spline_map_class_init,
	(GtkObjectInitFunc) gdam_spline_map_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_FILTER;
      spline_map_type = gtk_type_unique (parent, &spline_map_info);
    }
  return spline_map_type;
}
