/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_UBS_EVENT_H_
#define __GDAM_UBS_EVENT_H_

#include <gdam/common/gdamautoconf.h>

#if HAVE_LINUX_INPUT_H

typedef struct _GdamUsbHid GdamUsbHid;
typedef struct _GdamUsbHidClass GdamUsbHidClass;

#include "../gdaminputdevicefile.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  /* XXX: Range event here??? Don't export TYPE_USB_HID ?? */
GtkType gdam_usb_hid_get_type();
#define GDAM_TYPE_USB_HID			(gdam_usb_hid_get_type ())
#define GDAM_USB_HID(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_USB_HID, GdamUsbHid))
#define GDAM_USB_HID_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_USB_HID, GdamUsbHidClass))
#define GDAM_IS_USB_HID(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_USB_HID))
#define GDAM_IS_USB_HID_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_USB_HID))

/* There are two base types of usb hid events... the basic type is an
 * InputIntEvent, for keys, relative values, etc.   The other is
 * an InputEventRange, for absolute values.
 */
typedef struct _GdamUsbHidEvent GdamUsbHidEvent;
struct _GdamUsbHidEvent {
	GdamInputIntEvent	event;
	const char*		name;
	unsigned short 		type;
	unsigned short 		code;
	int 			value;
};

typedef struct _GdamUsbHidEventRange GdamUsbHidEventRange;
struct _GdamUsbHidEventRange {
	GdamInputEventRange	event;
	const char*		name;
	unsigned short 		type;
	unsigned short 		code;
	int 			value;
};



/* Different types of usb events... do we need to export these?? */
int gdam_usbhid_event_get_type();
int gdam_usbhid_event_range_get_type();
int gdam_usbhid_event_variable_get_type();
int gdam_usbhid_event_absolute_get_type();
int gdam_usbhid_event_key_get_type();

struct _GdamUsbHidClass {
	GdamInputDeviceFileClass		input_device_file_class;
};
struct _GdamUsbHid {
	GdamInputDeviceFile		input_device_file;
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif

#endif
