/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

/* gdamoutputdevice -- wrap the generic output device which
 *                     provides a file descriptor you can poll.
 */
#ifndef __GDAM_DEVICE_DUMMY_H_
#define __GDAM_DEVICE_DUMMY_H_


typedef struct _GdamDeviceDummy GdamDeviceDummy;
typedef struct _GdamDeviceDummyClass GdamDeviceDummyClass;
typedef struct _GdamDeviceDummyOptions GdamDeviceDummyOptions;

#include <gdam/server/gdamsink.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

GtkType gdam_device_dummy_get_type();
#define GDAM_TYPE_DEVICE_DUMMY                 (gdam_device_dummy_get_type ())
#define GDAM_DEVICE_DUMMY(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_DEVICE_DUMMY, GdamDeviceDummy))
#define GDAM_DEVICE_DUMMY_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_DEVICE_DUMMY, GdamDeviceDummyClass))
#define GDAM_IS_DEVICE_DUMMY(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_DEVICE_DUMMY))
#define GDAM_IS_DEVICE_DUMMY_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_DEVICE_DUMMY))

struct _GdamDeviceDummyClass {
        GdamSinkClass           sink_class;
};

struct _GdamDeviceDummyOptions {
        /* output sample rate / type */
        int             sampling_rate;
        int             num_channels;   /* must be 1,2,4 */
        int             bytes_per_sample;
        int             bits_per_sample;

        /* the block size of the device in samples. */
        int             block_size;

        /* used in subclass specific ways. */
        int             log_frag_size;
        int             num_fragments;

};


struct _GdamDeviceDummy {
  GdamSink              sink;

  /* options which control the output mechanism. */
  GdamDeviceDummyOptions options;

  /* how to reduce sound so it can be played. */
  int                   sample_size;

  /* number of bytes we must temporarily allocate to do
   * the sample type conversion. */
  int                   conversion_sample_size;

  int			total_sample_size;
  int			timeout_sub_id;
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
