/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

//#if HAVE_PTHREAD_H
#include <pthread.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include "gdamthreadeddevice.h"

static void gdam_threaded_device_init(GdamThreadedDevice* threaded_device)
{
}
static void* run_feed_device(void* threaded_dev)
{
	GdamThreadedDeviceClass* klass;
	klass = GDAM_THREADED_DEVICE_CLASS(GTK_OBJECT(threaded_dev)->klass);

	g_error("XXX: not implemented");
	return NULL;
}

static gboolean gdam_threaded_device_open_fd(GdamOutputDevice* output_device)
{
	GdamThreadedDeviceClass* klass;
	GdamThreadedDevice* threaded_dev;
	GtkObjectClass* object_class;
	pthread_t thread;

	threaded_dev = GDAM_THREADED_DEVICE(output_device);
	object_class = GTK_OBJECT(output_device)->klass;
	klass = GDAM_THREADED_DEVICE_CLASS(object_class);

	if (klass->open_device != NULL
	 && ! (*klass->open_device)(threaded_dev) ) {
		gdam_debug("%s::open_device failed",
			gtk_type_name(object_class->type));
		return FALSE;
	}

	if (pipe(threaded_dev->pipe_fds) < 0) {
		g_warning("error creating pipe: %s", strerror(errno));
		if (klass->close_device)
			(*klass->close_device)(threaded_dev);
		return FALSE;
	}

	if (pthread_create(&thread, NULL, run_feed_device, threaded_dev) < 0) {
		/* XXX: strerror(errno) on pthread_create??? */
		g_warning("pthread_create failed: %s", strerror(errno));
	}

	threaded_dev->pthread_handle = thread;
	output_device->device_fd = threaded_dev->pipe_fds[1];

	return TRUE;
}

static void gdam_threaded_device_close_fd(GdamOutputDevice* output_device)
{
	GdamThreadedDevice* threaded_device;
	void* return_value;
	threaded_device = GDAM_THREADED_DEVICE(output_device);

	g_return_if_fail(threaded_device->pipe_fds[1] >= 0);

	/* XXX: is this a correct check for a valid pthread_handle?? */
	g_return_if_fail(threaded_device->pthread_handle != 0);

	close(threaded_device->pipe_fds[1]);
	threaded_device->pipe_fds[1] = -1;

	/* XXX: we block here!!! XXX
	 *      we should ref count and add something to the main loop... */
	if (pthread_join(threaded_device->pthread_handle, &return_value) < 0) {
		g_warning("pthread_join failed: %s", strerror(errno));
	}
	threaded_device->pthread_handle = 0;
	output_device->device_fd = -1;
}

static void gdam_threaded_device_class_init(GdamThreadedDeviceClass* threaded_device_class)
{
}

GtkType gdam_threaded_device_get_type()
{
	static GtkType threaded_device_type = 0;
	if (!threaded_device_type) {
		static const GtkTypeInfo threaded_device_info = {
			"GdamThreadedDevice",
			sizeof(GdamThreadedDevice),
			sizeof(GdamThreadedDeviceClass),
			(GtkClassInitFunc) gdam_threaded_device_class_init,
			(GtkObjectInitFunc) gdam_threaded_device_init,
			/* reserved_1 */ NULL,
			/* reserved_2 */ NULL,
			(GtkClassInitFunc) NULL
		};
		GtkType parent = GDAM_TYPE_OUTPUT_DEVICE;
		threaded_device_type = gtk_type_unique(parent,
		                                 &threaded_device_info);
	}
	return threaded_device_type;
}
//#endif
