/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_THREADED_DEVICE_H_
#define __GDAM_THREADED_DEVICE_H_


typedef struct _GdamThreadedDevice GdamThreadedDevice;
typedef struct _GdamThreadedDeviceClass GdamThreadedDeviceClass;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "../gdamoutputdevice.h"

GtkType gdam_threaded_device_get_type();
#define GDAM_TYPE_THREADED_DEVICE			(gdam_threaded_device_get_type ())
#define GDAM_THREADED_DEVICE(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_THREADED_DEVICE, GdamThreadedDevice))
#define GDAM_THREADED_DEVICE_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_THREADED_DEVICE, GdamThreadedDeviceClass))
#define GDAM_IS_THREADED_DEVICE(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_THREADED_DEVICE))
#define GDAM_IS_THREADED_DEVICE_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_THREADED_DEVICE))

struct _GdamThreadedDeviceClass {
	GdamOutputDeviceClass		output_device_class;

	/* Start to open the device.
	 * This will be done in the parent thread. */
	gboolean		(*open_device)(GdamThreadedDevice*);

	/* This command is always executed in a subprocess. */
	void			(*feed_device)(GdamThreadedDevice*);

	/* Also done in the parent-thread: close the device. */
	gboolean		(*close_device)(GdamThreadedDevice*);
};
struct _GdamThreadedDevice {
	GdamOutputDevice	output_device;

	int			pipe_fds[2];
	guint			pthread_handle;

	/* Termination protocol: the child should check
	 * child_close_requested whenever it can, and if it
	 * is set, call `child_close_acknowledged'
	 * and exit immediately. 
	 *
	 * consequences:
	 *    - shutdown may block normally,
	 *    - to exit the program, you should set child_close_requested,
	 *      close the write end of the pipe, then pthread_join.
	 *    - a derived-class which blocks in feed-device forever may prevent
	 *      exit.  ("fix" this with a timeout!!!)
	 */
	gboolean		child_close_requested;
	gboolean		child_close_acknowledged;
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
