/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamcircularbuffer.h"

GdamCircularBuffer* gdam_circular_buffer_new   (int buffer_size)
{
  GdamCircularBuffer *buffer = g_new (GdamCircularBuffer, 1);
  buffer->buffer_size = buffer_size;
  buffer->buffer = g_new (GdamSample, buffer_size);
  buffer->read_index = buffer->amount_buffered = 0;
  return buffer;
}
void                gdam_circular_buffer_resize(GdamCircularBuffer* buf,
                                                int new_buffer_size,
						gboolean filled)
{
  GdamSample *new_buf = g_new (GdamSample, new_buffer_size);
  int initial = gdam_circular_buffer_read (buf, new_buf, new_buffer_size);
  gdam_sample_zero (new_buf + initial, new_buffer_size - initial);
  g_free (buf->buffer);
  buf->buffer = new_buf;
  buf->buffer_size = new_buffer_size;
  buf->read_index = 0;
  buf->amount_buffered = filled ? new_buffer_size : initial;
}

#if 0
void                gdam_circular_buffer_append(GdamCircularBuffer* buf,
                                                const GdamSample*   data,
						int               data_length)
{
}
#endif


int                 gdam_circular_buffer_read  (GdamCircularBuffer* buf,
                                                GdamSample*         data,
						int               data_length)
{
  if (data_length > buf->amount_buffered)
    data_length = buf->amount_buffered;
  if (data_length + buf->read_index >= buf->buffer_size)
    {
      int piece_1_length = buf->buffer_size - buf->read_index;
      gdam_sample_copy (data, buf->buffer + buf->read_index, piece_1_length);
      gdam_sample_copy (data + piece_1_length,
			buf->buffer, data_length - piece_1_length);
      buf->read_index = data_length - piece_1_length;
    }
  else
    {
      gdam_sample_copy (data, buf->buffer + buf->read_index, data_length);
      buf->read_index += data_length;
    }
  buf->amount_buffered -= data_length;
  return data_length;
}
#if 0
int                 gdam_circular_buffer_peek  (GdamCircularBuffer* buf,
                                                GdamSample*         data,
						int               data_length)
{
}
#endif

void gdam_circular_buffer_construct(GdamCircularBuffer* buffer)
{
  memset (buffer, 0, sizeof (GdamCircularBuffer));
}

void gdam_circular_buffer_destruct(GdamCircularBuffer* buffer)
{
  g_free (buffer->buffer);
  buffer->buffer = NULL;
  buffer->buffer_size = buffer->read_index = buffer->amount_buffered = 0;
}


#if !G_CAN_INLINE
/* A few functions are inlined for efficiency... */
G_INLINE_FUNC GdamSample
gdam_circular_buffer_read_one(GdamCircularBuffer* buf)
{
  GdamSample rv;
  int read_index = buf->read_index;
  g_assert (buf->amount_buffered > 0);
  rv = buf->buffer[read_index];
  if (++read_index == buf->buffer_size)
    read_index = 0;
  buf->read_index = read_index;
  buf->amount_buffered--;
  return rv;
}
G_INLINE_FUNC GdamSample
gdam_circular_buffer_peek_one(GdamCircularBuffer* buf)
{
  g_assert (buf->amount_buffered > 0);
  return buf->buffer[buf->read_index];
}
G_INLINE_FUNC void
gdam_circular_buffer_append_one(GdamCircularBuffer* buf, GdamSample sample)
{
  if (buf->amount_buffered == buf->buffer_size)
    {
      buf->buffer[buf->read_index++] = sample;
      if (buf->read_index == buf->buffer_size)
	buf->read_index = 0;
    }
  else
    {
      int write_index = buf->read_index + buf->amount_buffered;
      write_index %= buf->buffer_size;
      buf->buffer[write_index] = sample;
      buf->amount_buffered++;
    }
}
#endif

