/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamnamespace.h"
#include "gdamdispatch.h"

static GHashTable* gdam_int_namespace = NULL;

gboolean gdam_namespace_int_lookup(const char* our_namespace,
                                   const char* name,
				   int* rv)
{
  gpointer val;
  gpointer the_ns;

  if (!gdam_int_namespace)
    return FALSE;
  if ((the_ns =
       ((GHashTable *) g_hash_table_lookup (gdam_int_namespace,
					    our_namespace))) == NULL)
    return FALSE;
  if (g_hash_table_lookup_extended ((GHashTable *) the_ns, name, NULL, &val))
    {
      *rv = GPOINTER_TO_INT (val);
      return TRUE;
    }
  return FALSE;
}

void   gdam_namespace_int_set(const char* our_namespace,
                                   const char* name,
				   int rv)
{
  GHashTable *ns;
  if (!gdam_int_namespace)
    gdam_int_namespace = g_hash_table_new (g_str_hash, g_str_equal);

  ns = (GHashTable *) g_hash_table_lookup (gdam_int_namespace, our_namespace);
  if (!ns)
    g_hash_table_insert (gdam_int_namespace,
			 g_strdup (our_namespace),
			 ns = g_hash_table_new (g_str_hash, g_str_equal));
  if (!g_hash_table_lookup_extended (ns, name, NULL, NULL))
    {
      g_hash_table_insert (ns, g_strdup (name), GINT_TO_POINTER (rv));
    }
  else
    {
      g_hash_table_insert (ns, (gpointer) name, GINT_TO_POINTER (rv));
    }
}

static GHashTable* local_source_by_id = NULL;
static GHashTable* local_sink_by_id = NULL;
static int local_source_id = -1;
static int local_sink_id = -1;

static gboolean unref_remove(gpointer key, gpointer value, gpointer user_data)
{
  gtk_object_unref (GTK_OBJECT (value));
  return TRUE;
}

void gdam_namespace_local_clean()
{
  if (!local_source_by_id)
    {
      local_source_by_id = g_hash_table_new (NULL, NULL);
      local_sink_by_id = g_hash_table_new (NULL, NULL);
    }
  else
    {
      g_hash_table_foreach_remove (local_source_by_id, unref_remove, NULL);
      g_hash_table_foreach_remove (local_sink_by_id, unref_remove, NULL);
    }
  local_source_id = 0;
  local_sink_id = 0;
}


GdasSource*  gdam_namespace_local_source(int source_id)
{
  gpointer rv = g_hash_table_lookup (local_source_by_id,
				     GINT_TO_POINTER (source_id));
  if (!rv)
    {
      rv = gdam_dispatch_get_source (source_id);
      if (!rv)
	return NULL;
      g_hash_table_insert (local_source_by_id,
			   GINT_TO_POINTER (source_id), rv);
    }
  return GDAS_SOURCE (rv);
}
GdamSink* gdam_namespace_local_sink(int sink_id)
{
  gpointer rv = g_hash_table_lookup (local_sink_by_id,
				     GINT_TO_POINTER (sink_id));
  if (!rv)
    {
      rv = gdam_dispatch_get_sink (sink_id);
      if (!rv)
	return NULL;
      g_hash_table_insert (local_sink_by_id, GINT_TO_POINTER (sink_id), rv);
    }
  return GDAM_SINK (rv);
}

int gdam_namespace_alloc_local_source(GdasSource* source)
{
  int rv = --local_source_id;
  g_assert (rv < 0);		/* if fails, you need
				 * to be calling "clean" first.. */
  gtk_object_ref (GTK_OBJECT (source));
  g_hash_table_insert (local_source_by_id, GINT_TO_POINTER (rv), source);
  return rv;
}


int gdam_namespace_alloc_local_sink(GdamSink* sink)
{
  int rv = --local_sink_id;
  gtk_object_ref (GTK_OBJECT (sink));
  g_hash_table_insert (local_sink_by_id, GINT_TO_POINTER (rv), sink);
  return rv;
}
