/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamserverobject.h"
#include "gdamdispatch.h"
static GtkObjectClass* parent_class = NULL;

static void gdam_server_object_init(GdamServerObject* server_object)
{
  server_object->global_id = gdam_dispatch_obtain_global_id (server_object);
}
static void gdam_server_object_destroy(GtkObject* object)
{
  GdamServerObject *server_object;
  server_object = GDAM_SERVER_OBJECT (object);
  if (server_object->connection != NULL)
    {
      g_hash_table_remove (server_object->connection->local_id_table,
			   GINT_TO_POINTER (server_object->local_id));
      server_object->connection = NULL;
      server_object->local_id = 0;
    }
  if (server_object->global_id != 0)
    {
      gdam_dispatch_release_global_id (server_object->global_id);
      server_object->global_id = 0;
    }
  (*parent_class->destroy) (object);
}

static void gdam_server_object_class_init(GdamServerObjectClass* klass)
{
  GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);
  object_class->destroy = gdam_server_object_destroy;
}

GtkType gdam_server_object_get_type()
{
  static GtkType server_object_type = 0;
  if (!server_object_type)
    {
      static const GtkTypeInfo server_object_info = {
	"GdamServerObject",
	sizeof (GdamServerObject),
	sizeof (GdamServerObjectClass),
	(GtkClassInitFunc) gdam_server_object_class_init,
	(GtkObjectInitFunc) gdam_server_object_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GTK_TYPE_OBJECT;
      parent_class = gtk_type_class (parent);
      server_object_type = gtk_type_unique (parent, &server_object_info);
    }
  return server_object_type;
}
