/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdasmodulator.h"
#include <gdam/server/gdamconnection.h>

static void gdas_modulator_init(GdasMix* mix)
{
}

/* XXX: this function needs a C level interface. */
static void gdas_modulator_arg_set_filtered(GtkObject* object, GdamArg* arg)
{
  GdasSource *source;
  GdasMix *mix = GDAS_MIX (object);
  GdasMixElement *element;
  int source_id = GDAM_ARG_INT (arg);
  if (source_id != 0)
    {
      source = gdam_connection_get_source (NULL, source_id);
      if (source == NULL)
	{
	  gdam_debug ("modulator_set_filtered: "
		      "no source of id %d", source_id);
	  return;
	}
    }
  else
    {
      source = NULL;
    }
  element = gdas_mix_add_named (mix, source, -1, "filtered");
  if (element)
    element->remove_when_ended = TRUE;
  if (source != NULL)
    gtk_object_unref ((GtkObject *) source);
}
static void gdas_modulator_arg_get_filtered(GtkObject* object, GdamArg* arg)
{
  GdasSource *filtered;
  filtered = gdas_mix_get_named (GDAS_MIX (object), "filtered");
  GDAM_ARG_INT (arg) = (filtered != NULL)
    ? GDAM_SERVER_OBJECT (filtered)->global_id : 0;
}

static void gdas_modulator_arg_set_modulator(GtkObject* object, GdamArg* arg)
{
  GdasSource *source;
  GdasMix *mix = GDAS_MIX (object);
  int source_id = GDAM_ARG_INT (arg);
  GdasMixElement *element;
  if (source_id != 0)
    {
      source = gdam_connection_get_source (NULL, source_id);
      if (source == NULL)
	{
	  gdam_debug ("modulator_set_modulator: "
		      "no source of id %d", source_id);
	  return;
	}
    }
  else
    {
      source = NULL;
    }
  element = gdas_mix_add_named (mix, source, -1, "modulator");
  if (element)
    element->remove_when_ended = TRUE;
  if (source != NULL)
    gtk_object_unref ((GtkObject *) source);
}
static void gdas_modulator_arg_get_modulator(GtkObject* object, GdamArg* arg)
{
  GdasSource *mod;
  mod = gdas_mix_get_named (GDAS_MIX (object), "modulator");
  GDAM_ARG_INT (arg) = mod ? GDAM_SERVER_OBJECT (mod)->global_id : 0;
}
static void modulate_samples(GdasMix*          mix,
                             GdamSample*       dest,
			     const GdamSample* source,
			     int               num_samples)
{
  gdam_sample_modulate (dest, source, num_samples);
}

static void gdas_modulator_class_init(GdasMixClass* mix_class)
{
  GtkObjectClass *object_class = GTK_OBJECT_CLASS (mix_class);
  mix_class->add_samples = modulate_samples;
  gdam_arg_register (object_class,
		     "filtered",
		     GDAM_ARG_TYPE_INT,
		     gdas_modulator_arg_set_filtered,
		     gdas_modulator_arg_get_filtered,
		     NULL, NULL, GDAM_ARG_VITAL);
  gdam_arg_register (object_class,
		     "modulator",
		     GDAM_ARG_TYPE_INT,
		     gdas_modulator_arg_set_modulator,
		     gdas_modulator_arg_get_modulator,
		     NULL, NULL, GDAM_ARG_VITAL);
}

GtkType gdas_modulator_get_type()
{
  static GtkType modulator_type = 0;
  if (!modulator_type)
    {
      static const GtkTypeInfo modulator_info = {
	"GdasModulator",
	sizeof (GdasMix),
	sizeof (GdasMixClass),
	(GtkClassInitFunc) gdas_modulator_class_init,
	(GtkObjectInitFunc) gdas_modulator_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      modulator_type = gtk_type_unique (GDAS_TYPE_MIX, &modulator_info);
    }
  return modulator_type;
}
