/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAS_DYNAMIC_RANGE_H_
#define __GDAS_DYNAMIC_RANGE_H_


/*
 * Virtual base class.
 *
 * A dynamic range source is one that tracks the 
 * maximum volume and attempts to adjust the 
 * buffer to compensate in some way.
 *
 * Usually sources should linearly interpolate from
 * `last_max_volume' to `max_volume' to make
 * the effect smooth.
 *
 * example: GdasCompressor
 */

typedef struct _GdasDynamicRange GdasDynamicRange;
typedef struct _GdasDynamicRangeClass GdasDynamicRangeClass;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "../gdasblockfilter.h"

GtkType gdas_dynamic_range_get_type();
#define GDAS_TYPE_DYNAMIC_RANGE			(gdas_dynamic_range_get_type ())
#define GDAS_DYNAMIC_RANGE(obj)              (GTK_CHECK_CAST ((obj), GDAS_TYPE_DYNAMIC_RANGE, GdasDynamicRange))
#define GDAS_DYNAMIC_RANGE_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAS_TYPE_DYNAMIC_RANGE, GdasDynamicRangeClass))
#define GDAM_IS_DYNAMIC_RANGE(obj)           (GTK_CHECK_TYPE ((obj), GDAS_TYPE_DYNAMIC_RANGE))
#define GDAM_IS_DYNAMIC_RANGE_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAS_TYPE_DYNAMIC_RANGE))

struct _GdasDynamicRangeClass {
	GdasBlockFilterClass	block_filter_class;
	void                  (*process)(GdasDynamicRange*  range,
	                                 GdamSample*        out,
					 const GdamSample*  in,
					 int                num_samples);
};
struct _GdasDynamicRange {
	GdasBlockFilter		block_filter;

	GdamSampleValue		max_sample;
	GdamSampleValue		last_max_sample;
	gboolean		first_time;
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
