/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAS_EQUALIZER_H_
#define __GDAS_EQUALIZER_H_

#include <gdam/server/gdasfilter.h>

GtkType gdas_equalizer_get_type();
#define GDAS_TYPE_EQUALIZER   gdas_equalizer_get_type()
#define GDAS_EQUALIZER(ptr) ((GdasEqualizer*) gtk_type_check_object_cast ((GtkTypeObject*) (ptr), GDAS_TYPE_EQUALIZER))

/*
 * Implements a simple audio filter,
 * as described in "The Audio-EQ Cookbook",
 * by Robert Bristow-Johnson, pbjrbj@viconet.com.
 * which is widely circulated on the web.
 */

typedef struct _GdasEqualizer GdasEqualizer;

typedef enum {
	GDAS_EQUALIZER_LOW_PASS = 300,
	GDAS_EQUALIZER_HIGH_PASS,
	GDAS_EQUALIZER_BAND_PASS,
	GDAS_EQUALIZER_NOTCH,
	GDAS_EQUALIZER_PEAKING_EQ,
	GDAS_EQUALIZER_LOW_SHELF,
	GDAS_EQUALIZER_HIGH_SHELF
} GdasEqualizerMode;


struct _GdasEqualizer {
	GdasFilter			filter;

	/* y[n] = x_coeff[0] * x[n] 
	        + x_coeff[1] * x[n-1] 
	        + x_coeff[2] * x[n-2] 
	        + y_coeff[1] * y[n-1] 
	        + y_coeff[2] * y[n-2]           [where x is input, y is output]
	 */
	GdamFixed			x_coeff[3];
	GdamFixed			y_coeff[3];
	GdamSample			x[2];
	GdamSample			y[2];

	int				effect;

	gdouble                         frequency;
	gdouble				bandwidth;
	gdouble				db_gain;

	GdasEqualizerMode		type;
};

#endif
