/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAS_FLANGE_H_
#define __GDAS_FLANGE_H_

#include "../gdasbufferingfilter.h"
#include "../gdasecho.h"

typedef struct _GdasFlange GdasFlange;
typedef struct _GdasFlangeClass GdasFlangeClass;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

GtkType gdas_flange_get_type();
#define GDAS_TYPE_FLANGE			(gdas_flange_get_type ())
#define GDAS_FLANGE(obj)              (GTK_CHECK_CAST ((obj), GDAS_TYPE_FLANGE, GdasFlange))
#define GDAS_FLANGE_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAS_TYPE_FLANGE, GdasFlangeClass))
#define GDAM_IS_FLANGE(obj)           (GTK_CHECK_TYPE ((obj), GDAS_TYPE_FLANGE))
#define GDAM_IS_FLANGE_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAS_TYPE_FLANGE))

struct _GdasFlangeClass {
	GdasBufferingFilterClass	buffering_filter_class;
};
struct _GdasFlange {
	GdasBufferingFilter		buffering_filter;

	/* At each step, multiply the echo by a number
	 * between MIN_ECHO_FACTOR and MIN_ECHO_FACTOR+DELTA_ECHO_FACTOR,
	 * which are linearly mapped from samples
	 * from modulator. */
	GdasSource*			modulator;

	GdamFixed			min_echo_factor;
	GdamFixed			delta_echo_factor;

	gboolean			chorus;

	GdamFixed			effect;

	int				num_echoes;
	GdasEchoElement*		echoes;
	int				max_echo_delay;
};

void gdas_flange_set_modulator(GdasFlange* flange, GdasSource* mod);
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
