/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAS_GRANULAR_FILTER_H_
#define __GDAS_GRANULAR_FILTER_H_


typedef enum {
	GDAS_GRANULAR_FILTER_POISSON,
	GDAS_GRANULAR_FILTER_GAUSSIAN,
	GDAS_GRANULAR_FILTER_FLAT,
} GdasGranularFilterMode;

typedef struct _GdasGranularFilter GdasGranularFilter;
typedef struct _GdasGranularFilterClass GdasGranularFilterClass;
typedef struct _GdasGranuleState GdasGranuleState;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "../gdasfilter.h"
#include "../gdamcachebuffer.h"

GtkType gdas_granular_filter_get_type();
#define GDAS_TYPE_GRANULAR_FILTER			(gdas_granular_filter_get_type ())
#define GDAS_GRANULAR_FILTER(obj)              (GTK_CHECK_CAST ((obj), GDAS_TYPE_GRANULAR_FILTER, GdasGranularFilter))
#define GDAS_GRANULAR_FILTER_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAS_TYPE_GRANULAR_FILTER, GdasGranularFilterClass))
#define GDAS_IS_GRANULAR_FILTER(obj)           (GTK_CHECK_TYPE ((obj), GDAS_TYPE_GRANULAR_FILTER))
#define GDAS_IS_GRANULAR_FILTER_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAS_TYPE_GRANULAR_FILTER))

/*
 * GdasGranularFilter
 *
 *     A filter which transfers the sound according to an
 *     envelope which a sum of "grains", each consisting
 *     of an attack and a decay, and random distributed
 *     (either according to a poisson or gaussian distribution).
 *
 *
 * Parameters:
 *     attack_time
 *           The onset time of a granule in seconds.
 *     decay_time
 *           The decay time of a granule in seconds.
 *     mean_grain_distance
 *           The mean time between the start of two granules in seconds.
 *     grain_volume
 *           The maximum volume of a grain.
 *     is_poisson
 *           Use poisson (v. gaussian) distibutions
 *     clamp
 *           If the grain volumes add up to more than 1, clamp to 1.
 */

typedef struct _GdasGranule GdasGranule;

struct _GdasGranularFilterClass {
	GdasFilterClass		filter_class;
};
struct _GdasGranule {
	GdamCacheBuffer*	cache_buffer;
	int			num_time_vols;
	gint64*			time_vols;
	/* TODO: finish me! */
};
struct _GdasGranularFilter {
	GdasFilter		filter;

	/* distribution of the granules */
	GdasGranularFilterMode  distribution;
	int			mean_granule_dist;
	int			stddev_granule_dist;

	int			next_granule_dist;
	GdasGranuleState*	granules;

	gboolean		clamp;

	GdamSampleValue		grain_volume;
	GdamSampleValue		attack_rate;
	guint32			attack_rate_frac;
	GdamSampleValue		decay_rate;
	guint32			decay_rate_frac;
	guint32			fraction;
	gfloat			last_attack_time;
	gfloat			last_decay_time;

	int			num_grains;
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
