/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAS_LADSPA_PLUGIN_H_
#define __GDAS_LADSPA_PLUGIN_H_

#include <gdam/common/gdamautoconf.h>

#if HAVE_LADSPA_H


/*
 * Some general comments.
 *
 * Ladspa categorizes each port in two ways:
 *
 *        input/output         -- whether data flows in to or out of 
 *                                the plugin via this port.
 *        audio_data/control   -- whether the ports value changes
 *                                every sample or every block.
 *
 * We use slightly different terminology
 * everywhere but gdam_ladspa_descriptor_get, which does
 * the translation:
 *
 *              +---------+--------+
 *              | Input   | Output |
 * +------------+---------+--------+
 * | Control    | Control | Meter  |
 * +------------+---------+--------+
 * | Audio_data | Input   | Output |
 * +------------+---------+--------+
 *
 * That is, we just divide each port into one
 * of four categories:
 *
 *	input
 *	output
 *	control
 *	meter
 *
 * So please, try not to get confused -- we will *always* use
 * our terminology in comments and so on.
 */

#include "../gdassource.h"


typedef struct _GdasLadspaPlugin GdasLadspaPlugin;
typedef struct _GdasLadspaPluginClass GdasLadspaPluginClass;
typedef struct _GdamLadspaInput GdamLadspaInput;
typedef struct _GdamLadspaDescriptor GdamLadspaDescriptor;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

GtkType gdas_ladspa_plugin_get_type();
#define GDAS_TYPE_LADSPA_PLUGIN			(gdas_ladspa_plugin_get_type ())
#define GDAS_LADSPA_PLUGIN(obj)              (GTK_CHECK_CAST ((obj), GDAS_TYPE_LADSPA_PLUGIN, GdasLadspaPlugin))
#define GDAS_LADSPA_PLUGIN_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAS_TYPE_LADSPA_PLUGIN, GdasLadspaPluginClass))
#define GDAM_IS_LADSPA_PLUGIN(obj)           (GTK_CHECK_TYPE ((obj), GDAS_TYPE_LADSPA_PLUGIN))
#define GDAM_IS_LADSPA_PLUGIN_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAS_TYPE_LADSPA_PLUGIN))

struct _GdasLadspaPluginClass {
	GdasSourceClass		source_class;
};

struct _GdasLadspaPlugin {
	GdasSource		source;

	GdamLadspaDescriptor*	descriptor;

	gpointer		left_handle;
	gpointer		right_handle;

	/* whether the plugin has two inputs, so that the
	 * right_handle == NULL and is unused... */
	gboolean		native_stereo;

	/* Array of inputs. */
	GdamLadspaInput*	inputs;

	/* If more data than this is required, multiple processing
	 * phases must be used. */
	int			buffer_size;

	gfloat*			left_controls;
	gfloat*			right_controls;

	/* These are "control" outputs. */
	gfloat*			left_meters;
	gfloat*			right_meters;

	gfloat**		left_output_lines;
	gfloat**		right_output_lines;

	int			which_left_output_to_use;
	int			which_right_output_to_use;
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif

#endif
