/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAS_MINI_NETWORK_H_
#define __GDAS_MINI_NETWORK_H_


typedef struct _GdasMiniNetwork GdasMiniNetwork;
typedef struct _GdasMiniNetworkClass GdasMiniNetworkClass;
typedef struct _GdasMiniNetworkPipe GdasMiniNetworkPipe;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/* Used to make a trivial *very* highpass filter to prevent
 * pure dc signals. */
#define GDAS_MINI_NETWORK_DC_BLOCK_LENGTH	1024

#include "../gdasfilter.h"

/*
 * GdasMiniNetworkNode
 */
typedef struct _GdasMiniNetworkNode GdasMiniNetworkNode;
struct _GdasMiniNetworkNode
{
  guint           node_id;
  const char     *class_name;
  char           *serialized;
  int             num_inputs;
  void          (*disconnect)(GdasMiniNetworkNode* node,
			      guint                connect_id);

  /* Returns index into the input parallel arrays... */
  gint          (*connect)   (GdasMiniNetworkNode *node,
                              const char          *input_name,
                              GdasMiniNetworkNode *new_input);
  void          (*update)    (GdasMiniNetworkNode *node);
  void          (*destroy)   (GdasMiniNetworkNode *node);

  void          (*set_params)(GdasMiniNetworkNode *node,
                              const char          *description);

  gdouble         last_output[2];
  gdouble         output[2];

  /* Which inputs we need data from. */
  guint8                 *needed_inputs;
  GdasMiniNetworkNode   **input_nodes;
  guint                  *pipe_ids;

  GSList                 *output_pipes;

  /* Used when traversing the tree... */
  gboolean                has_traversed;
};


typedef GdasMiniNetworkNode* 
                   (*GdasMiniNetworkNodeMaker)(const char* options,
                                               gpointer    maker_data);
                
void gdas_mini_network_node_type_register
                        (const char*               node_type,
                         GdasMiniNetworkNodeMaker  maker,
                         gpointer                  maker_data);


GtkType gdas_mini_network_get_type();
#define GDAS_TYPE_MINI_NETWORK                  (gdas_mini_network_get_type ())
#define GDAS_MINI_NETWORK(obj)              (GTK_CHECK_CAST ((obj), GDAS_TYPE_MINI_NETWORK, GdasMiniNetwork))
#define GDAS_MINI_NETWORK_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAS_TYPE_MINI_NETWORK, GdasMiniNetworkClass))
#define GDAM_IS_MINI_NETWORK(obj)           (GTK_CHECK_TYPE ((obj), GDAS_TYPE_MINI_NETWORK))
#define GDAM_IS_MINI_NETWORK_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAS_TYPE_MINI_NETWORK))

/*
 *   ___    _            __  __ _      _ _  _     _                  _
 *  / __|__| |__ _ _ __ |  \/  (_)_ _ (_) \| |___| |___ __ _____ _ _| |__
 * | (_ / _` / _` | '  \| |\/| | | ' \| | .` / -_)  _\ V  V / _ \ '_| / /
 *  \___\__,_\__,_|_|_|_|_|  |_|_|_||_|_|_|\_\___|\__|\_/\_/\___/_| |_\_\
 * 
 */
struct _GdasMiniNetworkClass
{
  GdasFilterClass                 filter_class;
};
struct _GdasMiniNetwork
{
  GdasFilter                      filter;

  GdasMiniNetworkNode            *input;
  GdasMiniNetworkNode            *output;

  GSList                         *all_nodes;
  GSList                         *eval_order;
  guint                           last_node_id;
  guint                           last_pipe_id;


  /* map from pipe_id => input_node */
  GHashTable                     *pipe_input;

  /* whether the nodes can be evaluated.  for example,
   * a zero delay loop cannot be evaluated in this
   * architecture. */
  gboolean                        can_flow;

  gboolean                        dc_block;
  int                             dc_block_index;
  int                             left_buffer_sum;
  int                             right_buffer_sum;
  GdamSample                     *dc_block_buffer;
};

/*          unneeded it turns out...
struct _GdasMiniNetworkPipe {
        guint                           pipe_id;
        GdasMiniNetworkNode*            input;
        GdasMiniNetworkNode*            output;
};
*/


guint gdas_mini_network_add_node(GdasMiniNetwork    *network,
                                 const char         *type_name,
                                 const char         *options);
guint gdas_mini_network_add_pipe(GdasMiniNetwork    *network,
                                 guint               dest,
                                 const char         *input_name,
                                 guint               src);
void gdas_mini_network_remove_pipe(GdasMiniNetwork   *network,
                                   guint              pipe_id);
void gdas_mini_network_remove_node(GdasMiniNetwork   *network,
                                   guint              node_id);

/* Private. */
gboolean  gdas_mini_network_recompute_eval_order(GdasMiniNetwork*);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
