/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdasoctavedown.h"
#include <gdam/common/gdamplugin.h>

static void gdas_octave_down_init(GdasOctaveDown* octave_down)
{
}

static int gdas_octave_down_read(GdasSource* source, GdamSample* buffer, int n)
{
  int num_read;

  /* Were the last left/right samples negative. */
  gboolean last_r_neg, last_l_neg;

  /* Are we negating the left and right channel. */
  gboolean invert_r, invert_l;

  int i;

  GdasOctaveDown *down;

  down = GDAS_OCTAVE_DOWN (source);

  num_read = gdas_filter_sub_read (GDAS_FILTER (source), buffer, n);
  if (num_read <= 0)
    return num_read;

  last_l_neg = down->last_l_neg;
  last_r_neg = down->last_r_neg;
  invert_l = down->invert_l;
  invert_r = down->invert_r;
  for (i = 0; i < num_read; i++)
    {
      gboolean is_l_neg = (buffer[i].s[0] < 0);
      gboolean is_r_neg = (buffer[i].s[1] < 0);
      if (last_l_neg && !is_l_neg)
	invert_l ^= 1;
      if (last_r_neg && !is_r_neg)
	invert_r ^= 1;
      if (invert_l)
	buffer[i].s[0] = -buffer[i].s[0];
      if (invert_r)
	buffer[i].s[1] = -buffer[i].s[1];
      last_l_neg = is_l_neg;
      last_r_neg = is_r_neg;
    }
  down->last_l_neg = last_l_neg;
  down->last_r_neg = last_r_neg;
  down->invert_l = invert_l;
  down->invert_r = invert_r;
  return num_read;
}

static void gdas_octave_down_class_init(GdasOctaveDownClass* klass)
{
  GDAS_SOURCE_CLASS (klass)->read = gdas_octave_down_read;
}

GtkType gdas_octave_down_get_type()
{
  static GtkType octave_down_type = 0;
  if (!octave_down_type)
    {
      static const GtkTypeInfo octave_down_info = {
	"GdasOctaveDown",
	sizeof (GdasOctaveDown),
	sizeof (GdasOctaveDownClass),
	(GtkClassInitFunc) gdas_octave_down_class_init,
	(GtkObjectInitFunc) gdas_octave_down_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAS_TYPE_FILTER;
      octave_down_type = gtk_type_unique (parent, &octave_down_info);
      gdam_plugin_check_version (__FILE__, VERSION);
    }
  return octave_down_type;
}
