/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAS_PLUCKED_STRING_H_
#define __GDAS_PLUCKED_STRING_H_


typedef struct _GdasPluckedString GdasPluckedString;
typedef struct _GdasPluckedStringClass GdasPluckedStringClass;

#include <gdam/server/gdasfilter.h>
#include <gdam/server/gdamcircularbuffer.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

GtkType gdas_plucked_string_get_type();
#define GDAS_TYPE_PLUCKED_STRING			(gdas_plucked_string_get_type ())
#define GDAS_PLUCKED_STRING(obj)              (GTK_CHECK_CAST ((obj), GDAS_TYPE_PLUCKED_STRING, GdasPluckedString))
#define GDAS_PLUCKED_STRING_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAS_TYPE_PLUCKED_STRING, GdasPluckedStringClass))
#define GDAM_IS_PLUCKED_STRING(obj)           (GTK_CHECK_TYPE ((obj), GDAS_TYPE_PLUCKED_STRING))
#define GDAM_IS_PLUCKED_STRING_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAS_TYPE_PLUCKED_STRING))

struct _GdasPluckedStringClass {
	GdasFilterClass		filter_class;
};
struct _GdasPluckedString {
	GdasFilter			filter;

	/* A circular buffer of length L makes a delay of
	 * L - 0.5, assuming you read, then write.
	 * Minimum length is 2. */
	GdamCircularBuffer		lowpass;

	/* All-Pass filter coefficient to make the remaining (fractional)
	 * delay up. */
	GdamFixed			a;

	/* Feedback-loop strength. */
	GdamFixed			gain;

	gdouble				hertz;

	GdamFixed			filter_coeff;

	GdamSample			last_allpass_in, last_allpass_out;
};

void gdas_plucked_string_set_hertz(GdasPluckedString*  string,
                                   gdouble             hertz);
void gdas_plucked_string_set_gain (GdasPluckedString*  string,
                                   gdouble             gain);
void gdas_plucked_string_pluck    (GdasPluckedString*  filter,
                                   gdouble             velocity);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
