/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAS_RECORDER_H_
#define __GDAS_RECORDER_H_


typedef struct _GdasRecorder GdasRecorder;
typedef struct _GdasRecorderClass GdasRecorderClass;
#include <gdam/server/gdasfilter.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "../gdamcachebuffer.h"

GtkType gdas_recorder_get_type();
#define GDAS_TYPE_RECORDER			(gdas_recorder_get_type ())
#define GDAS_RECORDER(obj)              (GTK_CHECK_CAST ((obj), GDAS_TYPE_RECORDER, GdasRecorder))
#define GDAS_RECORDER_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAS_TYPE_RECORDER, GdasRecorderClass))
#define GDAM_IS_RECORDER(obj)           (GTK_CHECK_TYPE ((obj), GDAS_TYPE_RECORDER))
#define GDAM_IS_RECORDER_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAS_TYPE_RECORDER))

struct _GdasRecorderClass {
	GdasFilterClass		recorder_class;
};
struct _GdasRecorder {
	GdasFilter		recorder;
	GdamBuffer		buffer;
	gboolean		is_recording;
};

/* Stop recording. */
void gdas_recorder_stop              (GdasRecorder*  recorder);

/* Start recording. */
void gdas_recorder_start             (GdasRecorder*  recorder);

/* Drain the buffer. */
void gdas_recorder_empty             (GdasRecorder*  recorder);

/* Make a buffer out of the source, optionally removing the 
 * data from this buffer at the same time. */
GdamCacheBuffer* gdas_recorder_make_buffer
                                     (GdasRecorder*  recorder,
                                      gboolean       drain);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
