/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAS_RESPEED_H_
#define __GDAS_RESPEED_H_

typedef struct _GdamBlockSourceClass GdamBlockSourceClass;
typedef struct _GdamBlockSource GdamBlockSource;

#include <gdam/server/gdasblockfilter.h>

typedef struct _GdasRespeedClass GdasRespeedClass;
typedef struct _GdasRespeed GdasRespeed;

GtkType gdas_respeed_get_type();
#define GDAS_TYPE_RESPEED   gdas_respeed_get_type()
#define GDAS_RESPEED(ptr) ((GdasRespeed*) gtk_type_check_object_cast ((GtkTypeObject*) (ptr), GDAS_TYPE_RESPEED))


/* Algorithm:

     - we play two tracks:
     		the old track:  beginning at the start of the 
                                output buffer, and replaying
				the back buffer + 
				replay_amount of the input buffer
		the new track:  beginning as early as possible,
		                except clipped at the beginning
				if we are speeding up.


     - they will fade linearly over the course of their intersection. */

struct _GdasRespeedClass {
	GdasBlockFilterClass	block_filter_class;
};

struct _GdasRespeed {
	GdasBlockFilter		block_filter;

	/* Copied from the last samples into this buffer to
	 * fade the margin. */
	GdamSample*		back_buffer;

	/* The length of the back buffer.  At each iteration,
	 * we will copy this to the front of the buffer
	 * fading as needed. */
	int			back_buffer_size;

	/* How much of the input to play twice (fading out it
	 * itself). */
	int			replay_amount;

	/* When true, we use a smoother function than a linear cut. */
	unsigned		smoothing : 1;

	/* The input_block_size, maintained separately from rate 
	 * for convenience... */
	int			input_size;

};

void     gdas_respeed_set_rate         (GdasRespeed*      respeed,
                                        double            rate);

#endif
