#ifndef __GDAS_XMMS_INPUT_H_
#define __GDAS_XMMS_INPUT_H_

typedef struct _GdasXmmsInput GdasXmmsInput;
typedef struct _GdasXmmsInputClass GdasXmmsInputClass;
typedef struct _GdasXmmsSampleBuffer GdasXmmsSampleBuffer;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "../gdassource.h"
#include <gdam/common/gdamsampleconverter.h>

GtkType gdas_xmms_input_get_type();
#define GDAS_TYPE_XMMS_INPUT			(gdas_xmms_input_get_type ())
#define GDAS_XMMS_INPUT(obj)              (GTK_CHECK_CAST ((obj), GDAS_TYPE_XMMS_INPUT, GdasXmmsInput))
#define GDAS_XMMS_INPUT_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAS_TYPE_XMMS_INPUT, GdasXmmsInputClass))
#define GDAS_XMMS_INPUT_GET_CLASS(obj)    (GDAS_XMMS_INPUT_CLASS(GTK_OBJECT(obj)->klass))
#define GDAS_IS_XMMS_INPUT(obj)           (GTK_CHECK_TYPE ((obj), GDAS_TYPE_XMMS_INPUT))
#define GDAS_IS_XMMS_INPUT_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAS_TYPE_XMMS_INPUT))

struct _GdasXmmsInputClass
{
  GdasSourceClass	source_class;
};

struct _GdasXmmsSampleBuffer
{
  gint64                start;
  GdamBuffer            buffer;
};

struct _GdasXmmsInput
{
  GdasSource		source;

  int			fd;

  GPollFD		poll_fd;

  guint			gsource_id;

  char                 *plugin_type;

  GByteArray           *incoming_control;
  GdamBuffer            outgoing;
  GdasXmmsSampleBuffer  current;
  gint64                position;

  /* configuration that may be changed at any time. */
  int			buffer_size;

  int			xmms_helper_pid;

  char                 *plugin_name;

  /* protocol state:
   */

  /* the virtual device sample format */
  int                   total_sample_size;
  int                   sample_bits;            /* 8 or 16 */
  gboolean              is_big_endian;
  gboolean              is_signed;
  int                   num_channels;

  /* the number of samples we need to get to satisfy the
   * last audio command;  if 0, we should be trying to find a newline in
   * `incoming_control'.
   */
  int			num_sample_bytes_to_read;
  int                   sample_byte_index;
  gint64                sample_bytes_start_pos;
  void                 *raw_samples;
  GdamSampleConverter   sample_converter;
  GdamSampleConverter   channel_converter;

  /* we mostly track the virtual device OPEN protocol
   * to know the sample format.  This flag, indicating whether
   * we think the device is open, is useful for debugging.
   */
  gboolean              virtual_device_open;


  /* this is a stack of old buffers, used once we seek.
   *
   * The idea is that at that point ``current'' will be
   * copied to the head of this list and left empty;
   * the new, empty `current' will have start == new_position.
   *
   * but we need to collect the incoming data so we can
   * play sound continuously.
   */
  GSList               *old_buffers;
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
