/* SYNTAX: (linear-envelope (double-array time-volume-pairs)) */
/* BLURB:
 * Create an envelop which have the shape of a sequence
 * of trapezoids; the corners of consecutive trapezoids
 * coincide and are specified as time,volume pairs in
 * the (flattened) array TIME-VOLUME-PAIRS.
 *
 * For example:
 *     0 0  1 1  2 0                             is a triangle.
 *     0 0  0.5 0  0.5 1  1.5 1  1.5 0  2 0      is a square pulse.
 */

#include "synthlib.h"

/* returns the y value on a line given the x value */
static inline gdouble
lerp (gdouble x,
      gdouble x0,
      gdouble y0,
      gdouble x1,
      gdouble y1)
{
  return (x - x0) * (y1 - y0) / (x1 - x0) + y0;
}

int main (int argc, char **argv)
{
  guint num_items;
  gdouble *items;
  gdouble t = 0.0;
  gdouble secs;
  guint i;
  synth_lib_init (argc, argv);
  synth_lib_get_double_array (&num_items, &items);
  secs = 1.0 / synth_lib_sampling_rate;

  if (num_items % 2 == 1 || num_items < 4)
    synth_lib_error ("time/volume pair array must have even length, >= 4");
  for (i = 2; i < num_items; i += 2)
    if (items[i - 2] > items[i])
      synth_lib_error ("time/volume array must be ascending in time");
  i = 0;
  for (;;)
    {
      while (t >= items[2 * i + 2])
	{
	  i++;
	  if (i == num_items / 2)
	    return 0;
	}
      synth_lib_output (lerp (t, 
			      items[2 * i + 0],
			      items[2 * i + 1],
			      items[2 * i + 2],
			      items[2 * i + 3]));
      t += secs;
    }
  return 0;
}
