#include "../gdamintervalmap.h"

static gboolean list_compare (GSList *a, GSList *b)
{
  while (a != NULL && b != NULL)
    {
      if (strcmp (a->data, b->data) != 0)
        return FALSE;
      a = a->next;
      b = b->next;
    }
  if (a != NULL)
    return FALSE;
  if (b != NULL)
    return FALSE;
  return TRUE;
}

static gint my_strcmp (gconstpointer a, gconstpointer b)
{
  return strcmp ((const char *) a,(const char *) b);
}

static void interval_test(GdamIntervalMap *map, int val, char *arg, ...)
{
  GSList *list = NULL;
  GSList *got_list;
  GdamIntervalMapPost *post;
  va_list arg_list;
  post = gdam_interval_map_lookup (map, val);
  if (post == NULL)
    got_list = NULL;
  else
    got_list = g_slist_copy (post->objects);
  got_list = g_slist_sort (got_list, my_strcmp);
  va_start (arg_list, arg);
  while (arg != NULL)
    {
      list = g_slist_prepend (list, arg);
      arg = va_arg (arg_list, char *);
    }
  va_end (arg_list);
  list = g_slist_sort (list, my_strcmp);
  g_assert (list_compare (list, got_list));
  g_slist_free (list);
  g_slist_free (got_list);
}




int main (int argc, char** argv)
{
  GdamIntervalMap *map;
  map = gdam_interval_map_new ();

  gdam_interval_map_add (map, 1, 2, "a");
  interval_test (map, 0, NULL);
  interval_test (map, 1, "a", NULL);
  interval_test (map, 2, "a", NULL);
  interval_test (map, 3, NULL);

  gdam_interval_map_add (map, 2, 3, "b");
  interval_test (map, 0, NULL);
  interval_test (map, 1, "a", NULL);
  interval_test (map, 2, "a", "b", NULL);
  interval_test (map, 3, "b", NULL);
  interval_test (map, 4, NULL);

  gdam_interval_map_add (map, 0, 5, "d");
  interval_test (map, 0, "d", NULL);
  interval_test (map, 1, "a", "d", NULL);
  interval_test (map, 2, "a", "b", "d", NULL);
  interval_test (map, 3, "b", "d", NULL);
  interval_test (map, 4, "d", NULL);
  interval_test (map, 5, "d", NULL);
  interval_test (map, 6, NULL);

  gdam_interval_map_clear (map);
  gdam_interval_map_add (map, 50, 100, "d");
  interval_test (map, 49, NULL);
  interval_test (map, 50, "d", NULL);
  interval_test (map, 60, "d", NULL);
  interval_test (map, 75, "d", NULL);
  interval_test (map, 76, "d", NULL);
  interval_test (map, 80, "d", NULL);
  interval_test (map, 100, "d", NULL);
  interval_test (map, 101, NULL);
  gdam_interval_map_add (map, 50, 75, "e");
  interval_test (map, 49, NULL);
  interval_test (map, 50, "d", "e", NULL);
  interval_test (map, 60, "d", "e", NULL);
  interval_test (map, 75, "d", "e", NULL);
  interval_test (map, 76, "d", NULL);
  interval_test (map, 80, "d", NULL);
  interval_test (map, 100, "d", NULL);
  interval_test (map, 101, NULL);
  gdam_interval_map_add (map, 76, 100, "f");
  interval_test (map, 49, NULL);
  interval_test (map, 50, "d", "e", NULL);
  interval_test (map, 60, "d", "e", NULL);
  interval_test (map, 75, "d", "e", NULL);
  interval_test (map, 76, "d", "f", NULL);
  interval_test (map, 80, "d", "f", NULL);
  interval_test (map, 100, "d", "f", NULL);
  interval_test (map, 101, NULL);
  gdam_interval_map_add (map, 75, 100, "g");
  interval_test (map, 49, NULL);
  interval_test (map, 50, "d", "e", NULL);
  interval_test (map, 60, "d", "e", NULL);
  interval_test (map, 75, "d", "e", "g", NULL);
  interval_test (map, 76, "d", "f", "g", NULL);
  interval_test (map, 80, "d", "f", "g", NULL);
  interval_test (map, 100, "d", "f", "g", NULL);
  interval_test (map, 101, NULL);

  return 0;
}

