#include "testresult.h"


TestResult *test_result_new                ()
{
  TestResult *result;
  result = g_new (TestResult, 1);
  result->num_samples_compared = 0;
  result->squared_deviations = 0.0;
  result->max_deviation = 0.0;
  result->num_delta = 0;
  result->is_finished = FALSE;
  return result;
}

#define MAX_READ   1024

#define SAMPLE_TO_DOUBLE(sample_value)  			\
	(((gdouble)(sample_value)) / GDAM_SAMPLE_ONE)
#define COMPUTE_DELTA(sample_ptr_0, sample_ptr_1, index)	\
	SAMPLE_TO_DOUBLE ((sample_ptr_1)->s[index] - (sample_ptr_0)->s[index])

void        test_result_process            (TestResult   *result,
                                            int           num_samples,
                                            GdamSample   *samples_0,
					    GdamSample   *samples_1)
{
  int i;
  g_assert (result->num_delta == 0);
  for (i = 0; i < num_samples; i++)
    {
      gdouble left_delta = COMPUTE_DELTA (samples_0 + i, samples_1 + i, 0);
      gdouble right_delta = COMPUTE_DELTA (samples_0 + i, samples_1 + i, 1);
      if (left_delta < 0.0)
        left_delta = (-left_delta);
      if (right_delta < 0.0)
        right_delta = (-right_delta);
      result->squared_deviations += left_delta * left_delta;
      result->squared_deviations += right_delta * right_delta;
      if (left_delta > result->max_deviation)
        result->max_deviation = left_delta;
      if (right_delta > result->max_deviation)
        result->max_deviation = right_delta;
    }
  result->num_samples_compared += num_samples;
}

void        test_result_process_overflow   (TestResult   *result,
                                            int           num_samples,
                                            GdamSample   *samples,
					    int           which_source)
{
  g_assert (which_source == 0 || which_source == 1);
  if (which_source == 0)
    {
      g_assert (result->num_delta <= 0);
      result->num_delta -= num_samples;
    }
  else /*if (which_source == 1)*/
    {
      g_assert (result->num_delta >= 0);
      result->num_delta += num_samples;
    }
}
void test_result_finish (TestResult   *result)
{
  g_return_if_fail (! result->is_finished);
  result->is_finished = TRUE;
}

void        test_result_destroy            (TestResult   *test_result)
{
  g_free (test_result);
}

TestResult *test_result_compare_sources    (GdasSource   *source_0,
                                            GdasSource   *source_1)
{
  GdamBuffer buffer_0;
  GdamBuffer buffer_1;
  TestResult *result;
  gboolean a_eof = FALSE, b_eof = FALSE;
  gdam_buffer_construct (&buffer_0);
  gdam_buffer_construct (&buffer_1);

  result = test_result_new ();

  while (!a_eof && !b_eof)
    {
      GdamSample buf[MAX_READ];
      int num_read;
      g_return_val_if_fail (gdas_source_is_ready_to_read (source_0), FALSE);
      g_return_val_if_fail (gdas_source_is_ready_to_read (source_1), FALSE);
      num_read = gdas_source_read (source_0, buf, MAX_READ);
      g_message ("gdas_source_read 0: %d samples", num_read);
      gdam_buffer_append (&buffer_0, buf, sizeof (GdamSample) * num_read);
      a_eof = (num_read <= 0);

      num_read = gdas_source_read (source_1, buf, MAX_READ);
      g_message ("gdas_source_read 1: %d samples", num_read);
      gdam_buffer_append (&buffer_1, buf, sizeof (GdamSample) * num_read);
      b_eof = (num_read <= 0);

      while (buffer_0.size > 0 && buffer_1.size > 0)
	{
	  int to_compare = MIN (buffer_0.size, buffer_1.size);
	  GdamSample buf0[MAX_READ];
	  GdamSample buf1[MAX_READ];
	  to_compare /= sizeof (GdamSample);
	  if (to_compare > 0)
	    {
	      if (to_compare > MAX_READ)
		to_compare = MAX_READ;
	      gdam_buffer_read (&buffer_0, buf0, to_compare);
	      gdam_buffer_read (&buffer_1, buf1, to_compare);
	      test_result_process (result, to_compare, buf0, buf1);
	    }
	}
    }
  g_assert (buffer_0.size == 0 || buffer_1.size == 0);
  if (buffer_0.size > 0)
    {
      GdamSample *tmp_buf;
      int num_samples = buffer_0.size / sizeof (GdamSample);
      tmp_buf = alloca (num_samples * sizeof (GdamSample));
      gdam_buffer_read (&buffer_0, tmp_buf, num_samples * sizeof (GdamSample));
      test_result_process_overflow (result, num_samples, tmp_buf, 0);
    }
  else if (buffer_1.size > 0)
    {
      GdamSample *tmp_buf;
      int num_samples = buffer_1.size / sizeof (GdamSample);
      tmp_buf = alloca (num_samples * sizeof (GdamSample));
      gdam_buffer_read (&buffer_1, tmp_buf, num_samples * sizeof (GdamSample));
      test_result_process_overflow (result, num_samples, tmp_buf, 1);
    }

  test_result_finish (result);

  return result;
}

gboolean    test_result_is_with_thresholds (TestResult   *result, 
                                            const char   *threshold)
{
  if (strcmp (threshold, "print") == 0)
    {
      g_message ("is_within_thresholds: num_samples=%lld; sqdeviations=%.7g; "
      			"maxdev=%.6g; num_delta=%d",
		    result->num_samples_compared,
		    result->squared_deviations,
		    result->max_deviation,
		    result->num_delta);
      return TRUE;
    }
  else
    {
      g_warning ("don't know how to cope with threshold `%s'", threshold);
      return FALSE;
    }
}
