/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __TEST_SOURCE_H_
#define __TEST_SOURCE_H_


typedef struct _TestSource TestSource;
typedef struct _TestSourceClass TestSourceClass;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "../gdassource.h"

GtkType test_source_get_type();
#define TEST_TYPE_SOURCE			(test_source_get_type ())
#define TEST_SOURCE(obj)              (GTK_CHECK_CAST ((obj), TEST_TYPE_SOURCE, TestSource))
#define TEST_SOURCE_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), TEST_TYPE_SOURCE, TestSourceClass))
#define TEST_IS_SOURCE(obj)           (GTK_CHECK_TYPE ((obj), TEST_TYPE_SOURCE))
#define TEST_IS_SOURCE_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), TEST_TYPE_SOURCE))

typedef struct _TestSourceData TestSourceData;
struct _TestSourceData {
	int		ref_count;
        GSList         *first_block;   /* GArray<GdamSample> */
        GSList         *last_block;
};
TestSourceData *test_source_data_new      ();
void            test_source_data_add_chunk(TestSourceData    *data,
                                           int                num_samples,
                                           GdamSample        *samples);
void            test_source_data_add_mono (TestSourceData    *data,
                                           int                num_samples,
                                           GdamSampleValue   *values);
void            test_source_data_add_array(TestSourceData    *data,
                                           GArray            *samples);
void            test_source_data_ref      (TestSourceData    *source);
void            test_source_data_unref    (TestSourceData    *source);

struct _TestSourceClass {
	GdasSourceClass		source_class;
};
struct _TestSource {
	GdasSource		source;

        TestSourceData         *source_data;

	/* list node of the block our next sample should
	 * come from. */
        GSList                 *current_block;

	/* index in current block of next sample to output. */
	int			block_index;
};


GdasSource     *test_source_new           (TestSourceData    *test);
GdasSource     *test_source_from_spec     (const char        *spec);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
