#include <stdio.h>
#include <ctype.h>
#include <gdam/server/gdassource.h>
#include <gdam/common/gdamplugin.h>
#include <string.h>
#include <stdlib.h>
#include "testresult.h"
#include "testsource.h"
#include <gdam/server/gdammain.h>

/*
 * Grammar:
 *     word   = \w+
 *     whitespace = \s+
 *
 *     source = <word:word=value whitespace .... >
 *            | @input@
 *     value = \w\S+
 *           | source
 *           | "[^"]*"
 *
 * don't forget: done_constructing
 */
static GdasSource *parse_source(const char **pdesc, GdasSource *filtered);

static char *parse_value(const char **pdesc, GdasSource *filtered)
{
  const char *desc = *pdesc;
  if (*desc == '<' || *desc == '@')
    {
      GdasSource *source;
      source = parse_source (pdesc, filtered);
      if (source == NULL)
	{
	  g_warning ("error parsing: %s", desc);
	  return NULL;
	}
      return g_strdup_printf ("%d", GDAM_SERVER_OBJECT (source)->global_id);
    }
  if (*desc == '"')
    {
      int length;
      const char *end_quote;
      char *rv;
      end_quote = strchr (desc + 1, '"');
      if (end_quote == NULL)
	{
	  g_warning ("no terminal \" character");
	  return NULL;
	}
      length = end_quote - (desc + 1);
      rv = g_new (char, length + 1);
      memcpy (rv, desc + 1, length);
      rv[length] = '\0';
      *pdesc = end_quote + 1;
      return rv;
    }
  else
    {
      const char *end_word = desc;
      char *rv;
      while (!isspace (*end_word) && *end_word != '>')
	end_word++;
      rv = g_new (char, end_word - desc + 1);
      memcpy (rv, desc, end_word - desc);
      rv[end_word - desc] = '\0';
      *pdesc = end_word;
      return rv;
    }
}
static gboolean set_str_argument (GdasSource      *source,
                                  char            *word,
				  char            *value)
{
}
              
static GdasSource *parse_source(const char **pdesc, GdasSource *filtered)
{
  const char *desc = *pdesc;
  while (*desc != '\0' && isspace (*desc))
    desc++;
  g_message ("parse_source (%s)", desc);
  if (*desc == '<')
    {
      GdasSource *source;
      desc++;
      while (*desc != '\0' && isspace (*desc))
	desc++;

      /* read source type */
      {
	const char *end_word = desc;
	char *source_str;
	GtkType source_type;
	while (isalnum (*end_word) && *end_word != '\0')
	  {
	    end_word++;
	  }
	source_str = alloca (end_word + 1 - desc);
	memcpy (source_str, desc, end_word - desc);
	source_str[end_word - desc] = '\0';

	if (*end_word != ':')
	  {
	    g_warning ("expected : after typename");
	    return NULL;
	  }
	desc = end_word + 1;

	source_type = gdam_plugin_get_type_by_name ("server", source_str);
	if (source_type == 0)
	  {
	    g_warning ("couldn't make source of type `%s'", source_str);
	    return NULL;
	  }
	source = GDAS_SOURCE (gtk_type_new (source_type));
	g_return_val_if_fail (source != NULL, NULL);
      }


      /* parse arguments and set them. */
      while (*desc != '>' && *desc != '\0')
	{
	  const char *end_word;
	  char *word;
	  const char *value_string;
	  const char *end_value;
	  char *value;
	  while (*desc != '\0' && isspace (*desc))
	    desc++;
	  end_word = desc;
	  while (*end_word && isalnum (*end_word))
	    end_word++;
	  if (*end_word != '=')
	    {
	      g_warning ("error parsing = after key");
	      return NULL;
	    }
	  word = alloca (end_word - desc + 1);
	  memcpy (word, desc, end_word - desc);
	  word[end_word - desc] = '\0';
	  value_string = end_word + 1;
	  end_value = value_string;
	  value = parse_value (&end_value, filtered);
	  if (value == NULL || end_value == value_string)
	    {
	      g_warning ("got NULL value string");
	      return NULL;
	    }
	  if (!set_str_argument (source, word, value))
	    g_error ("set_str_argument: `%s' [value=%s]", word, value);
	  
	  while (*end_value && isspace (*end_value))
	    end_value++;
	  if (*end_value == ':')
	    {
	      desc = end_value + 1;
	    }
	  else if (*end_value == '>')
	    {
	      desc = end_value;
	    }
	  else
	    {
	      g_warning ("excepted : or >, got `%s'", end_value);
	      return NULL;
	    }
	}
	return source;
    }
  else if (strncmp (desc, "@input@", 7) == 0)
    {
      *pdesc = desc + 7;
      return filtered;
    }
  else
    {
      g_warning ("parse_source, source should begin " "with < or @input@");
      return NULL;
    }
}
void __run_test(const char *raw_description,
                const char *filter_description,
                const char *filtered_description,
                const char *human_description,
                const char *thresholds,
                const char *file,
                int         lineno)
{
  int num_raw_samples;
  GdamSample *raw_samples;
  int num_filtered_samples;
  const char *end_filter_desc;
  GdamSample *filtered_samples;
  TestResult* result;
  GdasSource *test_input;
  GdasSource *test_output;
  GdasSource *filter;

  test_input = test_source_from_spec (raw_description);
  if (test_input == NULL)
    {
      g_error ("internal test error [%s]; "
	       "error parsing raw description; "
	       "file %s, line %d", human_description, file, lineno);
    }

  test_output = test_source_from_spec (filtered_description);
  if (test_output == NULL)
    {
      g_error ("internal test error [%s]; "
	       "error parsing filtered description; "
	       "file %s, line %d", human_description, file, lineno);
    }

  gtk_object_ref (GTK_OBJECT(test_output));
  gtk_object_ref (GTK_OBJECT(test_input));

  end_filter_desc = filter_description;
  filter = parse_source (&end_filter_desc, test_input);
  if (filter == NULL)
    {
      g_error ("internal test error; "
	       "filter parse_source; " "file %s, line %d", file, lineno);
    }

  result = test_result_compare_sources (test_output, filter);

  if (result == NULL)
    {
      g_error ("test failed; "
	       "the comparator failed, indicating "
	       "the source had a read error"
	       "file %s, line %d", file, lineno);
    }

  if (!test_result_is_with_thresholds (result, thresholds))
    {
      g_error ("test failed; "
		"the test ran but was not within thresholds, "
		"file %s, line %d", file, lineno);
    }
  gtk_object_sink (GTK_OBJECT (test_input));
  gtk_object_sink (GTK_OBJECT (filter));
  gtk_object_unref (GTK_OBJECT(test_output));
  gtk_object_unref (GTK_OBJECT(test_input));
}

#define run_test(raw_description, filter_description, filtered_description, human_description, thresholds) \
        __run_test(raw_description, filter_description, \
                   filtered_description, human_description, thresholds, \
                   __FILE__, __LINE__)


int main(int argc, char **argv)
{
  gdam_init (NULL, NULL);
  run_test("m:0:1:0:1:0:1:0:-1",
	   "@input@",
	   "m:0:0.5:0:0.5:0:0.5:0:-0.5",
	   "internal test case",
	   "print");
  run_test("m:0:1:0:1:0:1:0:-1",
	   "<GdasGain:gain=0.5:filtered=@input@>",
	   "m:0:1:0:1:0:1:0:-1",
	   "0.5 gain test case",
	   "print");
  run_test("m:0:1:0:1:0:1:0:-1:-0.1,-0.2,+0.5",
	   "<GdasGain:gain=-1:filtered=@input@>",
	   "m:0:-1:0:-1:0:-1:0:1:0.1,0.2,-0.5",
	   "-1 gain test case",
	   "print");
  return 0;
}

