#include "debpanner.h"
#include "gdamcursor.h"
#include <stdlib.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkmain.h>

static GtkObjectClass *parent_class = NULL;

enum
{
  ARG_0,
  ARG_X_ADJUSTMENT,
  ARG_Y_ADJUSTMENT
};
#define PANNER_MASK	(GDK_EXPOSURE_MASK |		\
			 GDK_POINTER_MOTION_MASK |	\
			 GDK_POINTER_MOTION_HINT_MASK |	\
			 GDK_ENTER_NOTIFY_MASK |	\
			 GDK_BUTTON_PRESS_MASK |	\
			 GDK_BUTTON_RELEASE_MASK |	\
			 GDK_BUTTON1_MOTION_MASK)

static gboolean compute_rectangle_coords (DebPanner    *panner,
                                          int          *x1_out,
                                          int          *x2_out,
                                          int          *y1_out,
                                          int          *y2_out)
{
  int width, height;
  GtkWidget *widget;
  widget = GTK_WIDGET (panner);
  width = widget->allocation.width;
  height = widget->allocation.height;
  if (panner->x_range == NULL)
    *x1_out = *x2_out = width / 2;
  else
    {
      gfloat v = panner->x_range->value;
      gfloat x_range = (panner->x_range->upper - panner->x_range->lower);
      gfloat ratio = (gfloat) width / x_range;
      v -= panner->x_range->lower;
      *x1_out = (int) (v * ratio);
      v += panner->x_range->page_size;
      *x2_out = (int) (v * ratio);
    }

  if (panner->y_range == NULL)
    *y1_out = *y2_out = height / 2;
  else
    {
      gfloat v = panner->y_range->value;
      gfloat y_range = (panner->y_range->upper - panner->y_range->lower);
      gfloat ratio = (gfloat) height / y_range;
      v -= panner->y_range->lower;
      *y1_out = (int) (v * ratio);
      v += panner->y_range->page_size;
      *y2_out = (int) (v * ratio);
    }
  return TRUE;
}

#if 1
#define DEBUG_LINE(dir, c1, c2, d)
#else
#define DEBUG_LINE(dir, c1, c2, d)				\
	g_message ("draw_%sline: %d - %d @ %d", #dir, c1, c2, d)
#endif

#if 0
#define PAINT_VLINE(widget, y1, y2, x)				\
      DEBUG_LINE(v, y1, y2, x);					\
      gtk_paint_vline (widget->style, widget->window,		\
		       widget->state, NULL, widget, NULL,	\
                       y1, y2, x);
#define PAINT_HLINE(widget, x1, x2, y)				\
      DEBUG_LINE(h, x1, x2, y);					\
      gtk_paint_hline (widget->style, widget->window,		\
		       widget->state, NULL, widget, NULL,	\
                       x1, x2, y);
#else
#define WHICH_GC		fg_gc   
#define PAINT_VLINE(widget, y1, y2, x)				\
      DEBUG_LINE(v, y1, y2, x);					\
      gdk_draw_line (widget->window,                            \
                     widget->style->WHICH_GC[state],            \
      		     x, y1, x, y2);
#define PAINT_HLINE(widget, x1, x2, y)				\
      DEBUG_LINE(h, x1, x2, y);					\
      gdk_draw_line (widget->window,                            \
                     widget->style->WHICH_GC[state],            \
      		     x1, y, x2, y);
#endif

static void deb_panner_draw (GtkWidget *widget, GdkRectangle *rect)
{
  DebPanner *panner = DEB_PANNER (widget);
  gboolean x_in_bound = FALSE, y_in_bound = FALSE;
  int x1, x2, y1, y2;
  int x_min, y_min, x_max, y_max;
  guint state = widget->state;
  GdkRectangle _new_rect;
  GtkStyle *style;

  if (!widget->window)
    return;

  if (!compute_rectangle_coords (panner, &x1, &x2, &y1, &y2))
    return;

  style = widget->style;
  if (rect == NULL)
    {
      _new_rect.x = 0;
      _new_rect.y = 0;
      _new_rect.width = widget->allocation.width;
      _new_rect.height = widget->allocation.height;
      rect = &_new_rect;
    }
  gtk_paint_flat_box (style, widget->window, GTK_STATE_NORMAL, GTK_SHADOW_NONE,
		      NULL, widget, "panner",
		      rect->x, rect->y, rect->width, rect->height);

  /* draw the light grey lines on the panner's background. */
  x_min = rect->x;
  y_min = rect->y;
  x_max = rect->x + rect->width - 1;
  y_max = rect->y + rect->height - 1;
  if (x_min <= x1 && x1 <= x_max)
    {
      PAINT_VLINE (widget, y_min, y_max, x1);
      x_in_bound = TRUE;
    }
  if (x_min <= x2 && x2 <= x_max)
    {
      PAINT_VLINE (widget, y_min, y_max, x2);
      x_in_bound = TRUE;
    }
  if (y_min <= y1 && y1 <= y_max)
    {
      PAINT_HLINE (widget, x_min, x_max, y1);
      y_in_bound = TRUE;
    }
  if (y_min <= y2 && y2 <= y_max)
    {
      PAINT_HLINE (widget, x_min, x_max, y2);
      y_in_bound = TRUE;
    }

  /* draw the box in those lines.
   * XXX: this is somewhat broken.
   */
  if (x_in_bound && y_in_bound
   && (x2 - x1) >= 2 && (y2 - y1) >= 2)
    {
      gdk_draw_rectangle (widget->window, widget->style->dark_gc[state], TRUE,
		          x1, y1, y2 - y1 - 1, x2 - x1 - 1);
    }
}
static gint deb_panner_expose_event(GtkWidget      *panner,
                                    GdkEventExpose *event)
{
  deb_panner_draw (panner, &event->area);
  return 0;
}

static void block_adjustment_signals (DebPanner *panner)
{
  if (panner->x_value_changed != 0)
    gtk_signal_handler_block (GTK_OBJECT (panner->x_range),
                              panner->x_value_changed);
  if (panner->x_changed != 0)
    gtk_signal_handler_block (GTK_OBJECT (panner->x_range),
                              panner->x_changed);
  if (panner->y_value_changed != 0)
    gtk_signal_handler_block (GTK_OBJECT (panner->y_range),
                              panner->y_value_changed);
  if (panner->y_changed != 0)
    gtk_signal_handler_block (GTK_OBJECT (panner->y_range),
                              panner->y_changed);
}

static void unblock_adjustment_signals (DebPanner *panner)
{
  if (panner->x_value_changed != 0)
    gtk_signal_handler_unblock (GTK_OBJECT (panner->x_range),
                                panner->x_value_changed);
  if (panner->x_changed != 0)
    gtk_signal_handler_unblock (GTK_OBJECT (panner->x_range),
                                panner->x_changed);
  if (panner->y_value_changed != 0)
    gtk_signal_handler_unblock (GTK_OBJECT (panner->y_range),
                                panner->y_value_changed);
  if (panner->y_changed != 0)
    gtk_signal_handler_unblock (GTK_OBJECT (panner->y_range),
                                panner->y_changed);
}

static void deb_panner_center (DebPanner         *panner,
                               int                x,
			       int                y)
{
  int width, height;
  GtkWidget *widget;
  widget = GTK_WIDGET (panner);
  width = widget->allocation.width;
  height = widget->allocation.height;
  g_return_if_fail (width > 0);
  g_return_if_fail (height > 0);
  block_adjustment_signals (panner);
  if (panner->x_range != NULL)
    {
      gfloat v;
      v = x * (panner->x_range->upper - panner->x_range->lower) / width
        + panner->x_range->lower
	- panner->x_range->page_size / 2;
      gtk_adjustment_set_value (panner->x_range, v);
    }
  if (panner->y_range != NULL)
    {
      gfloat v;
      v = y * (panner->y_range->upper - panner->y_range->lower) / height
        + panner->y_range->lower
	- panner->y_range->page_size / 2;
      gtk_adjustment_set_value (panner->y_range, v);
    }
  unblock_adjustment_signals (panner);
  deb_panner_draw (GTK_WIDGET (widget), NULL);
}

static int  deb_panner_button_press_event (GtkWidget      *widget,
                                           GdkEventButton *event)
{
  if (event->button == 4)
    return 0;
  else if (event->button == 5)
    return 0;
  else
    {
      deb_panner_center (DEB_PANNER (widget), event->x, event->y);
    }
  DEB_PANNER (widget)->is_dragging = TRUE;
  gtk_grab_add (widget);
  return 0;
}

static gboolean set_cursor 			 (gpointer        ptr)
{
  g_return_val_if_fail (GTK_WIDGET (ptr)->window != NULL, TRUE);
  gdam_set_window_cursor (GTK_WIDGET (ptr)->window, GDAM_MOVE_CURSOR);
  return FALSE;
}

static void deb_panner_realize                   (GtkWidget      *widget)
{
  (*GTK_WIDGET_CLASS (parent_class)->realize) (widget);
  g_idle_add (set_cursor, widget);
}

static int  deb_panner_motion_notify_event       (GtkWidget      *widget,
                                                  GdkEventMotion *event)
{
  int x, y;
  guint state;

  if (!DEB_PANNER (widget)->is_dragging)
    return 0;

  if (event->is_hint)
    gdk_window_get_pointer (widget->window, &x, &y, &state);
  else
    {
      if (event->window != widget->window)
	{
	  gdk_window_get_pointer (widget->window, &x, &y, &state);
	}
      else
	{
	  x = event->x;
	  y = event->y;
	  state = event->state;
	}
    }
  if (state == 0)
    return 0;

  deb_panner_center (DEB_PANNER (widget), x, y);
  return 0;
}

static int  deb_panner_button_release_event (GtkWidget      *widget,
                                             GdkEventButton *event)
{
  guint state;
  int x, y;

  if (event->button == 4)
    return 0;
  else if (event->button == 5)
    return 0;

  if (event->window != widget->window)
    {
      gdk_window_get_pointer (widget->window, &x, &y, &state);
    }
  else
    {
      x = event->x;
      y = event->y;
      state = event->state;
    }
  deb_panner_center (DEB_PANNER (widget), x, y);
  DEB_PANNER (widget)->is_dragging = FALSE;
  gtk_grab_remove (widget);
  return 0;
}

static void deb_panner_adjustment_changed(GtkAdjustment    *adj,
                                          gpointer          user_data)
{
  deb_panner_draw (GTK_WIDGET (user_data), NULL);
}
static void deb_panner_adjustment_value_changed(GtkAdjustment    *adj,
                                                DebPanner        *panner)
{
  deb_panner_draw (GTK_WIDGET (panner), NULL);
}

static void
deb_panner_set_arg (GtkObject  *object,
		    GtkArg     *arg,
		    guint	arg_id)
{
  DebPanner *panner = DEB_PANNER (object);
  GtkObject *adj_object;
  GtkAdjustment **in_panner = NULL;
  guint *connect_id_changed = NULL;
  guint *connect_id_value_changed = NULL;
  switch (arg_id)
    {
      case ARG_X_ADJUSTMENT:
	in_panner = &panner->x_range;
	connect_id_changed = &panner->x_changed;
	connect_id_value_changed = &panner->x_value_changed;
	break;
      case ARG_Y_ADJUSTMENT:
	in_panner = &panner->y_range;
	connect_id_changed = &panner->y_changed;
	connect_id_value_changed = &panner->y_value_changed;
	break;
      default:
        return;
    }

  if (*connect_id_changed != 0)
    gtk_signal_disconnect (GTK_OBJECT (*in_panner), *connect_id_changed);
  if (*connect_id_value_changed != 0)
    gtk_signal_disconnect (GTK_OBJECT (*in_panner), *connect_id_value_changed);

  if (*in_panner != NULL)
    gtk_object_unref (GTK_OBJECT (panner->x_range));
  adj_object = GTK_VALUE_OBJECT (*arg);
  if (adj_object == NULL)
    {
      *in_panner = NULL;
      *connect_id_changed = 0;
      *connect_id_value_changed = 0;
    }
  else
    {
      *in_panner = GTK_ADJUSTMENT (adj_object);
      gtk_object_ref (adj_object);
      gtk_object_sink (adj_object);
      *connect_id_changed 
      	= gtk_signal_connect (adj_object, "changed",
	                      GTK_SIGNAL_FUNC (deb_panner_adjustment_changed),
			      panner);
      *connect_id_value_changed 
      	= gtk_signal_connect (adj_object, "value_changed",
			GTK_SIGNAL_FUNC (deb_panner_adjustment_value_changed),
			      panner);
                                             
    }
}

static void
deb_panner_get_arg (GtkObject  *object,
		    GtkArg     *arg,
		    guint	arg_id)
{
  DebPanner *panner = DEB_PANNER (object);
  GtkAdjustment *adjustment = NULL;
  switch (arg_id)
    {
      case ARG_X_ADJUSTMENT:
	adjustment = panner->x_range;
	break;

      case ARG_Y_ADJUSTMENT:
	adjustment = panner->y_range;
	break;
	
      default:
        break;
    }
  if (adjustment == NULL)
    GTK_VALUE_OBJECT (*arg) = NULL;
  else
    GTK_VALUE_OBJECT (*arg) = GTK_OBJECT (adjustment);
}

static void deb_panner_destroy (GtkObject *object)
{
  DebPanner *panner = DEB_PANNER (object);
  if (panner->x_range != NULL)
    {
      gtk_signal_disconnect_by_data (GTK_OBJECT (panner->x_range), object);
      gtk_object_unref (GTK_OBJECT (panner->x_range));
      panner->x_range = NULL;
    }
  if (panner->y_range != NULL)
    {
      gtk_signal_disconnect_by_data (GTK_OBJECT (panner->y_range), object);
      gtk_object_unref (GTK_OBJECT (panner->y_range));
      panner->y_range = NULL;
    }
  (*parent_class->destroy) (object);
}

static void deb_panner_init (DebPanner* panner)
{
  gint old_mask;
  old_mask = gtk_widget_get_events (GTK_WIDGET (panner));
  gtk_widget_set_events (GTK_WIDGET (panner), old_mask | PANNER_MASK);
  gtk_drawing_area_size (GTK_DRAWING_AREA (panner), 200, 200);
}

static void deb_panner_class_init (DebPannerClass* panner_class)
{
  GtkWidgetClass *klass = GTK_WIDGET_CLASS (panner_class);
  GtkObjectClass *object_class = GTK_OBJECT_CLASS (panner_class);
  klass->draw = deb_panner_draw;
  klass->expose_event = deb_panner_expose_event;
  klass->button_press_event = deb_panner_button_press_event;
  klass->button_release_event = deb_panner_button_release_event;
  klass->motion_notify_event = deb_panner_motion_notify_event;
  klass->realize = deb_panner_realize;
  object_class->set_arg = deb_panner_set_arg;
  object_class->get_arg = deb_panner_get_arg;
  object_class->destroy = deb_panner_destroy;
  gtk_object_add_arg_type ("DebPanner::x-adjustment",
                           GTK_TYPE_ADJUSTMENT,
			   GTK_ARG_READWRITE,
			   ARG_X_ADJUSTMENT);
  gtk_object_add_arg_type ("DebPanner::y-adjustment",
                           GTK_TYPE_ADJUSTMENT,
			   GTK_ARG_READWRITE,
			   ARG_Y_ADJUSTMENT);
}

GtkWidget *deb_panner_new (GtkAdjustment *x_adj, GtkAdjustment *y_adj)
{
  return GTK_WIDGET (gtk_object_new (DEB_TYPE_PANNER,
                                     "x-adjustment",
				     x_adj,
                                     "y-adjustment",
				     y_adj,
				     NULL));
}

GtkType deb_panner_get_type()
{
  static GtkType panner_type = 0;
  if (!panner_type) {
    static const GtkTypeInfo panner_info =
    {
      "DebPanner",
      sizeof(DebPanner),
      sizeof(DebPannerClass),
      (GtkClassInitFunc) deb_panner_class_init,
      (GtkObjectInitFunc) deb_panner_init,
      /* reserved_1 */ NULL,
      /* reserved_2 */ NULL,
      (GtkClassInitFunc) NULL
    };
    GtkType parent = GTK_TYPE_DRAWING_AREA;
    panner_type = gtk_type_unique (parent, &panner_info);
    parent_class = gtk_type_class (parent);
  }
  return panner_type;
}

static GtkAdjustment *parse_adj (const char *spec)
{
  char **vals = g_strsplit (spec, ":", 4);
  gfloat cur = 0, lower = 0, upper = 0, page = 0;
  if (vals[0] != NULL)
    {
      cur = atof (vals[0]);
      if (vals[1] != NULL)
        {
	  lower = atof (vals[1]);
	  if (vals[2] != NULL)
	    {
	      upper = atof (vals[2]);
	      if (vals[3] != NULL)
		{
		  page = atof (vals[3]);
	        }
	    }
	}
    }
  return GTK_ADJUSTMENT (gtk_adjustment_new(cur,
  			lower, upper, page / 10, page / 100, page));
}

/*
 * To make a DebPanner:
 *
 *   string1:    CUR_VALUE:MIN_VALUE:MAX_VALUE:PAGE_SIZE
 *                    [for horizontal direction]
 *   string2:    CUR_VALUE:MIN_VALUE:MAX_VALUE:PAGE_SIZE
 *                    [for vertical direction]
 *
 * Here is a description of the fields.
 *
 *     cur_value:        initial value for the adjustment.
 *     min_value:        minimum value for the adjustment.
 *     max_value:        maximum value for the adjustment.
 *     page_size:        width of the adjustment
 */
GtkWidget* deb_panner_create_func          (const char*   name,
                                            const char*   string1,
                                            const char*   string2,
                                            int           int1,
                                            int           int2)
{
  GtkAdjustment *x_adj;
  GtkAdjustment *y_adj;

  x_adj = parse_adj (string1);
  g_return_val_if_fail (x_adj != NULL, NULL);
  y_adj = parse_adj (string2);
  g_return_val_if_fail (y_adj != NULL, NULL);

  return deb_panner_new (x_adj, y_adj);
}

