/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdambuffermanagerskin.h"
#include <gtk/gtkclist.h>
#include <gdam/component/gdambuffermanager.h>
#include "gdamskinutil.h"
#include <stdlib.h>
#include <string.h>

static GtkObjectClass* parent_class;

static void gdam_buffer_manager_skin_destroy(GtkObject* object);
static void gdam_buffer_manager_skin_class_init(GdamSkinClass* klass);

static void gdam_buffer_manager_skin_init(GdamBufferManagerSkin* buffer_manager_skin)
{
}

GtkType gdam_buffer_manager_skin_get_type()
{
  static GtkType buffer_manager_skin_type = 0;
  if (!buffer_manager_skin_type)
    {
      static const GtkTypeInfo buffer_manager_skin_info = {
	"GdamBufferManagerSkin",
	sizeof (GdamBufferManagerSkin),
	sizeof (GdamBufferManagerSkinClass),
	(GtkClassInitFunc) gdam_buffer_manager_skin_class_init,
	(GtkObjectInitFunc) gdam_buffer_manager_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_SKIN;
      parent_class = gtk_type_class (parent);
      buffer_manager_skin_type = gtk_type_unique (parent,
						  &buffer_manager_skin_info);
    }
  return buffer_manager_skin_type;
}

typedef struct _CListInfo CListInfo;
typedef char*   (*CListEntryGetter)(GdamBufferManagerInfo*   info);

struct _CListInfo {
	int                        column;
	GdamBufferManagerSkin*       skin;

	/* We reference count the clist. */
	GtkCList*                  clist;
	int			   selected_connect_id;

	int                        num_columns;
	CListEntryGetter*	   column_fillers;

	GdamBufferManagerNotifier* notifier;
};

static void clist_set_default(GtkCList*         clist,
                              int               row,
			      int               column,
			      GdkEvent*         event,
			      CListInfo*        clist_info)
{
  GdamBufferManagerSkin* skin;
  GdamBufferManagerInfo* bufinfo;
  char *buffer_id_txt;
  int buffer_id;
  skin = clist_info->skin;
  buffer_id_txt = NULL;
  gtk_clist_get_text (clist, row, clist_info->column, &buffer_id_txt);
  if (!buffer_id_txt || (buffer_id = atoi (buffer_id_txt)) <= 0)
    {
      g_warning ("error getting buffer id from %d,%d in clist",
		 row, clist_info->column);
      return;
    }
  gdam_buffer_set_default (buffer_id);
  bufinfo = gdam_buffer_manager_info_by_id (buffer_id);
  g_return_if_fail (bufinfo != NULL);
  if (skin->id_label)
    {
      char str[63];
      sprintf (str, "%d", buffer_id);
      gtk_label_set_text (skin->id_label, str);
    }
  if (skin->description_entry)
    {
      gtk_entry_set_text (skin->description_entry, 
	  		  bufinfo->description);
    }
  if (skin->bpm_adj)
    {
      float bpm;
      if (bufinfo->beat_info)
	bpm = gdam_beat_info_get_bpm (bufinfo->beat_info, 0);
      else
	bpm = 0;
      gtk_adjustment_set_value (skin->bpm_adj, bpm);
    }
  if (skin->fb_adj)
    {
      float fb;
      if (bufinfo->beat_info)
	fb = gdam_beat_info_get_first_beat (bufinfo->beat_info, 0);
      else
	fb = 0;
      gtk_adjustment_set_value (skin->fb_adj, fb);
    }
}

static void clist_buffer_add_callback(GdamBufferManagerInfo*   info,
                                      gpointer                 user_data)
{
  CListInfo *clist_info = (CListInfo *) user_data;
  GtkCList *clist = clist_info->clist;
  char **strings;
  int i;
  int num_columns = clist_info->num_columns;
  int new_idx;
  g_return_if_fail (num_columns == clist->columns);
  strings = (char **) alloca (sizeof (char *) * num_columns);
  for (i = 0; i < num_columns; i++)
    {
      strings[i] = (*clist_info->column_fillers[i]) (info);
      gdam_debug ("buffer_add_callback: strings[%d]=%s", i, strings[i]);
    }
  new_idx = gtk_clist_append (clist, strings);
  /* TODO: is this safe?   what if info disappears? */
  gtk_clist_set_row_data (clist, new_idx, info);
  for (i = 0; i < num_columns; i++)
    g_free (strings[i]);
}

static void clist_buffer_remove_callback(GdamBufferManagerInfo*   info,
                                         gpointer                 user_data)
{
  CListInfo *clist_info = (CListInfo *) user_data;
  GtkCList *clist = clist_info->clist;
  int i;
  for (i = 0; i < clist->rows; i++)
    {
      char *txt = NULL;
      int id;
      gtk_clist_get_text (clist, i, clist_info->column, &txt);
      if (txt == NULL || (id = atoi (txt)) <= 0)
	continue;
      if (id == (int) info->buffer_id)
	{
	  gtk_clist_remove (clist, i);
	  return;
	}
    }
}

static void clist_buffer_change_callback(GdamBufferManagerInfo*   info,
                                         gpointer                 user_data)
{
  CListInfo *clist_info = (CListInfo *) user_data;
  GtkCList *clist = clist_info->clist;
  GdamBufferManagerSkin* skin = clist_info->skin;
  int i;
  guint default_id = gdam_buffer_get_default ();
  /* If the buffer which changed is the currently selected one, 
   * update our other displays.
   */
  if (info->buffer_id == default_id)
    {
      if (skin->bpm_adj)
	{
	  float bpm;
	  if (info->beat_info)
	    bpm = gdam_beat_info_get_bpm (info->beat_info, 0);
	  else
	    bpm = 0;
	  gtk_adjustment_set_value (skin->bpm_adj, bpm);
	}
      if (skin->id_label)
	{
	  char str[63];
	  sprintf (str, "%d", info->buffer_id);
	  gtk_label_set_text (skin->id_label, str);
	}
      if (skin->description_entry)
	{
	  gtk_entry_set_text (skin->description_entry, 
			      info->description);
	}
      if (skin->fb_adj)
	{
	  float fb;
	  if (info->beat_info)
	    fb = gdam_beat_info_get_first_beat (info->beat_info, 0);
	  else
	    fb = 0;
	  gtk_adjustment_set_value (skin->fb_adj, fb);
	}
      if (skin->num_beats_adj)
	{
	  gdouble num_beats;
	  gdouble beat_length = 1.0;
	  if (info->beat_info)
	    {
	      float bpm;
	      bpm = gdam_beat_info_get_bpm (info->beat_info, 0);
	      beat_length = 60.0 / bpm;
	    }
	  if (info->length > 0)
	    num_beats = (info->length / 44100.0) / beat_length;
	  else
	    num_beats = 0;
	  gtk_adjustment_set_value (skin->num_beats_adj, num_beats);
	}
    }
  /* Find this info in the clist and replace its entry. */
  for (i = 0; i < clist->rows; i++)
    {
      char *txt = NULL;
      int id;
      gtk_clist_get_text (clist, i, clist_info->column, &txt);
      if (txt == NULL || (id = atoi (txt)) <= 0)
	continue;
      if (id == (int) info->buffer_id)
	{
	  char **strings;
	  int j;
	  int num_columns = clist_info->num_columns;
	  int new_idx;
	  /* remove old entry (easier to just change each column's text?? ) */
	  gtk_clist_freeze (clist);
	  gtk_clist_remove (clist, i);
	  /* readd old entry  */
	  strings = (char **) alloca (sizeof (char *) * num_columns);
	  for (j = 0; j < num_columns; j++)
	    {
	      strings[j] = (*clist_info->column_fillers[j]) (info);
	      gdam_debug ("buffer_add_callback: strings[%d]=%s", j, strings[j]);
	    }
	  new_idx = gtk_clist_insert (clist, i, strings);
	  /* TODO: is this safe?   what if info disappears? */
	  gtk_clist_set_row_data (clist, new_idx, info);
	  gtk_clist_thaw (clist);
	  for (i = 0; i < num_columns; i++)
	    g_free (strings[i]);
	  return;
	}
    }
}

static gboolean is_one_of(const char* key, ...)
{
  va_list args;
  char *cmp;
  va_start (args, key);
  while ((cmp = va_arg (args, char *)) != NULL)
    {
      if (g_strcasecmp (key, cmp) == 0)
	{
	  va_end (args);
	  return TRUE;
	}
    }
  va_end (args);
  return FALSE;
}

static int find_clist_column(int num_cols, char** cols)
{
  int out = 0;
  while (out < num_cols && *cols != NULL)
    {
      if (is_one_of (*cols++,
		     "buffer id", "buffer-id", "buffer_id", (char *) NULL))
	return out;
      out++;
    }
  return -1;
}

static char* buffer_id_getter(GdamBufferManagerInfo* info)
{
  return g_strdup_printf ("%d", info->buffer_id);
}
static char* description_getter(GdamBufferManagerInfo* info)
{
  return g_strdup (info->description);
}
static char* bpm_getter(GdamBufferManagerInfo* info)
{
  g_return_val_if_fail (info != NULL, g_strdup ("error"));
  if (info->beat_info == NULL)
    {
      return g_strdup ("none");
    }
  return g_strdup_printf ("%f", gdam_beat_info_get_bpm (info->beat_info, 0));
}

static char* unknown_getter(GdamBufferManagerInfo* info)
{
  return g_strdup ("unknown");
}

static CListEntryGetter get_column_filler(const char* title)
{
  if (is_one_of (title, "buffer_id", "buffer-id", "buffer id", (char *) NULL))
    return buffer_id_getter;

  if (is_one_of (title, "description", (char *) NULL))
    return description_getter;

  if (is_one_of (title, "bpm", "tempo", (char *) NULL))
    return bpm_getter;

  return unknown_getter;
}

static void on_scrlist_buffer_drag(GtkSelectionData* selection_data,
				   gpointer	     udata)
{
  CListInfo *info = (CListInfo *) udata;
  GtkCList *clist = info->clist;
  GList *selected = clist->selection;
  int num = selected ? GPOINTER_TO_INT (selected->data) : 0;
  gpointer data = selected ? gtk_clist_get_row_data (clist, num) : NULL;
  GdamBufferManagerInfo *buf_info = (GdamBufferManagerInfo *) data;
  char str[32];
  if (selected)
    sprintf (str, "%d", buf_info->buffer_id);
  else
    sprintf (str, "1");

  gtk_selection_data_set (selection_data, selection_data->target, 8,
			  (guchar *) strdup (str), strlen (str));
}

static void connect_clist(GdamBufferManagerSkin* skin,
                          char**                 column_names,
			  GtkCList*              clist)
{
  CListInfo *info = g_new (CListInfo, 1);
  int i;
  info->selected_connect_id
    = gtk_signal_connect (GTK_OBJECT (clist),
			  "select_row",
			  GTK_SIGNAL_FUNC (clist_set_default), info);
  gdam_widget_string_source_set (GTK_WIDGET (clist),
				 on_scrlist_buffer_drag,
				 "gdam-buffer-id-text", info);

  info->skin = skin;
  info->clist = clist;
  info->num_columns = clist->columns;
  info->column = find_clist_column (info->num_columns, column_names);
  if (info->column < 0)
    g_warning ("no clist column titled Buffer Id");
  info->column_fillers = g_new (CListEntryGetter, info->num_columns);
  for (i = 0; i < info->num_columns; i++)
    {
      char *title = column_names[i];
      if (title == NULL)
	break;
      gdam_debug ("filler for column %d based on %s", i, title);
      info->column_fillers[i] = get_column_filler (title);
    }

  for (; i < info->num_columns; i++)
    info->column_fillers[i] = get_column_filler (NULL);

  gtk_object_ref (GTK_OBJECT (clist));
  skin->clist_list = g_slist_prepend (skin->clist_list, info);

  info->notifier = g_new0 (GdamBufferManagerNotifier, 1);
  info->notifier->buffer_add_callback = clist_buffer_add_callback;
  info->notifier->buffer_remove_callback = clist_buffer_remove_callback;
  info->notifier->buffer_change_callback = clist_buffer_change_callback;
  info->notifier->user_data = info;
  gdam_buffer_manager_subscribe (info->notifier);
}

static void disconnect_clist(gpointer value, gpointer user_data)
{
  CListInfo *info = (CListInfo *) value;
  gtk_signal_disconnect (GTK_OBJECT (info->clist), info->selected_connect_id);
  gdam_buffer_manager_unsubscribe (info->notifier);
  g_free (info);
}

static gboolean gdam_buffer_manager_clist_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  if (!GTK_IS_CLIST (object))
    {
      g_warning ("%s only valid for CList's", handler_names[0]);
      return TRUE;
    }
  connect_clist (GDAM_BUFFER_MANAGER_SKIN (skin),
		 handler_names + 1, GTK_CLIST (object));
  return TRUE;
}

typedef struct _FloatInfo FloatInfo;
struct _FloatInfo {
    GtkObject*  object;
    float	amt;
};

/* Sets the delay value to be based on a bpm (either its own or the value from
 * the infomanager) */
static void set_num_beats (GtkWidget* wid, FloatInfo* float_info) 
{
  GdamBufferManagerSkin* skin;
  skin = GDAM_BUFFER_MANAGER_SKIN (float_info->object);
  gdam_buffer_manager_set_num_beats (gdam_buffer_get_default (), 
      				     float_info->amt);
}

static gboolean gdam_buffer_manager_set_num_beats_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  FloatInfo *float_info;
  GdamBufferManagerSkin* bufman;
  float amount;
  char *endp;

  if (handler_names[1] == NULL)
    {
      g_warning ("%s needs a real-valued argument", handler_names[0]);
      return TRUE;
    }
  amount = strtod (handler_names[1], &endp);
  if (endp == handler_names[1] || *endp != '\0')
    return TRUE;
  bufman = GDAM_BUFFER_MANAGER_SKIN (skin);

  float_info = g_new (FloatInfo, 1);
  float_info->object = GTK_OBJECT (skin);
  float_info->amt = amount;
  gtk_signal_connect (object,
		      signal_name, 
		      GTK_SIGNAL_FUNC (set_num_beats), 
		      float_info);
  return TRUE;
}

static gboolean gdam_buffer_manager_description_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamBufferManagerSkin* bufman;
  bufman = GDAM_BUFFER_MANAGER_SKIN (skin);
  if (!GTK_IS_ENTRY (object))
    {
      g_warning ("%s only valid for Entries", handler_names[0]);
      return TRUE;
    }
  bufman->description_entry = GTK_ENTRY (object);
  return TRUE;
}

static gboolean gdam_buffer_manager_id_label_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamBufferManagerSkin* bufman;
  bufman = GDAM_BUFFER_MANAGER_SKIN (skin);
  if (!GTK_IS_LABEL (object))
    {
      g_warning ("%s only valid for Labels", handler_names[0]);
      return TRUE;
    }
  bufman->id_label = GTK_LABEL (object);
  return TRUE;
}


static void bpm_adj_changed (GtkAdjustment*         adj, 
    			     GdamBufferManagerSkin* skin)
{
  gdam_buffer_manager_set_bpm (gdam_buffer_get_default (), 
      			       adj->value);
}

static void fb_adj_changed (GtkAdjustment*         adj, 
    			    GdamBufferManagerSkin* skin)
{
  gdam_buffer_manager_set_first_beat (gdam_buffer_get_default (), 
      			              adj->value);
}

static gboolean gdam_buffer_manager_bpm_adjustment_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamBufferManagerSkin* bufman;
  GtkAdjustment* adj;
  adj = util_get_widget_adjustment (GTK_WIDGET (object));
  bufman = GDAM_BUFFER_MANAGER_SKIN (skin);
  if (!GTK_IS_ADJUSTMENT (adj))
    {
      g_warning ("%s only valid for adjustments", handler_names[0]);
      return TRUE;
    }
  bufman->bpm_adj = adj;
  gtk_signal_connect (GTK_OBJECT (adj),
		      "value_changed",
		      GTK_SIGNAL_FUNC (bpm_adj_changed), 
		      skin);
  return TRUE;
}

static gboolean gdam_buffer_manager_fb_adjustment_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamBufferManagerSkin* bufman;
  GtkAdjustment* adj;
  adj = util_get_widget_adjustment (GTK_WIDGET (object));
  bufman = GDAM_BUFFER_MANAGER_SKIN (skin);
  if (!GTK_IS_ADJUSTMENT (adj))
    {
      g_warning ("%s only valid for adjustments", handler_names[0]);
      return TRUE;
    }
  bufman->fb_adj = adj;
  gtk_signal_connect (GTK_OBJECT (adj),
		      "value_changed",
		      GTK_SIGNAL_FUNC (fb_adj_changed), 
		      skin);
  return TRUE;
}

static void num_beats_adj_changed (GtkAdjustment*         adj, 
    				   GdamBufferManagerSkin* skin)
{
  gdam_buffer_manager_set_num_beats (gdam_buffer_get_default (), 
      				     adj->value);
}

static gboolean gdam_buffer_manager_num_beats_adjustment_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamBufferManagerSkin* bufman;
  GtkAdjustment* adj;
  adj = util_get_widget_adjustment (GTK_WIDGET (object));
  bufman = GDAM_BUFFER_MANAGER_SKIN (skin);
  if (!GTK_IS_ADJUSTMENT (adj))
    {
      g_warning ("%s only valid for adjustments", handler_names[0]);
      return TRUE;
    }
  bufman->num_beats_adj = adj;
  gtk_signal_connect (GTK_OBJECT (adj),
		      "value_changed",
		      GTK_SIGNAL_FUNC (num_beats_adj_changed), 
		      skin);
  return TRUE;
}




static void do_remove_buffer(GdamBufferManagerSkin* skin)
{
  gdam_buffer_manager_soft_destroy (gdam_buffer_get_default ());
}
static gboolean gdam_buffer_manager_remove_selected_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name,
			     GTK_SIGNAL_FUNC (do_remove_buffer),
			     (GtkObject *) skin);
  return TRUE;
}

static void do_destroy_buffer(GdamBufferManagerSkin* skin)
{
	/** XXX: we use the default buffer, not the buffer selected
	         here! (in the clists...) */
  gdam_buffer_manager_hard_destroy (gdam_buffer_get_default ());
}

static gboolean gdam_buffer_manager_destroy_selected_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name,
			     GTK_SIGNAL_FUNC (do_destroy_buffer),
			     (GtkObject *) skin);
  return TRUE;
}

static void gdam_buffer_manager_skin_destroy(GtkObject* object)
{
  GdamBufferManagerSkin *skin = GDAM_BUFFER_MANAGER_SKIN (object);
  g_slist_foreach (skin->clist_list, disconnect_clist, NULL);
  g_slist_free (skin->clist_list);
  skin->clist_list = NULL;
  (*parent_class->destroy) (object);
}

static GdamSkinGenericConnect connect_infos[] =
{
  { "gdam_buffer_manager_clist", 
    gdam_buffer_manager_clist_handler },
  { "gdam_buffer_manager_description_entry", 
    gdam_buffer_manager_description_entry_handler },
  { "gdam_buffer_manager_id_label", 
    gdam_buffer_manager_id_label_handler },
  { "gdam_buffer_manager_bpm_adjustment", 
    gdam_buffer_manager_bpm_adjustment_handler },
  { "gdam_buffer_manager_fb_adjustment", 
    gdam_buffer_manager_fb_adjustment_handler },
  { "gdam_buffer_manager_num_beats_adjustment", 
    gdam_buffer_manager_num_beats_adjustment_handler },
  { "gdam_buffer_manager_set_num_beats", 
    gdam_buffer_manager_set_num_beats_handler },
  { "gdam_buffer_manager_remove_selected",
    gdam_buffer_manager_remove_selected_handler },
  { "gdam_buffer_manager_destroy_selected",
    gdam_buffer_manager_destroy_selected_handler },
};

static void gdam_buffer_manager_skin_class_init(GdamSkinClass* klass)
{
  GdamSkinClass *skin_class = GDAM_SKIN_CLASS (klass);
  GTK_OBJECT_CLASS (klass)->destroy = gdam_buffer_manager_skin_destroy;
  gdam_skin_class_register_generic_handlers
    (skin_class, G_N_ELEMENTS (connect_infos), connect_infos);
}

