#include "gdambufferviewport.h"
#include "gdamskinutil.h"
#include <gdam/common/gdamalloca.h>
#include <gdam/common/gdamsample.h>
#if 0
  if (viewport->fg_gc == NULL)
    {
      /* We base most of our colors on the style of the viewport widget */
      viewport->beat_line_gc = widget->style->fg_gc[GTK_STATE_PRELIGHT];
      viewport->beat_gc = gdk_gc_new (widget->window);
      /* Make the beat bars yellow */
      gdk_gc_set_foreground (viewport->beat_gc,
			     util_alloc_color_cached (65535, 65535, 0));
    }
#endif


/* --- Data rendering:  Set the render function generically --- */
void
gdam_buffer_viewport_set_render_func (GdamBufferViewport *viewport,
				      const char *render_mode,
				      GdamBufferViewportRenderInitFunc init,
				      GdamBufferViewportRenderFunc render_fg,
				      GdamBufferViewportRenderFunc render_bg,
				      GDestroyNotify render_state_destroy,
				      gpointer            render_data,
				      GDestroyNotify      destroy)
{
  if (viewport->render_data_destroy != NULL)
    (*viewport->render_data_destroy) (viewport->render_data);
  viewport->render_data_destroy = destroy;
  viewport->render_data = render_data;
  viewport->render_init = init;
  viewport->render_fg = render_fg;
  viewport->render_bg = render_bg;
  viewport->render_state_destroy = render_state_destroy;
  {
    char *new_string = g_strdup (render_mode);
    g_free (viewport->render_mode);
    viewport->render_mode = new_string;
  }
}

/* --- Data rendering:  Rendering as a waveform --- */
typedef struct _WaveformRenderState WaveformRenderState;
struct _WaveformRenderState
{
  guint num_fg_segments;
  guint num_bg_segments;
  guint num_sel_fg_segments;
  guint num_sel_bg_segments;
  GdkSegment *segments;	  /* num_fg_segments followed by num_bg_segments */
};

static gpointer
make_waveform_render_state (GdamBufferViewportRenderInfo *render_info)
{
  GdkDrawable *drawable = render_info->drawable;
  GdamBufferViewport *viewport = render_info->viewport;
  GdkGC **gcs = render_info->data;
  gboolean is_double = render_info->is_double;
  guint num_columns = render_info->num_columns;
  guint num_rows = render_info->num_rows;
  gpointer values = render_info->values;
  guint x = render_info->x;
  guint y = render_info->y;
  guint width = render_info->width;
  guint height = render_info->height;
  guint draw_width = render_info->draw_width;
  guint draw_height = render_info->draw_height;
  gboolean has_selection = render_info->has_selection;
  gint selection_start = render_info->selection_start;
  gint selection_end = render_info->selection_end;
  gint first_valid_column = render_info->valid_output_start;
  gint last_valid_column = render_info->valid_output_end;
  GdkSegment *fg_segments = alloca (sizeof (GdkSegment) * 2 * width);
  GdkSegment *bg_segments = alloca (sizeof (GdkSegment) * 2 * width);
  GdkSegment *sel_fg_segments = alloca (sizeof (GdkSegment) * 2 * width);
  GdkSegment *sel_bg_segments = alloca (sizeof (GdkSegment) * 2 * width);
  guint num_fg_segments = 0;
  guint num_bg_segments = 0;
  guint num_sel_fg_segments = 0;
  guint num_sel_bg_segments = 0;
  gint xat, x_end;
  WaveformRenderState *rv;
  g_return_val_if_fail (num_rows == 1 || num_rows == 2, FALSE);

  if (selection_end < selection_start)
    has_selection = FALSE;

  if (viewport != NULL)
    {
      GtkWidget *widget = GDAM_SKIN_GET_WIDGET (viewport);
      g_return_val_if_fail (widget != NULL, FALSE);
      if (gcs[0] == NULL)
	{
	  gcs[0] = widget->style->fg_gc[GTK_STATE_NORMAL];
	  gdk_gc_ref (gcs[0]);
	}
      if (gcs[1] == NULL)
	{
	  gcs[1] = widget->style->bg_gc[GTK_STATE_NORMAL];
	  gdk_gc_ref (gcs[1]);
	}
    }
  g_return_val_if_fail (gcs[0] != NULL && gcs[1] != NULL, FALSE);

#define SWAP_SELECTION_SEGMENT_ARRAYS()			\
  G_STMT_START{						\
    { GdkSegment *tmp = sel_fg_segments;		\
      sel_fg_segments = fg_segments;			\
      fg_segments = tmp;				\
    } { guint tmp = num_fg_segments;			\
      num_fg_segments = num_sel_fg_segments;		\
      num_sel_fg_segments = tmp;			\
    }							\
    { GdkSegment *tmp = sel_bg_segments;		\
      sel_bg_segments = bg_segments;			\
      bg_segments = tmp;				\
    } { guint tmp = num_bg_segments;			\
      num_bg_segments = num_sel_bg_segments;		\
      num_sel_bg_segments = tmp;			\
    }							\
  }G_STMT_END

  /* swap the fg/bg array while in the selected area */
  if (has_selection && selection_start < (int) x && (int) x < selection_end)
    SWAP_SELECTION_SEGMENT_ARRAYS ();

  xat = x;
  x_end = x + width;

  while (xat < x_end)
    {
      /* y_start > y_end ==> all background, which is a good default */
      gint y_start = y + draw_height;
      gint y_end = y;
      gint xtmp;

      /* analyze the situation around x */
      gint x_sample_start = num_columns * xat / draw_width;
      gint x_sample_end = num_columns * (xat + 1) / draw_width;

      /* scan for min/max values of y */
      /* the same scanning code applies to ints and doubles,
       * modulo the definition of GET_Y(), so
       * we make a macro to avoid code duplication.
       */
#define DO_MIN_MAX_SCAN()						\
	  if (num_rows == 1)						\
	    {								\
	      /* single-valued wave data with doubles */		\
	      for (xtmp = x_sample_start; xtmp < x_sample_end; xtmp++)	\
		{							\
		  gint y = GET_Y (xtmp);				\
		  if (y < y_start)					\
		    y_start = y;					\
		  if (y > y_end)					\
		    y_end = y;						\
		}							\
	    }								\
	  else								\
	    {								\
	      /* min/max data (two values) with doubles */		\
	      for (xtmp = x_sample_start; xtmp < x_sample_end; xtmp++)	\
		{							\
		  gint y_min = GET_Y (xtmp * 2 + 0);			\
		  gint y_max = GET_Y (xtmp * 2 + 1);			\
		  if (y_min < y_start)					\
		    y_start = y_min;					\
		  if (y_max > y_end)					\
		    y_end = y_max;					\
		}							\
	    }

      if (is_double)
	{
	  gdouble *vals = values;
#define GET_Y(x) ((gint) (vals[x] * (draw_height / 2) + draw_height / 2))
	  DO_MIN_MAX_SCAN ()
#undef GET_Y
	}
      else
	{
	  gint *vals = values;
#define GET_Y(x) ((gint) (((gint64) vals[x] * (draw_height / 2) / GDAM_SAMPLE_ONE) + draw_height / 2))
	  DO_MIN_MAX_SCAN ()
#undef GET_Y
	}

      /* macros for adding to the fg/bg arrays and
       * a SWAP macro to simplify the handling of the selected
       * patch.
       */
#define ADD_SEGMENT(array, num, y_start, y_end)		\
  G_STMT_START{						\
      array[num].x1 = xat;				\
      array[num].y1 = y_start;				\
      array[num].x2 = xat;				\
      array[num].y2 = y_end;				\
      num++;						\
  }G_STMT_END
#define ADD_FG_SEGMENT(y1,y2)	ADD_SEGMENT(fg_segments, num_fg_segments, y1, y2)
#define ADD_BG_SEGMENT(y1,y2)	ADD_SEGMENT(bg_segments, num_bg_segments, y1, y2)
      /* swap the fg/bg array while in the selected area */
      if (has_selection && xat == selection_start)
	SWAP_SELECTION_SEGMENT_ARRAYS ();

      /* --- fill the fg/bg segment arrays --- */
      if (x_sample_end < first_valid_column
       || x_sample_start >= last_valid_column
       || y_start > y_end
       || y_end < (int) y
       || y_start >= (int)(y + height))
	ADD_BG_SEGMENT (y, y + height);
      else if (y_start <= (int) y && y_end >= (int)(y + height))
	ADD_FG_SEGMENT (y, y + height);
      else if (y_start < (int) y)
	{
	  /* we overlap since fg is drawn second */
	  ADD_FG_SEGMENT (y, y_end);
	  ADD_BG_SEGMENT (y_end, y + height);
	}
      else if (y_end >= (int) (y + height))
	{
	  /* we overlap since fg is drawn second */
	  ADD_BG_SEGMENT (y, y_start);
	  ADD_FG_SEGMENT (y_start, y + height);
	}
      else 
	{
	  /* we overlap since fg is drawn second */
	  ADD_BG_SEGMENT (y, y_start);
	  ADD_FG_SEGMENT (y_start, y_end);
	  ADD_BG_SEGMENT (y_end, y + height);
	}

      /* swap the fg/bg array while in the selected area */
      if (has_selection && xat == selection_end)
	SWAP_SELECTION_SEGMENT_ARRAYS ();

      xat++;
    }

  /* the fg/bg arrays are swappen when we are in the selected area;
   * unswap them as needed.
   */
  if (has_selection && selection_start <= xat && xat <= selection_end)
    SWAP_SELECTION_SEGMENT_ARRAYS ();


#undef ADD_SEGMENT
#undef ADD_FG_SEGMENT
#undef ADD_BG_SEGMENT
#undef SWAP_SELECTION_SEGMENT_ARRAYS

  rv = g_new (WaveformRenderState, 1);
  rv->num_fg_segments = num_fg_segments;
  rv->num_bg_segments = num_bg_segments;
  rv->num_sel_fg_segments = num_sel_fg_segments;
  rv->num_sel_bg_segments = num_sel_bg_segments;
  rv->segments = g_new (GdkSegment, num_fg_segments + num_bg_segments
			          + num_sel_fg_segments + num_sel_bg_segments);
  memcpy (rv->segments, fg_segments, num_fg_segments * sizeof (GdkSegment));
  memcpy (rv->segments + num_fg_segments,
	  bg_segments, num_bg_segments * sizeof (GdkSegment));
  memcpy (rv->segments + num_fg_segments + num_bg_segments,
	  sel_fg_segments, num_sel_fg_segments * sizeof (GdkSegment));
  memcpy (rv->segments + num_fg_segments + num_bg_segments + num_sel_fg_segments,
	  sel_bg_segments, num_sel_bg_segments * sizeof (GdkSegment));
  return rv;
}


static void
waveform_render_fg (GdamBufferViewportRenderInfo *render_info,
		    gpointer render_state)
{
  WaveformRenderState *waveform_state = render_state;
  GdkGC **gcs = render_info->data;
  gdk_draw_segments (render_info->drawable,
		     gcs[0],
		     waveform_state->segments,
		     waveform_state->num_fg_segments);
  gdk_draw_segments (render_info->drawable,
		     gcs[1],
		     waveform_state->segments
		     + waveform_state->num_fg_segments
		     + waveform_state->num_bg_segments,
		     waveform_state->num_sel_fg_segments);
}

static void
waveform_render_bg (GdamBufferViewportRenderInfo *render_info,
		    gpointer render_state)
{
  WaveformRenderState *waveform_state = render_state;
  GdkGC **gcs = render_info->data;
  gdk_draw_segments (render_info->drawable,
		     gcs[1],
		     waveform_state->segments + waveform_state->num_fg_segments,
		     waveform_state->num_bg_segments);
  gdk_draw_segments (render_info->drawable,
		     gcs[0],
		     waveform_state->segments
		     + waveform_state->num_fg_segments
		     + waveform_state->num_bg_segments
		     + waveform_state->num_sel_fg_segments,
		     waveform_state->num_sel_bg_segments);
}

static void
delete_waveform_render_state (gpointer render_state)
{
  WaveformRenderState *waveform_state = render_state;
  g_free (waveform_state->segments);
  g_free (waveform_state);
}

static void
free_2_gcs (GdkGC **gcs)
{
  if (gcs[0] != NULL)
    gdk_gc_unref (gcs[0]);
  if (gcs[1] != NULL)
    gdk_gc_unref (gcs[1]);
  g_free (gcs);
}

void
gdam_buffer_viewport_set_render_waveform (GdamBufferViewport *viewport,
					  GdkGC              *fg_gc,
					  GdkGC              *bg_gc)
{
  GdkGC **gcs;
  GtkWidget *widget = GDAM_SKIN_GET_WIDGET (viewport);
  if (fg_gc == NULL && widget != NULL)
    fg_gc = widget->style->fg_gc[GTK_STATE_NORMAL];
  if (bg_gc == NULL && widget != NULL)
    bg_gc = widget->style->bg_gc[GTK_STATE_NORMAL];

  gcs = g_new (GdkGC *, 2);
  gcs[0] = fg_gc;
  gcs[1] = bg_gc;
  if (fg_gc != NULL)
    gdk_gc_ref (fg_gc);
  if (bg_gc != NULL)
    gdk_gc_ref (bg_gc);
  gdam_buffer_viewport_set_render_func (viewport,
					"waveform",
					make_waveform_render_state,
					waveform_render_fg,
					waveform_render_bg,
					delete_waveform_render_state,
					gcs,
					(GDestroyNotify) free_2_gcs);
}


void
gdam_buffer_viewport_render_to_drawable (GdkDrawable        *drawable,
					 GdkGC              *fg_gc,
					 GdkGC              *bg_gc,
					 gboolean            is_double,
					 guint               num_samples,
					 gpointer            samples,
					 guint               x,
					 guint               y,
					 guint               width,
					 guint               height,
					 guint               draw_width,
					 guint               draw_height,
					 gboolean            has_selection,
					 gint                selection_start,
					 gint                selection_end)
{
  GdkGC *gcs[2];
  WaveformRenderState *state;
  GdamBufferViewportRenderInfo render_info;

  g_return_if_fail (fg_gc != NULL && bg_gc != NULL);
  gcs[0] = fg_gc;
  gcs[1] = bg_gc;

  render_info.drawable = drawable;
  render_info.data = gcs;
  render_info.viewport = NULL;
  render_info.is_double = is_double;
  render_info.num_columns = num_samples;
  render_info.num_rows = 1;
  render_info.valid_output_start = 0;
  render_info.valid_output_end = num_samples;
  render_info.values = samples;
  render_info.x = x;
  render_info.y = y;
  render_info.width = width;
  render_info.height = height;
  render_info.draw_width = draw_width;
  render_info.draw_height = draw_height;
  render_info.has_selection = has_selection;
  render_info.selection_start = selection_start;
  render_info.selection_end = selection_end;

  state = make_waveform_render_state (&render_info);
  waveform_render_bg (&render_info, state);
  waveform_render_fg (&render_info, state);
  delete_waveform_render_state (state);
};

/* --- Data rendering:  Rendering as a 2-d pixmap --- */
static inline void
render_2d_color_from_double (gdouble intensity, guint8 *rgb_out)
{
  if (intensity < 0.0)
    rgb_out[0] = rgb_out[1] = rgb_out[2] = 0;
  else if (intensity < 0.5)
    {
      rgb_out[0] = intensity * (255*2);
    }
  else if (intensity < 1.0)
    {
      rgb_out[0] = 255;
      rgb_out[1] = (intensity - 0.5) * (255*2);
    }
  else if (intensity < 1.5)
    {
      rgb_out[0] = 255;
      rgb_out[1] = 255;
      rgb_out[2] = (intensity - 1.0) * (255*2);
    }
  else
    rgb_out[0] = rgb_out[1] = rgb_out[2] = 255;
}

static inline void
render_2d_color_from_int (gint intensity, guint8 *rgb_out)
{
  render_2d_color_from_double (GDAM_SAMPLE_TO_FLOAT (intensity), rgb_out);
}

static void
render_2d_bg (GdamBufferViewportRenderInfo *render_info,
	      gpointer                      render_state)
{
  guint8 *rgb_data;
  gboolean use_alloca = (render_info->width * render_info->height <= 5000);
  guint8 *rgb_data_at;
  if (use_alloca)
    rgb_data = alloca (render_info->width * render_info->height * 3);
  else
    rgb_data = g_new (guint8, render_info->width * render_info->height * 3);

  rgb_data_at = rgb_data;

  {
    guint x_end = render_info->x + render_info->width;
    guint y_end = render_info->y + render_info->height;
    guint x, y;

#define DO_RENDER(color_from_func)					\
  G_STMT_START{								\
    for (y = render_info->y; y < y_end; y++)				\
      for (x = render_info->x; x < x_end; x++)				\
	{								\
	  gint data_x = x * render_info->num_columns			\
	              / render_info->draw_width;			\
	  gint data_y = y * render_info->num_rows			\
	              / render_info->draw_height;			\
	  guint value_index;						\
	  data_x = CLAMP (data_x, 0, (int) render_info->num_columns - 1);\
	  data_y = CLAMP (data_y, 0, (int) render_info->num_rows - 1);	\
	  value_index = data_x * render_info->num_rows + data_y;	\
	  color_from_func (values[value_index], rgb_data_at); 		\
	  rgb_data_at += 3;						\
	}								\
  }G_STMT_END
    if (render_info->is_double)
      {
	gdouble *values = render_info->values;
	DO_RENDER (render_2d_color_from_double);
      }
    else
      {
	gint *values = render_info->values;
	DO_RENDER (render_2d_color_from_int);
      }
#undef DO_RENDER
  }

  gdk_draw_rgb_image (render_info->drawable,
		      GDAM_SKIN_GET_WIDGET (render_info->viewport)->style->fg_gc[0],
		      render_info->x,
		      render_info->y,
		      render_info->width,
		      render_info->height,
		      GDK_RGB_DITHER_NORMAL,
		      rgb_data,
		      render_info->width * 3);

  if (!use_alloca)
    g_free (rgb_data);
}


void
gdam_buffer_viewport_set_render_2d_tmp (GdamBufferViewport *viewport)
{
  gdam_buffer_viewport_set_render_func (viewport,
					"2d",
					NULL,			/* make_state */
					NULL,			/* render_fg */
					render_2d_bg,		/* render_bg */
					NULL,			/* delete_state */
					NULL,			/* data */
					NULL			/* destroy */
					);
}
