/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamcursor.h"
#include "cursors.xbm"
#include <gdk/gdk.h>

static void change_win_cursor 	      	      (GdkWindow*          win, 
    					       GdamCursorType      curtype);

/* Cursor handling derived from gimp's cursorutil.[ch] */
typedef struct
{
  guchar    *bits;
  guchar    *mask_bits;
  gint       width, height;
  gint       x_hot, y_hot;
  GdkBitmap *bitmap;
  GdkBitmap *mask;
  GdkCursor *cursor;
} BitmapCursor;

static BitmapCursor gdam_cursors[] =
{
    {
	mouse_bits, mouse_mask_bits,
	mouse_width, mouse_height,
	mouse_x_hot, mouse_y_hot, NULL, NULL, NULL
    },
    {
	select_bits, select_mask_bits,
	select_width, select_height,
	select_x_hot, select_y_hot, NULL, NULL, NULL
    },
    {
	move_bits, move_mask_bits,
	move_width, move_height,
	move_x_hot, move_y_hot, NULL, NULL, NULL
    },
    {
	hmove_bits, hmove_mask_bits,
	hmove_width, hmove_height,
	hmove_x_hot, hmove_y_hot, NULL, NULL, NULL
    },
    {
	vmove_bits, vmove_mask_bits,
	vmove_width, vmove_height,
	vmove_x_hot, vmove_y_hot, NULL, NULL, NULL
    },
    {
	tempo_bits, tempo_mask_bits,
	tempo_width, tempo_height,
	tempo_x_hot, tempo_y_hot, NULL, NULL, NULL
    },
    {
	phase_bits, phase_mask_bits,
	phase_width, phase_height,
	phase_x_hot, phase_y_hot, NULL, NULL, NULL
    },
    {
	firstbeat_bits, firstbeat_mask_bits,
	firstbeat_width, firstbeat_height,
	firstbeat_x_hot, firstbeat_y_hot, NULL, NULL, NULL
    },
    {
	beatline_bits, beatline_mask_bits,
	beatline_width, beatline_height,
	beatline_x_hot, beatline_y_hot, NULL, NULL, NULL
    },
    {
	index_bits, index_mask_bits,
	index_width, index_height,
	index_x_hot, index_y_hot, NULL, NULL, NULL
    }
};

static void
create_cursor_bitmaps (BitmapCursor *bmcursor)
{
  if (bmcursor->bitmap == NULL)
    bmcursor->bitmap = gdk_bitmap_create_from_data (NULL, bmcursor->bits,
                                                    bmcursor->width,
                                                    bmcursor->height);
  if (bmcursor->mask == NULL) 
    bmcursor->mask = gdk_bitmap_create_from_data (NULL, bmcursor->mask_bits,
                                                  bmcursor->width,
                                                  bmcursor->height);
}

static void
create_cursor (BitmapCursor *bmcursor)
{ 
  if (bmcursor->bitmap == NULL ||
      bmcursor->mask == NULL)
    create_cursor_bitmaps (bmcursor);
  
  if (bmcursor->cursor == NULL)
    {
      GdkColor fg, bg;

      /* should have a way to configure the mouse colors */
      gdk_color_parse ("#FFFFFF", &bg);
      gdk_color_parse ("#000000", &fg);

      bmcursor->cursor = gdk_cursor_new_from_pixmap (bmcursor->bitmap,
                                                     bmcursor->mask,
                                                     &fg, &bg,
                                                     bmcursor->x_hot,
                                                     bmcursor->y_hot);
    }
  g_return_if_fail (bmcursor->cursor != NULL);
}

void
gdam_set_window_cursor (GdkWindow      *win,
                        GdamCursorType  curtype)
{
  BitmapCursor *bmcursor   = NULL;

  g_return_if_fail (curtype < GDAM_LAST_CURSOR_ENTRY);
  g_return_if_fail (curtype >= GDAM_MOUSE_CURSOR);

  curtype -= GDAM_MOUSE_CURSOR;
  bmcursor = &gdam_cursors[(int)curtype];

  if  (bmcursor->cursor == NULL)
    create_cursor (bmcursor);

  gdk_window_set_cursor (win, bmcursor->cursor);
}

