/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamdefault.h"
#include <gdam/component/gdamutil.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <pwd.h>
#include <sys/stat.h>


/*
 *              gdam_default_file_tilde_expand
 *
 * WARNING:  (should be fixed by beta)
 * You should only call this on stuff we have in here, since
 * the assert(0) should be removed before calling it
 * on user data.
 */
char* gdam_default_file_tilde_expand(const char* file)
{
  if (file[0] == '~')
    {
      if (file[1] == '/' || file[1] == 0)
	{
	  int len = strlen (getenv ("HOME")) + strlen (file + 2) + 2;
	  char *buf = (char *) malloc (len);
	  strcpy (buf, getenv ("HOME"));
	  if (file[1] == '/')
	    {
	      strcat (buf, "/");
	      strcat (buf, file + 2);
	    }
	  return buf;
	}
      else if (isalpha (file[1]))
	{
	  struct passwd *pw;
	  char *endname = strchr (file, '/');
	  if (!endname)
	    return strdup (file);
	  for (pw = getpwent (); pw; pw = getpwent ())
	    {
	      if (strncmp (pw->pw_name, file + 1, endname - file - 1) == 0)
		{
		  int len = strlen (pw->pw_dir) + strlen (endname) + 1;
		  char *buf = (char *) malloc (len);
		  strcpy (buf, pw->pw_dir);
		  strcat (buf, endname);
		  endpwent ();
		  return buf;
		}
	    }
	  fprintf (stderr, "tilde expansion failed:  no such user.\n");
	}
      else
	{
	  fprintf (stderr, "bad tilde expression... %s\n", file);
	}
    }
  return strdup (file);
}

char* gdam_default_file_find(const char* const* filelist)
{
  while (*filelist)
    {
      char *fname = gdam_default_file_tilde_expand (*filelist++);
      if (!fname)
	{
	  fprintf (stderr, "Tilde expansion failed on %s\n", filelist[-1]);
	  continue;
	}
      if (gdam_file_exists (fname))
	return fname;
      free (fname);
    }
  fprintf (stderr, "warning: No default file found.\n");
  return 0;
}
