/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include <stdlib.h>
#include <gdam/component/gdamecho.h>
#include <gdam/common/gdamsample.h>
#include <glade/glade.h>
#include <gdam/component/gdammodel.h>
#include <gdam/component/gdamutil.h>
#include <gtk/gtkspinbutton.h>
#include "gdamechoskin.h"
#include "gdamechobaseskin.h"
#include "gdamskinutil.h"

/*    ____ _                 ____  _          __  __ 
 *   / ___| | __ _ ___ ___  / ___|| |_ _   _ / _|/ _|
 *  | |   | |/ _` / __/ __| \___ \| __| | | | |_| |_ 
 *  | |___| | (_| \__ \__ \  ___) | |_| |_| |  _|  _|
 *   \____|_|\__,_|___/___/ |____/ \__|\__,_|_| |_|  
 */                                                 

static void gdam_echo_skin_class_init(GdamSkinClass* klass);

static void gdam_echo_skin_init(GdamEchoSkin* skin)
{
}

GtkType gdam_echo_skin_get_type()
{
  static GtkType echoskin_type = 0;
  if (!echoskin_type)
    {
      static const GtkTypeInfo echoskin_info = {
	"GdamEchoSkin",
	sizeof (GdamEchoSkin),
	sizeof (GdamEchoSkinClass),
	(GtkClassInitFunc) gdam_echo_skin_class_init,
	(GtkObjectInitFunc) gdam_echo_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      echoskin_type = gtk_type_unique (GDAM_TYPE_SKIN, &echoskin_info);
    }
  return echoskin_type;
}

typedef struct _DelayInfo DelayInfo;
struct _DelayInfo {
    GdamEcho*	echo;
    float	amt;
};

/*  ____  _    _         _____                 _   _                 
 * / ___|| | _(_)_ __   |  ___|   _ _ __   ___| |_(_) ___  _ __  ___ 
 * \___ \| |/ / | '_ \  | |_ | | | | '_ \ / __| __| |/ _ \| '_ \/ __|
 *  ___) |   <| | | | | |  _|| |_| | | | | (__| |_| | (_) | | | \__ \
 * |____/|_|\_\_|_| |_| |_|   \__,_|_| |_|\___|\__|_|\___/|_| |_|___/
 */                                                                 


/* Sets the delay value to be based on a bpm (either its own or the value from
 * the infomanager) */
static void delay_beat(GtkWidget* wid, DelayInfo* delay_info)
{
  float d;
  float bpm;
  float newval;
  GdamEchoBaseClass *echo_base_class;
  GdamModel *model = GDAM_MODEL (delay_info->echo);
  GtkObject *object = GTK_OBJECT (delay_info->echo);
  bpm = gdam_beat_info_get_bpm (&model->beat_info, 0);
  echo_base_class = GDAM_ECHO_BASE_CLASS (object->klass);
  delay_info->echo->beat_factor = delay_info->amt;
  /*  Consult infomanager for bpm? */
  if (bpm <= 1)
    {
      bpm = 60;
    }
  d = 60.0 / bpm;
  newval = d * delay_info->amt;
  gdam_echo_set_delay (delay_info->echo, newval);
}

/* A BPM value can be dropped onto one of the gui's fraction buttons,
 * and the filter's delay will be set to a fraction of the value dropped. */
static void on_delay_tempo_drop(const char* text, gpointer data)
{
  DelayInfo *delay_info = (DelayInfo *) data;
  float bpm = atof (text);
  float newval;
  if (bpm <= 1)
    return;
  newval = delay_info->amt * 60.0 / bpm;
  gdam_echo_set_delay (delay_info->echo, newval);
}


static gboolean gdam_echo_delay_by_beat_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  float amount;
  DelayInfo *delay_info;
  char *endp;
  if (handler_names[1] == NULL)
    {
      g_warning ("%s needs a real-valued argument", handler_names[0]);
      return TRUE;
    }
  amount = strtod (handler_names[1], &endp);
  if (endp == handler_names[1] || *endp != '\0')
    return TRUE;

  delay_info = g_new (DelayInfo, 1);
  delay_info->echo = GDAM_ECHO (skin->model);
  delay_info->amt = amount;
  gtk_signal_connect (object,
		      signal_name, GTK_SIGNAL_FUNC (delay_beat), delay_info);
  gdam_widget_string_dest_set (GTK_WIDGET (object),
			       on_delay_tempo_drop,
			       "gdam-tempo-text", delay_info);
  return TRUE;
}

static GdamSkinGenericConnect connect_infos[] =
{
  {
  "gdam_echo_delay_by_beat", gdam_echo_delay_by_beat_handler}
};

static void gdam_echo_skin_class_init(GdamSkinClass* skin_class)
{
  skin_class->model_type = GDAM_TYPE_ECHO;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
  //gdam_skin_class_register_generic_handlers(skin_class,
  //G_N_ELEMENTS(connect_infos),
  //connect_infos);
}
