#include <gtk/gtktable.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtkeditable.h>
#include <gtk/gtkradiobutton.h>
#include <gtk/gtktext.h>
#include <gtk/gtkentry.h>
#include <gtk/gtkwindow.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkhseparator.h>
#include <stdlib.h>
#include <gdam/component/gdamhistory.h>
#include "gdamlaunchserverdialog.h"

#define RUN_SECURE_DESCRIPTION						    \
	"Sorry... no connection to the gdam-server\n"                       \
	"could be established.  A server must be \n"                        \
	"running, so I'll start one.  There are two \n"                     \
	"possible modes:\n"	            				    \
	"\n"                                                                \
	"If I run the server in secure mode, only\n"                        \
	"this launcher may connect to your soundcard.\n"                    \
	"This is safer, but it prevents you from\n"                         \
	"sharing the soundcard among several clients.\n"                    \
	"\n"                                                                \
	"Running insecurely means that any number of\n"                     \
	"clients may connect.  Depending on how your\n"                     \
	"server is configured, clients may be allowed\n"                    \
	"to connect from any computer.  Currently\n"                        \
	"there is *no* authentication and any client\n"                     \
	"could potentially overwrite any file.  Do not\n"                   \
	"run insecurely if you are on an open network!!\n"

static void text_set(GtkText* widget, const char* text)
{
  gtk_text_insert (widget, NULL, NULL, NULL, text, strlen (text));
}

static GtkWidget* make_sep()
{
  GtkWidget *widget = gtk_hseparator_new ();
  gtk_widget_set_usize (widget, 200, 10);
  gtk_widget_show (widget);
  return widget;
}

static void
gdam_launch_server_dialog_init_unconnected(GdamLaunchServerDialog* rv)
{
  GSList *group;
  GtkBox *main_box;
  GtkTable *table;

  rv->toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  rv->main_vbox = gtk_vbox_new (FALSE, 1);
  main_box = GTK_BOX (rv->main_vbox);
  rv->title_text = gtk_text_new (NULL, NULL);
  gtk_widget_set_usize (rv->title_text, 350, 350);
  gtk_box_pack_start_defaults (main_box, rv->title_text);
  text_set (GTK_TEXT (rv->title_text), RUN_SECURE_DESCRIPTION);
  gtk_text_set_editable (GTK_TEXT (rv->title_text), FALSE);

  rv->server_conf_label = gtk_label_new ("Server Configuration:");
  gtk_box_pack_start_defaults (main_box, rv->server_conf_label);

  rv->run_securely_radio = gtk_radio_button_new_with_label (NULL, "Secure");
  group = gtk_radio_button_group (GTK_RADIO_BUTTON (rv->run_securely_radio));
  rv->run_insecurely_radio 
    = gtk_radio_button_new_with_label (group, "Insecure");
  group = gtk_radio_button_group (GTK_RADIO_BUTTON (rv->run_insecurely_radio));

  rv->action_table = gtk_hbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX (rv->action_table), rv->run_securely_radio, 
                      TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (rv->action_table), rv->run_insecurely_radio, 
                      TRUE, FALSE, 0);
  gtk_box_pack_start_defaults (main_box, rv->action_table);

  rv->server_conf_table = gtk_table_new (2, 2, FALSE);
  rv->server_port_label = gtk_label_new ("Server Port:");
  rv->server_options_label = gtk_label_new ("Server Options:");
  rv->server_port_entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (rv->server_port_entry), "6003");
  rv->server_options_entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (rv->server_options_entry), "");
  table = GTK_TABLE (rv->server_conf_table);
  gtk_table_attach_defaults (table, rv->server_port_label, 0, 1, 0, 1);
  gtk_table_attach_defaults (table, rv->server_port_entry, 1, 2, 0, 1);
  gtk_table_attach_defaults (table, rv->server_options_label, 0, 1, 1, 2);
  gtk_table_attach_defaults (table, rv->server_options_entry, 1, 2, 1, 2);
  gtk_box_pack_start_defaults (main_box, rv->server_conf_table);

  gtk_box_pack_start (main_box, make_sep(), TRUE, FALSE, 0);

  /* "Shall I save these settings?" */
  rv->save_settings_label = gtk_label_new ("Shall I save these settings?");
  gtk_box_pack_start_defaults (main_box, rv->save_settings_label);


  rv->use_settings_radio 
    = gtk_radio_button_new_with_label (NULL, "Save, don't ask again.");
  group = gtk_radio_button_group (GTK_RADIO_BUTTON (rv->use_settings_radio));
  rv->dont_save_settings_radio 
    = gtk_radio_button_new_with_label (group, "Don't save.");
  group 
    = gtk_radio_button_group (GTK_RADIO_BUTTON (rv->dont_save_settings_radio));
  rv->save_settings_radio 
    = gtk_radio_button_new_with_label (group, "Save, but ask again.");
  gtk_box_pack_start_defaults (main_box, rv->dont_save_settings_radio);
  gtk_box_pack_start_defaults (main_box, rv->save_settings_radio);
  gtk_box_pack_start_defaults (main_box, rv->use_settings_radio);

  rv->ok_button = gtk_button_new_with_label ("Ok");
  rv->exit_button = gtk_button_new_with_label ("Exit");
  rv->save_settings_now_button = gtk_button_new_with_label ("Save Settings");
  rv->lower_vbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start_defaults (GTK_BOX (rv->lower_vbox), rv->exit_button);
  gtk_box_pack_start_defaults (GTK_BOX (rv->lower_vbox),
			       rv->save_settings_now_button);
  gtk_box_pack_end_defaults (GTK_BOX (rv->lower_vbox), rv->ok_button);
  gtk_box_pack_start_defaults (main_box, rv->lower_vbox);

  gtk_container_add (GTK_CONTAINER (rv->toplevel), rv->main_vbox);
}

static const char* get_history(const char* key, const char* default_value)
{
  const char *rv;
  rv = gdam_history_get_entry ("LaunchServerSettings", key);
  if (rv == NULL)
    rv = default_value;
  return rv;
}
static void set_history(const char* key, const char* value)
{
  gdam_history_set_entry ("LaunchServerSettings", key, value);
}

/*
 * Loading and saving the settings from the history mechanism.
 */
static void load_settings(GdamLaunchServerDialog* dialog)
{
  const char *value;
  GtkWidget *toggle;

  value = get_history ("WhatToDo", "RunSecurely");
  toggle = NULL;
  if (strcmp (value, "RunInsecurely") == 0)
    {
      toggle = dialog->run_insecurely_radio;
    }
  else if (strcmp (value, "RunSecurely") == 0)
    {
      toggle = dialog->run_securely_radio;
    }
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), TRUE);

  value = get_history ("ServerPort", "6003");
  gtk_entry_set_text (GTK_ENTRY (dialog->server_port_entry), value);

  value = get_history ("ServerOptions", "");
  gtk_entry_set_text (GTK_ENTRY (dialog->server_options_entry), value);


  value = get_history ("Persistence", "SaveSettings");
  if (strcmp (value, "UseSettings") == 0)
    {
      toggle = dialog->use_settings_radio;
    }
  else if (strcmp (value, "DontSaveSettings") == 0)
    {
      toggle = dialog->dont_save_settings_radio;
    }
  else
    {
      toggle = dialog->save_settings_radio;
    }
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), TRUE);
}

static void save_settings(GdamLaunchServerDialog* dialog)
{
  const char *text;

  if (GTK_TOGGLE_BUTTON (dialog->run_insecurely_radio)->active)
    text = "RunInsecurely";
  else if (GTK_TOGGLE_BUTTON (dialog->run_securely_radio)->active)
    text = "RunSecurely";
  else
    text = "Exit";
  set_history ("WhatToDo", text);

  text = gtk_entry_get_text (GTK_ENTRY (dialog->server_port_entry));
  set_history ("ServerPort", text);

  text = gtk_entry_get_text (GTK_ENTRY (dialog->server_options_entry));
  set_history ("ServerOptions", text);

  if (GTK_TOGGLE_BUTTON (dialog->use_settings_radio)->active)
    text = "UseSettings";
  else if (GTK_TOGGLE_BUTTON (dialog->dont_save_settings_radio)->active)
    text = "DontSaveSettings";
  else
    text = "SaveSettings";
  set_history ("Persistence", text);
}

static void do_launch(GdamLaunchServerDialog* dialog)
{
  GdamChannelServerLaunch launch_mode;
  GdamChannel *channel = dialog->channel;
  const char *text;
  int tmp;

  /* set channel's launch_server options [port, options] */
  text = gtk_entry_get_text (GTK_ENTRY (dialog->server_options_entry));
  g_free (channel->launch_server_options);
  channel->launch_server_options = g_strdup (text);

  text = gtk_entry_get_text (GTK_ENTRY (dialog->server_port_entry));
  tmp = atoi (text);
  channel->launch_server_port = (tmp <= 0) ? 6003 : tmp;

  /* tell the channel to do the appropriate thing... */
  if (GTK_TOGGLE_BUTTON (dialog->run_insecurely_radio)->active)
    {
      launch_mode = GDAM_CHANNEL_SERVER_LAUNCH_INSECURE;
    }
  else if (GTK_TOGGLE_BUTTON (dialog->run_securely_radio)->active)
    {
      launch_mode = GDAM_CHANNEL_SERVER_LAUNCH_SECURE;
    }
  else
    {
      launch_mode = GDAM_CHANNEL_SERVER_LAUNCH_ABORT;
    }
  gdam_channel_set_launch_server (channel, launch_mode);

  if (dialog->on_launch_func != NULL)
    (*dialog->on_launch_func) (dialog->on_launch_data);
  if (dialog->on_launch_destroy != NULL)
    (*dialog->on_launch_destroy) (dialog->on_launch_data);
}

static void user_pressed_ok(GdamLaunchServerDialog* dialog)
{
  if (GTK_TOGGLE_BUTTON (dialog->save_settings_radio)->active
      || GTK_TOGGLE_BUTTON (dialog->use_settings_radio)->active)
    {
      save_settings (dialog);
    }

  do_launch (dialog);

  gtk_widget_destroy (dialog->toplevel);
  g_free (dialog);
}
static void user_pressed_exit(GdamLaunchServerDialog* dialog)
{
  if (dialog->on_launch_destroy != NULL)
    (*dialog->on_launch_destroy) (dialog->on_launch_data);
  gdam_channel_set_launch_server (dialog->channel,
				  GDAM_CHANNEL_SERVER_LAUNCH_ABORT);
}
static gboolean launch_server_idle(gpointer rv)
{
  GdamLaunchServerDialog *dialog;
  dialog = rv;
  do_launch (dialog);
  gtk_object_sink (GTK_OBJECT (dialog->toplevel));
  g_free (dialog);
  return FALSE;
}

GdamLaunchServerDialog*
gdam_launch_server_dialog_run(GdamChannel*   channel,
                              GHookFunc      on_launch,
			      gpointer       on_launch_data,
			      GDestroyNotify on_launch_destroy)
{
  GdamLaunchServerDialog *rv;
  rv = g_new0 (GdamLaunchServerDialog, 1);
  rv->channel = channel;
  rv->on_launch_func = on_launch;
  rv->on_launch_data = on_launch_data;
  rv->on_launch_destroy = on_launch_destroy;

  gdam_launch_server_dialog_init_unconnected (rv);
  load_settings (rv);
  if (GTK_TOGGLE_BUTTON (rv->use_settings_radio)->active)
    {
      g_idle_add (launch_server_idle, rv);
      return rv;
    }
  gtk_widget_show_all (rv->toplevel);

  gtk_signal_connect_object (GTK_OBJECT (rv->ok_button),
			     "clicked",
			     GTK_SIGNAL_FUNC (user_pressed_ok),
			     (GtkObject *) rv);

  gtk_signal_connect_object (GTK_OBJECT (rv->exit_button),
			     "clicked",
			     GTK_SIGNAL_FUNC (user_pressed_exit),
			     (GtkObject *) rv);

  gtk_signal_connect_object (GTK_OBJECT (rv->save_settings_now_button),
			     "clicked",
			     GTK_SIGNAL_FUNC (save_settings),
			     (GtkObject *) rv);
  return rv;
}
