/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamskinfactory.h"

static void gdam_skin_factory_init(GdamSkinFactory* skin_factory)
{
}
static void gdam_skin_factory_class_init(GdamSkinFactoryClass* skin_factory_class)
{
}

GtkType gdam_skin_factory_get_type()
{
  static GtkType skin_factory_type = 0;
  if (!skin_factory_type)
    {
      static const GtkTypeInfo skin_factory_info = {
	"GdamSkinFactory",
	sizeof (GdamSkinFactory),
	sizeof (GdamSkinFactoryClass),
	(GtkClassInitFunc) gdam_skin_factory_class_init,
	(GtkObjectInitFunc) gdam_skin_factory_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GTK_TYPE_OBJECT;
      skin_factory_type = gtk_type_unique (parent, &skin_factory_info);
    }
  return skin_factory_type;
}

GdamSkinFactory* gdam_skin_factory_new()
{
  return GDAM_SKIN_FACTORY (gtk_type_new (GDAM_TYPE_SKIN_FACTORY));
}

typedef struct _GdamSkinFactoryProduct GdamSkinFactoryProduct;
struct _GdamSkinFactoryProduct {
	char*         	    model_type_name;
	GdamSkinFactoryFunc builder;
	gpointer            builder_data;
};

GdamSkin*   gdam_skin_factory_skin      (GdamSkinFactory*    factory,
                                         GdamModel*          model)
{
  GtkType type;
  GdamSkinFactoryProduct *product = NULL;
  type = GTK_OBJECT (model)->klass->type;
  g_return_val_if_fail (type != 0, NULL);
  if (factory->model_types == NULL)
    {
      g_warning ("tried to use empty skin factory");
      return NULL;
    }
  while (type != 0)
    {
      product = g_hash_table_lookup (factory->model_types,
				     gtk_type_name (type));
      if (product != NULL)
	break;
      type = gtk_type_parent (type);
    }
  if (product == NULL)
    {
      g_warning ("couldn't make model's skin (model was a %s)",
		 gtk_type_name (GTK_OBJECT (model)->klass->type));
      return NULL;
    }
  return (*product->builder) (model, product->builder_data);
}

void        gdam_skin_factory_add_method(GdamSkinFactory*    skin_factory,
					 const char*         model_type_name,
                                         GdamSkinFactoryFunc builder,
					 gpointer            builder_data)
{
  GdamSkinFactoryProduct *product;
  product = g_new (GdamSkinFactoryProduct, 1);
  product->model_type_name = g_strdup (model_type_name);
  product->builder = builder;
  product->builder_data = builder_data;
  if (skin_factory->model_types == NULL)
    skin_factory->model_types = g_hash_table_new (g_str_hash, g_str_equal);

  g_hash_table_insert (skin_factory->model_types,
		       product->model_type_name, product);
}

