/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gtkpixbutton.h"
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkmain.h>

enum {
	PRESSED,
	RELEASED,
	CLICKED,
	TOGGLED,
	LAST_SIGNAL
};
static guint button_signals[LAST_SIGNAL] = { 0 };
static GtkObjectClass* parent_class = NULL;

static int  gtk_pix_button_expose_event        (GtkWidget*        widget,
                                                GdkEventExpose*   event);
static int  gtk_pix_button_configure_event     (GtkWidget*        widget,
                                                GdkEventConfigure*event);
static void gtk_pix_button_draw                (GtkWidget*        widget,
                                                GdkRectangle*     area);
static void gtk_pix_button_draw_default        (GtkWidget*        widget);
static void gtk_pix_button_draw_focus          (GtkWidget*        widget);
static void gtk_pix_button_size_request        (GtkWidget*        widget,
                                                GtkRequisition*   event);
static int  gtk_pix_button_button_press_event  (GtkWidget*        widget,
                                                GdkEventButton*   event);
static int  gtk_pix_button_button_release_event(GtkWidget*        widget,
                                                GdkEventButton*   event);
static int  gtk_pix_button_enter_notify_event  (GtkWidget*        widget,
                                                GdkEventCrossing* event);
static int  gtk_pix_button_leave_notify_event  (GtkWidget*        widget,
                                                GdkEventCrossing* event);

static void gtk_pix_button_init(GtkPixButton* pix_button)
{

  GTK_WIDGET_SET_FLAGS (pix_button, GTK_CAN_FOCUS | GTK_RECEIVES_DEFAULT);
  gtk_widget_add_events (GTK_WIDGET (pix_button),
			 GDK_EXPOSURE_MASK
			 | GDK_STRUCTURE_MASK
			 | GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
  pix_button->state = GTK_STATE_NORMAL;
}
static void gtk_pix_button_class_init(GtkPixButtonClass* klass)
{
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
  GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);
  widget_class->expose_event = gtk_pix_button_expose_event;
  widget_class->configure_event = gtk_pix_button_configure_event;
  widget_class->button_press_event = gtk_pix_button_button_press_event;
  widget_class->button_release_event = gtk_pix_button_button_release_event;
  widget_class->enter_notify_event = gtk_pix_button_enter_notify_event;
  widget_class->leave_notify_event = gtk_pix_button_leave_notify_event;
  widget_class->size_request = gtk_pix_button_size_request;
  widget_class->draw = gtk_pix_button_draw;
  widget_class->draw_default = gtk_pix_button_draw_default;
  widget_class->draw_focus = gtk_pix_button_draw_focus;
  button_signals[PRESSED] =
    gtk_signal_new ("pressed",
		    GTK_RUN_FIRST,
		    object_class->type,
		    GTK_SIGNAL_OFFSET (GtkPixButtonClass, pressed),
		    gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
  button_signals[RELEASED] =
    gtk_signal_new ("released",
		    GTK_RUN_FIRST,
		    object_class->type,
		    GTK_SIGNAL_OFFSET (GtkPixButtonClass, released),
		    gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
  button_signals[CLICKED] =
    gtk_signal_new ("clicked",
		    GTK_RUN_FIRST | GTK_RUN_ACTION,
		    object_class->type,
		    GTK_SIGNAL_OFFSET (GtkPixButtonClass, clicked),
		    gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
  button_signals[TOGGLED] =
    gtk_signal_new ("toggled",
		    GTK_RUN_FIRST | GTK_RUN_ACTION,
		    object_class->type,
		    GTK_SIGNAL_OFFSET (GtkPixButtonClass, toggled),
		    gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
  gtk_object_class_add_signals (object_class, button_signals, LAST_SIGNAL);
}

GtkType gtk_pix_button_get_type()
{
  static GtkType pix_button_type = 0;
  if (!pix_button_type)
    {
      static const GtkTypeInfo pix_button_info = {
	"GtkPixButton",
	sizeof (GtkPixButton),
	sizeof (GtkPixButtonClass),
	(GtkClassInitFunc) gtk_pix_button_class_init,
	(GtkObjectInitFunc) gtk_pix_button_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GTK_TYPE_EVENT_BOX;
      parent_class = gtk_type_class (parent);
      pix_button_type = gtk_type_unique (parent, &pix_button_info);
    }
  return pix_button_type;
}

static void gtk_pix_button_draw(GtkWidget* widget, GdkRectangle* area)
{
  GtkPixButton *button = GTK_PIX_BUTTON (widget);
  GdkWindow *window = widget->window;
  GdkPixmap *pixmap;
  //GdkGCValues values;
  if (!GTK_WIDGET_DRAWABLE (widget))
    {
      return;
    }
  pixmap = button->pixmaps[button->state];

  if (!pixmap)
    {
      //g_warning ("missing pixmap in draw()");
      return;
    }

  gdk_window_set_back_pixmap (widget->window, NULL, TRUE);
  gdk_window_clear_area (widget->window,
			 area->x, area->y, area->width, area->height);
  if (GTK_WIDGET_HAS_DEFAULT (widget) &&
      GTK_BUTTON (widget)->relief == GTK_RELIEF_NORMAL)
    {
      gtk_paint_box (widget->style, widget->window,
		     GTK_STATE_NORMAL, GTK_SHADOW_IN,
		     area, widget, "buttondefault",
		     area->x, area->y, area->width, area->height);
    }


  //gdk_gc_get_values(widget->style->fg_gc[0], &values);
  //gdk_gc_set_clip_origin(widget->style->fg_gc[0], 0, 0);
  gdk_gc_set_clip_mask (widget->style->fg_gc[0],
			button->masks[button->state]);

  gdk_window_copy_area (window,
			widget->style->fg_gc[0],
			0,
			0,
			pixmap,
			0,
			0,
			widget->allocation.width, widget->allocation.height);

  gdk_gc_set_clip_mask (widget->style->fg_gc[0], NULL);
  //gdk_gc_set_clip_origin(widget->style->fg_gc[0], 
  //values.clip_x_origin,
  //values.clip_y_origin);
}
static void
gtk_pix_button_draw_focus (GtkWidget *widget)
{
  gtk_widget_draw (widget, NULL);
}

static void
gtk_pix_button_draw_default (GtkWidget *widget)
{
  gtk_widget_draw (widget, NULL);
}

static int gtk_pix_button_button_press_event(GtkWidget*       widget,
                                         GdkEventButton*  event)
{
  GtkPixButton *button = GTK_PIX_BUTTON (widget);
  gtk_grab_add (widget);

  if (button->is_display)
    return FALSE;

  if (button->is_toggle)
    {
      if (button->state == GTK_STATE_NORMAL
	  || button->state == GTK_STATE_PRELIGHT)
	{
	  button->state = GTK_STATE_SELECTED;
	}
      else
	{
	  button->state = GTK_STATE_PRELIGHT;
	}
      gtk_pix_button_pressed (button);
      gtk_pix_button_toggled (button);
    }
  else
    {
      button->state = GTK_STATE_SELECTED;
      gtk_pix_button_pressed (button);
    }
  gtk_widget_draw_default (GTK_WIDGET (button));
  return TRUE;
}

static int gtk_pix_button_button_release_event(GtkWidget*       widget,
                                               GdkEventButton*  event)
{
  gboolean clicked = FALSE;
  gboolean toggled = FALSE;
  gboolean otherbutton = FALSE;
  GtkPixButton *button = GTK_PIX_BUTTON (widget);
  gtk_grab_remove (widget);

  if (button->is_display)
    return FALSE;

  if (button->is_toggle)
    {
      if (button->state == GTK_STATE_SELECTED)
	clicked = TRUE;
      gtk_pix_button_released (button);
      if (clicked)
	gtk_pix_button_clicked (button);
      return TRUE;
    }
  if (button->state != GTK_STATE_SELECTED
      && button->state != GTK_STATE_ACTIVE)
    {
      return TRUE;
    }

  if (button->state == GTK_STATE_SELECTED)
    {
      button->state = GTK_STATE_PRELIGHT;
      clicked = TRUE;
    }
  else
    {
      button->state = GTK_STATE_NORMAL;
    }
  gtk_pix_button_released (button);
  if (clicked)
    gtk_pix_button_clicked (button);
  gtk_widget_draw_default (GTK_WIDGET (button));
  return TRUE;
}

void gtk_pix_button_make_active(GtkPixButton* button)
{
  button->state = GTK_STATE_ACTIVE;
  gtk_widget_draw_default (GTK_WIDGET (button));
}
void gtk_pix_button_make_normal(GtkPixButton* button)
{
  button->state = GTK_STATE_NORMAL;
  gtk_widget_draw_default (GTK_WIDGET (button));
}
gboolean gtk_pix_button_get_active(GtkPixButton* button)
{
  if (button->state == GTK_STATE_ACTIVE)
    return TRUE;
  if (button->state == GTK_STATE_SELECTED)
    return TRUE;
  return FALSE;
}


static int gtk_pix_button_expose_event(GtkWidget*       widget,
                                       GdkEventExpose*  event)
{
  GtkPixButton *button = GTK_PIX_BUTTON (widget);
  gtk_widget_draw_default (GTK_WIDGET (button));
  return FALSE;
}
static int gtk_pix_button_configure_event       (GtkWidget*       widget,
                                                 GdkEventConfigure*event)
{
  GtkPixButton *button = GTK_PIX_BUTTON (widget);
  gtk_widget_draw_default (GTK_WIDGET (button));
  return FALSE;
}
static int gtk_pix_button_enter_notify_event(GtkWidget*       widget,
                                             GdkEventCrossing*event)
{
  GtkPixButton *button = GTK_PIX_BUTTON (widget);
  GtkWidget *event_widget;
  event_widget = gtk_get_event_widget ((GdkEvent *) event);
  if (event_widget != widget || event->detail == GDK_NOTIFY_INFERIOR)
    return FALSE;
  if (button->state != GTK_STATE_NORMAL && button->state != GTK_STATE_ACTIVE)
    return FALSE;
  if (button->state == GTK_STATE_NORMAL)
    button->state = GTK_STATE_PRELIGHT;
  else
    button->state = GTK_STATE_SELECTED;
  gtk_widget_draw_default (widget);
  return FALSE;
}
static int gtk_pix_button_leave_notify_event(GtkWidget*       widget,
                                             GdkEventCrossing*event)
{
  GtkPixButton *button = GTK_PIX_BUTTON (widget);
  GtkWidget *event_widget;
  event_widget = gtk_get_event_widget ((GdkEvent *) event);
  if (event_widget != widget || event->detail == GDK_NOTIFY_INFERIOR)
    return FALSE;
  if (button->state != GTK_STATE_PRELIGHT
      && button->state != GTK_STATE_SELECTED)
    return FALSE;
  if (button->state == GTK_STATE_PRELIGHT)
    button->state = GTK_STATE_NORMAL;
  else
    button->state = GTK_STATE_ACTIVE;
  gtk_widget_draw_default (widget);
  return FALSE;
}
static void gtk_pix_button_size_request    (GtkWidget*       widget,
                                            GtkRequisition*  requisition)
{
  GtkPixButton *pix_button = GTK_PIX_BUTTON (widget);
  int state = pix_button->state;
  requisition->width = pix_button->sizes[state].x;
  requisition->height = pix_button->sizes[state].y;
}

void gtk_pix_button_pressed(GtkPixButton* pix_button)
{
  gtk_signal_emit (GTK_OBJECT (pix_button), button_signals[PRESSED]);
}

void gtk_pix_button_released(GtkPixButton* pix_button)
{
  gtk_signal_emit (GTK_OBJECT (pix_button), button_signals[RELEASED]);
}
void gtk_pix_button_clicked(GtkPixButton* pix_button)
{
  gtk_signal_emit (GTK_OBJECT (pix_button), button_signals[CLICKED]);
}
void gtk_pix_button_toggled(GtkPixButton* pix_button)
{
  gtk_signal_emit (GTK_OBJECT (pix_button), button_signals[TOGGLED]);
}

void gtk_pix_button_set_pixmaps (GtkPixButton    *pix_button,
                                 GdkPixmap       *normal,
                                 GdkBitmap       *normal_mask,
                                 GdkPixmap       *active,
                                 GdkBitmap       *active_mask,
                                 GdkPixmap       *prelight,
                                 GdkBitmap       *prelight_mask,
                                 GdkPixmap       *selected,
                                 GdkBitmap       *selected_mask)
{
  int i;
  GdkPixmap *pixmaps[4] = { normal, active, prelight, selected};
  GdkBitmap *masks[4] 
    = { normal_mask, active_mask, prelight_mask, selected_mask};

  if (pix_button->loaded_images)
    return;
  pix_button->loaded_images = TRUE;

  for (i = 0; i < 4; i++)
    {
      gdk_window_get_size (pixmaps[i],
			   (gint *) &pix_button->sizes[i].x,
			   (gint *) &pix_button->sizes[i].y);
      pix_button->pixmaps[i] = pixmaps[i];
      pix_button->masks[i] = masks[i];
    }
  gtk_widget_draw (GTK_WIDGET (pix_button), NULL);
}

