/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "pixmapcache.h"
#include <gdam/common/gdamsearchpath.h>
static GHashTable* cache = NULL;

#define IMAGE_SEARCH_PATH			\
	"~/.gdam/skins:"			\
	GDAM_SKIN_PREFIX ":.:"			\
	"/etc/gdam/skins"

static const char *search_image_filename (const char *filename)
{
  static GHashTable *found_files = NULL;
  const char *rv;
  if (found_files == NULL)
    found_files = g_hash_table_new (g_str_hash, g_str_equal);
  rv = g_hash_table_lookup (found_files, filename);
  if (rv == NULL)
    {
      rv = gdam_search_path (filename, IMAGE_SEARCH_PATH,
                             GDAM_SEARCH_PATH_FIND_FILE);
      if (rv == NULL) 
        return NULL;
      g_hash_table_insert (found_files, (gpointer) filename, (gpointer) rv);
    }
  return rv;
}

PixmapInfo*     pixmap_cache_get_pixmap(GdkWindow*      window,
                                        const char*     filename)
{
  PixmapInfo *info;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  const char *real_filename = filename;

  if (filename[0] != '/')
    {
      real_filename = search_image_filename (filename);
      if (real_filename == NULL)
	{
	  g_warning ("couldn't find image %s in search path", filename);
	  return NULL;
	}
    }

  if (cache == NULL)
    {
      cache = g_hash_table_new (g_str_hash, g_str_equal);
    }
  info = (PixmapInfo *) g_hash_table_lookup (cache, real_filename);
  if (info)
    {
      return info;
    }
  pixmap = gdk_pixmap_create_from_xpm (window, &mask, NULL, real_filename);
  if (pixmap == NULL)
    {
      g_warning ("error loading %s", real_filename);
      return NULL;
    }
  info = g_new (PixmapInfo, 1);
  info->filename = g_strdup (real_filename);
  info->pixmap = pixmap;
  info->mask = mask;
  gdk_window_get_size (pixmap, &info->width, &info->height);
  g_hash_table_insert (cache, info->filename, info);
  return info;
}
