#ifndef __GDAM_BEAT_CALCULATOR_SKIN_H_
#define __GDAM_BEAT_CALCULATOR_SKIN_H_


typedef struct _GdamBeatCalculatorSkin GdamBeatCalculatorSkin;
typedef struct _GdamBeatCalculatorSkinClass GdamBeatCalculatorSkinClass;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "gdambuffereditorskin.h"
#include <gdam/skin/gdambufferviewport.h>
#include <gdam/skin/gtktrackdisplay.h>
#include <gdam/component/gdamsonginfo.h>
#include <gtk/gtkentry.h>
#include <gtk/gtklabel.h>

GtkType gdam_beat_calculator_skin_get_type();
#define GDAM_TYPE_BEAT_CALCULATOR_SKIN			(gdam_beat_calculator_skin_get_type ())
#define GDAM_BEAT_CALCULATOR_SKIN(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_BEAT_CALCULATOR_SKIN, GdamBeatCalculatorSkin))
#define GDAM_BEAT_CALCULATOR_SKIN_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_BEAT_CALCULATOR_SKIN, GdamBeatCalculatorSkinClass))
#define GDAM_IS_BEAT_CALCULATOR_SKIN(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_BEAT_CALCULATOR_SKIN))
#define GDAM_IS_BEAT_CALCULATOR_SKIN_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_BEAT_CALCULATOR_SKIN))

struct _GdamBeatCalculatorSkinClass
{
  GdamBufferEditorSkinClass	skin_class;
};
struct _GdamBeatCalculatorSkin
{
  GdamBufferEditorSkin            skin;

  GdamSongInfo                   *song_info;

  /* The beat information that is being played with. */
  GdamBeatInfo                   *working_beat_info;

  /* Information about the beat-line adornments that
   * are being displayed for the user.
   */
  GdamBufferViewportAdornment    *beat_info_adorn;
  GdamBufferViewportBeatInfo     *beat_info_data;
  GdamBufferViewportAdornment    *beat_array_adorn;
  GdamBufferViewportBeatArray    *beat_array_data;

  /* This flag is to prevent the "bpm" and "beat_info"
   * adjustments from getting into an infinite loop
   * when a new songinfo is loaded (ie. when we resync the
   * adjustments.) */
  gboolean                        is_setting_beatinfo;

  guint                           cache_done_event_id;

  GtkAdjustment                  *start_beat_adj;
  GtkAdjustment                  *bpm_adj;
  GtkAdjustment                  *range_start_adj;
  GtkAdjustment                  *range_end_adj;

  GtkEntry                       *filename_entry;
  GtkEntry*                       range_name_entry;

  gboolean                        is_caching;
  char*                           song;

  /* Should just duplicate the buffer_id in the editor... */
  guint                           buffer_id;

  GTimeVal                        first_tap_time;
  int                             taps;
  gboolean                        have_first_tap;
  GtkLabel                       *preview_label;
  GtkLabel                       *beatinfo_index_label;
  GtkLabel                       *num_ranges_label;

  /* For the beat-scanning process. */
  int                             scanning_pid;
  char                           *wav_filename;
  char                           *pitchtrack_filename;

  int                             beatinfo_index;
  gdouble                         next_first_beat;
  gdouble                         tapped_bpm;

  gdouble                         range_start;
  gdouble                         range_end;
  GtkTrackDisplay                *track_display;

  int				  ignore_count;
  int				  num_taps_to_ignore;
  gboolean			  follow_tapped_tempo;
  gboolean			  follow_tapped_phase;
  GList			  	 *tap_time_list;
  int				  phase_ignore_count;
  int				  phase_taps_to_ignore;
  int				  tempo_ignore_count;
  int				  tempo_taps_to_ignore;
  gdouble			  tap_latency;
  gboolean			  have_read_history;
  GTimeVal			  last_mp3_position_timeval;
  gdouble			  last_mp3_position;

  /* An extra adornment to display current position.  */
  GdamBufferViewportAdornment    *position_adorn;
  /* XXX remove ??? */
  GdamBufferViewportBeatInfo     *position_data;
};

void gdam_beat_calculator_skin_load(GdamBeatCalculatorSkin *skin,
                                    GdamSongInfo           *song_info);
void gdam_beat_calculator_skin_load_from_filename
                                   (GdamBeatCalculatorSkin *skin,
                                    const char             *filename);
void gdam_beat_calculator_skin_beat_info_changed
                                   (GdamBeatCalculatorSkin *skin);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
