/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAMFADERSKIN__H_
#define __GDAMFADERSKIN__H_

#include "../gdamskin.h"
#include "../../component/plugins/gdamfader.h"
#include <gtk/gtk.h>


typedef struct _GdamFaderSkinClass GdamFaderSkinClass;
typedef struct _GdamFaderSkin GdamFaderSkin;


GtkType gdam_fader_skin_get_type();
#define GDAM_TYPE_FADER_SKIN              (gdam_fader_skin_get_type ())
#define GDAM_FADER_SKIN(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_FADER_SKIN, GdamFaderSkin))
#define GDAM_FADER_SKIN_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_FADER_SKIN, GdamFaderSkinClass))
#define GDAM_IS_FADER_SKIN(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_FADER_SKIN))
#define GDAM_IS_FADER_SKIN_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_FADER_SKIN))

struct _GdamFaderSkinClass {
	GdamSkinClass		skin_class;
};



struct _GdamFaderSkin {
	GdamSkin		base;
	GList*			buffers;
	/* The drawing area for the samples... */
	GtkWidget*		drawing_area;
	/* Rendering information. */
	int			circle_radius;
	GdkGC*			circle_gc;
	GdkGC*			grid_gc;
	GdkGC*			profile_gc;
	GdkGC*			background;
	/* doublebuffering */
	GdkPixmap*		backing_pixmap;
	int			preview_num_fades;
	GdamFaderElement* 	preview_fade_array;
	/* If the user is dragging a fader point around,
	 * that is indicated here. */
	int			which_dragging;

	/* Offset from the center of the rendering where the user clicked. */
	int			drag_click_offset_x, drag_click_offset_y;

	/* Height and width of the drawing area. */
	int			draw_width, draw_height;

	/* Which of several effect-producing tacts we should take. */
	gboolean		fade_active;
	gboolean		perf_active;

	/* Used to calculate some effects */
	float 			beat_factor;
	float 			peak_width;
	float			base_width;
};

#endif

