/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdammicrophoneskin.h"
#include <math.h>
#include "../../component/plugins/gdamproducermicrophone.h"
#include "../../component/gdambuffermanager.h"
//#include "../gdamfactory.h"
#include "../gdamskinutil.h"
#include <gdam/common/gdamplugin.h>
#include <gtk/gtkdnd.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <glade/glade.h>
#include <stdlib.h>
#include <stdio.h>
#include <gdam/common/gdamplugin.h>



static void gdam_microphone_skin_class_init(GdamMicrophoneSkinClass* klass);
/*    ____ _                 ____  _          __  __ 
 *   / ___| | __ _ ___ ___  / ___|| |_ _   _ / _|/ _|
 *  | |   | |/ _` / __/ __| \___ \| __| | | | |_| |_ 
 *  | |___| | (_| \__ \__ \  ___) | |_| |_| |  _|  _|
 *   \____|_|\__,_|___/___/ |____/ \__|\__,_|_| |_|  
 */                                                 


static void gdam_microphone_skin_init(GdamMicrophoneSkin* skin)
{
}

GtkType gdam_microphone_skin_get_type()
{
  static GtkType microphone_skin_type = 0;
  if (!microphone_skin_type)
    {
      static const GtkTypeInfo microphone_skin_info = {
	"GdamMicrophoneSkin",
	sizeof (GdamMicrophoneSkin),
	sizeof (GdamMicrophoneSkinClass),
	(GtkClassInitFunc) gdam_microphone_skin_class_init,
	(GtkObjectInitFunc) gdam_microphone_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      microphone_skin_type = gtk_type_unique (GDAM_TYPE_SKIN,
					      &microphone_skin_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return microphone_skin_type;
}

static inline GdamProducerMicrophone* get_microphone(GdamMicrophoneSkin* skin)
{
  return GDAM_PRODUCER_MICROPHONE (GDAM_SKIN (skin)->model);
}

static void on_tempo_string_drop(const char* 		new_bpm_str,
				 gpointer		user_data)
{
  GdamMicrophoneSkin *microphone = GDAM_MICROPHONE_SKIN (user_data);
  GdamBeatInfo *new_beat;
  float new_bpm = atof (new_bpm_str);
  if (new_bpm <= 0)
    return;
  new_beat = gdam_beat_info_new ();
  gdam_beat_info_set_bpm (new_beat, new_bpm);
  gdam_model_set_beat_info (GDAM_SKIN (microphone)->model, new_beat);
  gdam_beat_info_destroy (new_beat);
}

static gboolean gdam_microphone_frame_drag_data_received_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gdam_widget_string_dest_set (GTK_WIDGET (object),
			       on_tempo_string_drop,
			       "gdam-tempo-text",
			       skin);
  return TRUE;
}




static GdamSkinGenericConnect connect_infos[] = {
       { "gdam_microphone_frame_drag_data_received",
          gdam_microphone_frame_drag_data_received_handler	},
};


static void gdam_microphone_skin_class_init(GdamMicrophoneSkinClass* klass)
{
  GdamSkinClass *skin_class = GDAM_SKIN_CLASS (klass);
  skin_class->model_type = GDAM_TYPE_PRODUCER_MICROPHONE;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
}
