#ifndef __GDAM_MINI_NETWORK_SKIN_H_
#define __GDAM_MINI_NETWORK_SKIN_H_


typedef struct _GdamMiniNetworkSkin GdamMiniNetworkSkin;
typedef struct _GdamMiniNetworkSkinClass GdamMiniNetworkSkinClass;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "../gdamskin.h"
#include "../../component/plugins/gdammininetwork.h"
#include <gtk/gtklabel.h>

GtkType gdam_mini_network_skin_get_type();
#define GDAM_TYPE_MINI_NETWORK_SKIN			(gdam_mini_network_skin_get_type ())
#define GDAM_MINI_NETWORK_SKIN(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_MINI_NETWORK_SKIN, GdamMiniNetworkSkin))
#define GDAM_MINI_NETWORK_SKIN_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_MINI_NETWORK_SKIN, GdamMiniNetworkSkinClass))
#define GDAM_IS_MINI_NETWORK_SKIN(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_MINI_NETWORK_SKIN))
#define GDAM_IS_MINI_NETWORK_SKIN_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_MINI_NETWORK_SKIN))

struct _GdamMiniNetworkSkinClass {
	GdamSkinClass		skin_class;
};

struct _GdamMiniNetworkSkin {
	GdamSkin		skin;

	GdkPixmap*		backing_pixmap;
	GtkWidget*		drawing_area;
	GtkLabel*		type_label;
	gdouble			area_length_in_grid;
	int			draw_width, draw_height;
	GdkGC*			background_gc;
	GdkGC*			line_gc;

	GHashTable*		node_table;

	/* For when the user clicks. */
	char*			default_type;

	/* For when the user selects a node. */
	GtkContainer*		edit_container;
	GtkWidget*		control_widget;
	GdamMiniNetworkNode*	controlled_node;

	/* For when a node is being dragged around. */
	GdamMiniNetworkNode*	dragging;

	/* x/y where we have dragged from. */
	int			drag_start_x;
	int			drag_start_y;

	/* tmp end point of pipe drag */
	int			rubber_band_x;
	int			rubber_band_y;

	/* tmp start point of pipe drag */
	int			con_start_x;
	int			con_start_y;

	/* If we are dragging a pipe into existance,
	 * this saves the first endpoint. */
	GdamMiniNetworkNode*	pipe_build_node;
	const char*		pipe_build_input;
	gboolean		has_io_nodes;
};

GdamMiniNetworkNode* gdam_mini_network_skin_create_node
                                (GdamMiniNetworkSkin* skin,
				 int                  x,
				 int                  y,
				 const char*          type,
				 const char*          options);

/* May return NULL if a pipe is already connected to a single-entry
 * input. */
GdamMiniNetworkPipe* gdam_mini_network_skin_create_pipe
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkNode* input,
                                 GdamMiniNetworkNode* output,
                                 const char*          name);


void gdam_mini_network_skin_move_node
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkNode* node,
			         int                  relative_x,
			         int                  relative_y);

void gdam_mini_network_skin_remove_pipe
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkPipe* pipe);

void gdam_mini_network_skin_remove_node
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkNode* node);

/*
 * This function splits a new into two pipes and a node.
 *
 * If the pipe is NULL, it is determined from x,y,
 * which will be the location of the new node.
 *
 *       pipe->source                             pipe->source    
 *            |                                        |          
 *            |                                        |          
 *            |                                        v          
 *            | `pipe'          becomes             new_node     
 *            |                                        |          
 *            |                                        |          
 *            v                                        v          
 *     pipe->destination                        pipe->destination
 *
 * with new_node having type `type', parameters `options'
 * and the upper pipe attaches pipe->source's output
 * to new_node's input `input_name'.
 */
void gdam_mini_network_skin_divide_pipe
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkPipe* pipe,
				 int                  x,
				 int                  y,
				 const char*          type,
				 const char*          options,
				 const char*          input_name);

/* gdam_mini_network_skin_gentle_remove_node attempts
 * to remove a node but retain the various connections:
 *
 *     InputNode1      InputNode2            InputNode1    InputNode2       
 *              \        /                      |      \  /   |            
 *             NodeToRemove       becomes       |       \/    |            
 *             /         \                      |       /\    |           
 *            /           \                     |      /  \   |             
 *    OutputNode1      OutputNode2          OutputNode1    OutputNode2          
 *
 * This can backfire in various ways:
 *    - OutputNodex may not accept as many inputs as NodeToRemove.
 *    - NodeToRemove may have contained a delay necessary to
 *      allow a loop to be evaluated.
 * Various pipes will disappear from the final diagram accordingly.
 */
void gdam_mini_network_skin_gentle_remove_node
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkNode* node);

void gdam_mini_network_skin_thaw(GdamMiniNetworkSkin* skin);
void gdam_mini_network_skin_freeze(GdamMiniNetworkSkin* skin);
/* Make a widget that lets you edit the node. */
GtkWidget* (*GdamMiniNetworkSkinNodeEditorFunc)(GdamMiniNetworkNode* node);

/* Register a new type of node. */
                           
void gdam_mini_network_skin_register_node_type
			(const char*		node_type,
	                 GdkPixmap*		pixmap,
	                 int			num_input_types,
	                 GdkRectangle*          input_areas,
			 char**			input_names,
	                 GdkRectangle*		output_area);
void gdam_mini_network_skin_make_dialog
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkNode* node);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
