/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_MOD_H_
#define __GDAM_MOD_H_

typedef struct _GdamModBuffer GdamModBuffer;
typedef struct _GdamModFile GdamModFile;
typedef struct _GdamModNote GdamModNote;
typedef struct _GdamModPattern GdamModPattern;
typedef enum _GdamModVariant GdamModVariant;

#include <glib.h>
#include <stdio.h>

enum _GdamModVariant {
	GDAM_MOD_MOD,		/* includes 16 and 31 instrument versions */
	GDAM_MOD_XM,
	GDAM_MOD_S3M
};

struct _GdamModBuffer {
	char*			sample_name;
	int			sample_length;
	int			finetune;
	int			linear_volume;
	int			repeat_offset;
	int			repeat_length;
	gint8*			sample_data;
};

struct _GdamModNote {
	guint8			sample;
	guint8			effect;
	guint8			effect_arg;
	
	//int			period;

	gdouble			hertz;
};

struct _GdamModPattern {
	GdamModNote		notes[64];
};
struct _GdamModFile {
	char*			title;
	int			num_buffers;
	GdamModBuffer**		buffers;

	GdamModVariant		variant;
	unsigned		strict : 1,
				amiga_notes : 1;

	/* This is the overall order to play the patterns in. */
	int			pattern_table_size;
	guint8*			pattern_table;

	int			num_patterns;
	GdamModPattern**	patterns;
};



GdamModFile* gdam_mod_file_parse  (FILE* fp);
void         gdam_mod_file_destroy(GdamModFile* file);

/* private mostly, or for streaming use. */
GdamModFile* gdam_mod_file_parse_header(FILE* fp);
GdamModBuffer* gdam_mod_buffer_parse(FILE* fp, GdamModFile* mod_file);
GdamModPattern* gdam_mod_pattern_parse(FILE* fp, GdamModFile* mod_file);

#endif
