/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdampluckedstringskin.h"
#include "../../component/plugins/gdampluckedstring.h"
#include "../gdamskinutil.h"
#include <gtk/gtksignal.h>
#include <gdam/common/gdamplugin.h>

static void gdam_plucked_string_skin_class_init(GdamSkinClass* skin_class);
static void gdam_plucked_string_skin_init(GdamPluckedStringSkin* plucked_string_skin)
{
}

GtkType gdam_plucked_string_skin_get_type()
{
  static GtkType plucked_string_skin_type = 0;
  if (!plucked_string_skin_type)
    {
      static const GtkTypeInfo plucked_string_skin_info = {
	"GdamPluckedStringSkin",
	sizeof (GdamPluckedStringSkin),
	sizeof (GdamPluckedStringSkinClass),
	(GtkClassInitFunc) gdam_plucked_string_skin_class_init,
	(GtkObjectInitFunc) gdam_plucked_string_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_SKIN;
      plucked_string_skin_type = gtk_type_unique (parent,
						  &plucked_string_skin_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return plucked_string_skin_type;
}


static void hertz_changed(GtkAdjustment* adjustment, GdamPluckedString* string)
{
  gdam_debug ("plucked_string: set_hertz: %g", adjustment->value);
  gdam_plucked_string_set_hertz (string, adjustment->value);
}
static void gain_changed(GtkAdjustment* adjustment, GdamPluckedString* string)
{
  gdam_debug ("plucked_string: gain_changed: %g", adjustment->value);
  gdam_plucked_string_set_gain (string, adjustment->value);
}

static void plucked_power_changed (GtkAdjustment*     adjustment,
			           GdamPluckedString* string)
{
  gdam_debug ("plucked_string: plucked_power_changed: %g", adjustment->value);
  gdam_plucked_string_set_pluck (string, adjustment->value);
}
static void plucked(GtkAdjustment* adjustment, GdamPluckedString* string)
{
  gdam_plucked_string_pluck (string);
}

static gboolean gdam_plucked_string_hertz_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkAdjustment *adjustment;
  adjustment = util_get_widget_adjustment (GTK_WIDGET (object));
  gtk_signal_connect (GTK_OBJECT (adjustment),
		      "value_changed",
		      GTK_SIGNAL_FUNC (hertz_changed), skin->model);
  gdam_delayed_adjustment_init (adjustment);
  return TRUE;
}
static gboolean gdam_plucked_string_gain_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkAdjustment *adjustment;
  adjustment = util_get_widget_adjustment (GTK_WIDGET (object));
  gtk_signal_connect (GTK_OBJECT (adjustment),
		      "value_changed",
		      GTK_SIGNAL_FUNC (gain_changed), skin->model);
  gdam_delayed_adjustment_init (adjustment);
  return TRUE;
}
static gboolean gdam_plucked_string_pluck_power_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkAdjustment *adjustment;
  adjustment = util_get_widget_adjustment (GTK_WIDGET (object));
  gtk_signal_connect (GTK_OBJECT (adjustment),
		      "value_changed",
		      GTK_SIGNAL_FUNC (plucked_power_changed), skin->model);
  gdam_delayed_adjustment_init (adjustment);
  return TRUE;
}

static gboolean gdam_plucked_string_pluck_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect (object,
		      signal_name, GTK_SIGNAL_FUNC (plucked), skin->model);
  return TRUE;
}

static void filter_coeff_changed(GtkAdjustment* adj, GdamPluckedString* plucked)
{
  gdouble coeff = adj->value;
  gdam_plucked_string_set_filter_coeff (plucked, coeff);
}


static gboolean gdam_plucked_string_filter_coeff_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkAdjustment *adjustment;
  adjustment = util_get_widget_adjustment (GTK_WIDGET (object));
  gtk_signal_connect (GTK_OBJECT (adjustment),
		      "value_changed",
		      GTK_SIGNAL_FUNC (filter_coeff_changed), skin->model);
  gdam_delayed_adjustment_init (adjustment);
  return TRUE;
}



static GdamSkinGenericConnect connect_infos[] = {
	{ "gdam_plucked_string_hertz",
           gdam_plucked_string_hertz_handler		},
	{ "gdam_plucked_string_gain",
           gdam_plucked_string_gain_handler		},
	{ "gdam_plucked_string_pluck_power",
           gdam_plucked_string_pluck_power_handler	},
        { "gdam_plucked_string_filter_coeff",
           gdam_plucked_string_filter_coeff_handler	},
	{ "gdam_plucked_string_pluck",
           gdam_plucked_string_pluck_handler		}	};

static void gdam_plucked_string_skin_class_init(GdamSkinClass* skin_class)
{
  skin_class->model_type = GDAM_TYPE_PLUCKED_STRING;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
}
