/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include <gdam/component/gdamproducermp3.h>
#include <gdam/component/gdamsongdatabase.h>
#include <gtk/gtkdnd.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkentry.h>
#include <glade/glade.h>
#include <string.h>
#include <stdlib.h>
#include "gdamproducermp3cacheableskin.h"
#include "../gdamproducermp3skin.h"
#include "../gdamskinutil.h"
#include <gdam/common/gdamplugin.h>

static void gdam_producer_mp3_cacheable_skin_class_init(GdamProducerMp3CacheableSkinClass* klass);
/*    ____ _                 ____  _          __  __ 
 *   / ___| | __ _ ___ ___  / ___|| |_ _   _ / _|/ _|
 *  | |   | |/ _` / __/ __| \___ \| __| | | | |_| |_ 
 *  | |___| | (_| \__ \__ \  ___) | |_| |_| |  _|  _|
 *   \____|_|\__,_|___/___/ |____/ \__|\__,_|_| |_|  
 */                                                 

static void gdam_producer_mp3_cacheable_skin_init(GdamProducerMp3CacheableSkin* skin)
{
}

GtkType gdam_producer_mp3_cacheable_skin_get_type()
{
  static GtkType producermp3cacheableskin_type = 0;
  if (!producermp3cacheableskin_type)
    {
      static const GtkTypeInfo producermp3cacheableskin_info = {
	"GdamProducerMp3CacheableSkin",
	sizeof (GdamProducerMp3CacheableSkin),
	sizeof (GdamProducerMp3CacheableSkinClass),
	(GtkClassInitFunc) gdam_producer_mp3_cacheable_skin_class_init,
	(GtkObjectInitFunc) gdam_producer_mp3_cacheable_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      producermp3cacheableskin_type =
	gtk_type_unique (GDAM_TYPE_PRODUCERMP3SKIN,
			 &producermp3cacheableskin_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return producermp3cacheableskin_type;
}


static void loopadj_changed(GtkAdjustment* loop_adj, GdamSkin* skin)
{
  GdamProducerMp3Cacheable *mp3;
  mp3 = GDAM_PRODUCER_MP3_CACHEABLE (skin->model);
  gdam_producer_mp3_cacheable_set_beats (mp3, loop_adj->value);
}


static void on_punch_in(GdamSkin* skin)
{
  GdamProducerMp3Cacheable *mp3;
  mp3 = GDAM_PRODUCER_MP3_CACHEABLE (skin->model);
  gdam_producer_mp3_cacheable_set_start (mp3);
}

static void do_export(GdamSkin* skin)
{
  GdamProducerMp3Cacheable *mp3;
  mp3 = GDAM_PRODUCER_MP3_CACHEABLE (skin->model);
  gdam_producer_mp3_cacheable_export (mp3);
}

static void do_write(GdamSkin* skin)
{
  GdamProducerMp3Cacheable *mp3;
  mp3 = GDAM_PRODUCER_MP3_CACHEABLE (skin->model);
  gdam_producer_mp3_cacheable_write_to_database (mp3);
}

static void do_read(GdamSkin* skin)
{
  GdamProducerMp3Cacheable *mp3;
  mp3 = GDAM_PRODUCER_MP3_CACHEABLE (skin->model);
  gdam_producer_mp3_cacheable_read_from_database (mp3);
}

static gboolean gdam_producer_mp3_cacheable_loop_beat_adjustment_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkAdjustment *loop_beat_adjustment;
  loop_beat_adjustment = util_get_widget_adjustment (GTK_WIDGET (object));
  gtk_signal_connect (GTK_OBJECT (loop_beat_adjustment),
		      "value_changed",
		      GTK_SIGNAL_FUNC (loopadj_changed), skin);
  gdam_delayed_adjustment_init (loop_beat_adjustment);
  return TRUE;
}

static gboolean gdam_producer_mp3_cacheable_punch_in_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name, on_punch_in, GTK_OBJECT (skin));
  return TRUE;
}

static gboolean gdam_producer_mp3_cacheable_export_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name, do_export, GTK_OBJECT (skin));
  return TRUE;
}

static gboolean gdam_producer_mp3_cacheable_write_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name, do_write, GTK_OBJECT (skin));
  return TRUE;
}

static gboolean gdam_producer_mp3_cacheable_read_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object, signal_name, do_read, GTK_OBJECT (skin));
  return TRUE;
}

static void on_loop_drop (const char* dropped_str, gpointer udata)
{
  GdamSkin *skin;
  GdamProducerMp3Cacheable *mp3;
  GdamSongInfo* si;
  GdamArg arg;
  char **commands;
  char* filename;
  gdouble start, end;
  gdouble length, beats, bpm;
  char *endp;
  int num_commands;

  skin = GDAM_SKIN (udata);
  num_commands = 0;

  g_return_if_fail (dropped_str != NULL);

  commands = g_strsplit (dropped_str, ":", 0);
  while (commands[num_commands] != NULL)
    num_commands++;

  if (num_commands < 3)
  {
    g_warning ("incomplete loop description: %s", dropped_str);
    return;
  }
  filename = commands[0];
  start = strtod (commands[1], &endp);
  if (*endp != '\0')
    {
      g_warning ("error parsing loop: %s", dropped_str);
      return;
    }
  end = strtod (commands[2], &endp);
  if (*endp != '\0')
    {
      g_warning ("error parsing loop: %s", dropped_str);
      return;
    }
  length = start - end;
  si = gdam_song_database_find (gdam_song_database_system (),
				filename);
  bpm = gdam_beat_info_get_bpm (si->beat_info, start);
  beats = length * bpm / 60;
  mp3 = GDAM_PRODUCER_MP3_CACHEABLE (skin->model);
  arg.name = "filename";
  arg.type = GDAM_ARG_TYPE_STRING;
  GDAM_ARG_STRING (&arg) = filename;
  gdam_arg_set_object_arg (GTK_OBJECT (mp3), &arg);
  arg.name = "punch_in";
  arg.type = GDAM_ARG_TYPE_DOUBLE;
  GDAM_ARG_DOUBLE (&arg) = start;
  gdam_arg_set_object_arg (GTK_OBJECT (mp3), &arg);
  gdam_producer_mp3_cacheable_set_beats (mp3, beats);
  gdam_producer_mp3_cacheable_export(mp3);
}

static gboolean gdam_producer_mp3_cacheable_loop_dest_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gdam_widget_string_dest_set (GTK_WIDGET (object),
			       on_loop_drop,
			       "gdam-file-loop-text", 
			       skin);
  return TRUE;
}

static GdamSkinGenericConnect connect_infos[] = {
	{ "gdam_producer_mp3_cacheable_loop_beat_adjustment",
	   gdam_producer_mp3_cacheable_loop_beat_adjustment_handler },
	{ "gdam_producer_mp3_cacheable_export",
	   gdam_producer_mp3_cacheable_export_handler },
	{ "gdam_producer_mp3_cacheable_write",
	   gdam_producer_mp3_cacheable_write_handler },
	{ "gdam_producer_mp3_cacheable_read",
	   gdam_producer_mp3_cacheable_read_handler },
	{ "gdam_producer_mp3_cacheable_loop_dest",
	   gdam_producer_mp3_cacheable_loop_dest_handler },
	{ "gdam_producer_mp3_cacheable_punch_in",
	   gdam_producer_mp3_cacheable_punch_in_handler }
	   								};


static void
gdam_producer_mp3_cacheable_skin_class_init(GdamProducerMp3CacheableSkinClass* klass)
{
  GdamSkinClass *skin_class = GDAM_SKIN_CLASS (klass);
  skin_class->model_type = GDAM_TYPE_PRODUCER_MP3_CACHEABLE;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
}

