/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_SAMPLING_SEQUENCER_SKIN_H_
#define __GDAM_SAMPLING_SEQUENCER_SKIN_H_


typedef struct _GdamSamplingSequencerSkin GdamSamplingSequencerSkin;
typedef struct _GdamSamplingSequencerSkinClass GdamSamplingSequencerSkinClass;
typedef struct _RerhythmInfo RerhythmInfo;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "../gdamskin.h"
#include "../gdambufferviewport.h"
#include "gdamsequencerskin.h"
#include "../../component/plugins/gdamproducercache.h"

GtkType gdam_sampling_sequencer_skin_get_type();
#define GDAM_TYPE_SAMPLING_SEQUENCER_SKIN                       (gdam_sampling_sequencer_skin_get_type ())
#define GDAM_SAMPLING_SEQUENCER_SKIN(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_SAMPLING_SEQUENCER_SKIN, GdamSamplingSequencerSkin))
#define GDAM_SAMPLING_SEQUENCER_SKIN_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_SAMPLING_SEQUENCER_SKIN, GdamSamplingSequencerSkinClass))
#define GDAM_IS_SAMPLING_SEQUENCER_SKIN(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_SAMPLING_SEQUENCER_SKIN))
#define GDAM_IS_SAMPLING_SEQUENCER_SKIN_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_SAMPLING_SEQUENCER_SKIN))

/* A struct which collects various parameters which might be used when 
 * re-working a beat
 */
struct _RerhythmInfo
{
  /* size of division which defines on-beat, in beats.  a value of
   * 0.5 means that 1.0 and 0.5 are whole beat, 0.75 is on the 
   * half-beat, and 0.125 is offbeat
   */
  gdouble         grain_size;

  /* allowed drift from perfect rhythm, in seconds */
  gdouble         leeway;

  /* subsamples which are on the downbeat are played whole_count
   * times.  The first is shifted by  whole_offset beats,
   * position increases by whole_delay beats each time */
  gdouble         whole_offset;
  int             whole_count;
  gdouble         whole_delay;

  /* as above, for subsamples which are on the offbeat */
  gdouble         half_offset;
  int             half_count;
  gdouble         half_delay;

  /* for samples which are not exactly on or offbeat */
  int             offbeat_count;
};

struct _GdamSamplingSequencerSkinClass
{
  GdamSequencerSkinClass          sequencer_skin_class;
};
struct _GdamSamplingSequencerSkin
{
  GdamSequencerSkin               sequencer_skin;
  GdamBufferViewport             *viewport;
  GdamBufferViewportBeatArray    *beat_array;
  GdamBufferViewportAdornment    *beat_array_adorn;
  GdamProducerCache              *cache;
  RerhythmInfo                    rerhythm_info;
  GtkWidget                      *rerhythm_toplevel;

  /* Ordered list of 'samples' from within the current cached buffer */
  GList                          *subsample_infos;                

  /* Ordered list of subsamples from various buffers */
  GList                          *all_subsample_infos;            

  int                             sampled_buffer_id;
  int                             current_subsample_index;

  /* Data used for analysis of buffer */
  int                            *last_samples;
  int                             num_last_samples;
  gdouble                         amplitude_threshold;
  gdouble                         amplitude_ratio;
  gdouble                         window_size;
  /* Distance between sample markers. */
  gdouble                         min_sample_length;
  int                             samples_per_second;
};

/* Add a subsample description to the list, returns index in list */
int 
gdam_sampling_sequencer_skin_add_sample (GdamSamplingSequencerSkin* ssskin,
                                         gdouble                start_time,
                                         gdouble                length);
void
gdam_sampling_sequencer_skin_get_sample (GdamSamplingSequencerSkin* ssskin,
                                         int                    index,
                                         gdouble*               start_time,
                                         gdouble*               length);
void 
gdam_sampling_sequencer_skin_set_sampled_buffer 
                                        (GdamSamplingSequencerSkin* ssskin,
                                         int                        buffer_id);
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
