#ifndef G_LOG_DOMAIN
#undef G_LOG_DOMAIN
#endif
#define G_LOG_DOMAIN		"GdamSongDatabaseMaker"

#include <gdam/component/gdamsongdatabase.h>
#include <gdam/component/gdamhistory.h>
#include <gdam/skin/gdamskin.h>
#include <gdam/common/gdamtildeexpand.h>
#include <gdam/common/gdamplugin.h>
#include "gdamsongdatabasemakerskin.h"
#include <gdam/common/gdammp3info.h>
#include "../gdamskinutil.h"
#include <gtk/gtksignal.h>
#include <glib.h>
#include <sys/wait.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <ctype.h>

static void handle_find_is_name_toggle(GtkToggleButton* button, 
                          GdamSongDatabaseMakerSkin* skin)
{
  int is_active = gtk_toggle_button_get_active (button) ? 1 : 0;
  skin->find_is_name = is_active;
}
static gboolean gdam_song_database_maker_find_is_name_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkToggleButton *button = GTK_TOGGLE_BUTTON (object);
  int is_active;
  g_return_val_if_fail (GTK_IS_TOGGLE_BUTTON (object), TRUE);
  is_active = gtk_toggle_button_get_active (button) ? 1 : 0;
  GDAM_SONG_DATABASE_MAKER_SKIN (skin)->find_is_name = is_active;
  gtk_signal_connect (object,
		      signal_name,
		      GTK_SIGNAL_FUNC (handle_find_is_name_toggle), skin);
  return TRUE;
}

static gboolean gdam_song_database_maker_expression_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkEntry *entry = GTK_ENTRY (object);
  g_return_val_if_fail (entry != NULL, TRUE);
  GDAM_SONG_DATABASE_MAKER_SKIN (skin)->find_name_or_regex = entry;
  return TRUE;
}
static gboolean gdam_song_database_maker_find_path_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkEntry *entry = GTK_ENTRY (object);
  g_return_val_if_fail (entry != NULL, TRUE);
  GDAM_SONG_DATABASE_MAKER_SKIN (skin)->find_starting_path = entry;
  return TRUE;
}

static void handle_traverse_mount_point_toggle(GtkToggleButton* button, 
                          GdamSongDatabaseMakerSkin* skin)
{
  int is_active = gtk_toggle_button_get_active (button) ? 1 : 0;
  skin->find_traverse_mount_points = is_active;
}
static gboolean gdam_song_database_maker_traverse_mount_point_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkToggleButton *button = GTK_TOGGLE_BUTTON (object);
  int is_active;
  g_return_val_if_fail (GTK_IS_TOGGLE_BUTTON (object), TRUE);
  is_active = gtk_toggle_button_get_active (button) ? 1 : 0;
  GDAM_SONG_DATABASE_MAKER_SKIN (skin)->find_traverse_mount_points
    = is_active;
  gtk_signal_connect (object,
		      signal_name,
		      GTK_SIGNAL_FUNC (handle_traverse_mount_point_toggle),
		      skin);
  return TRUE;
}
static void handle_traverse_symlink_toggle(GtkToggleButton* button, 
                          GdamSongDatabaseMakerSkin* skin)
{
  int is_active = gtk_toggle_button_get_active (button) ? 1 : 0;
  skin->find_traverse_symlinks = is_active;
}
static gboolean gdam_song_database_maker_traverse_symlink_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkToggleButton *button = GTK_TOGGLE_BUTTON (object);
  int is_active;
  g_return_val_if_fail (GTK_IS_TOGGLE_BUTTON (object), TRUE);
  is_active = gtk_toggle_button_get_active (button) ? 1 : 0;
  GDAM_SONG_DATABASE_MAKER_SKIN (skin)->find_traverse_symlinks = is_active;
  g_return_val_if_fail (GTK_IS_TOGGLE_BUTTON (object), TRUE);
  gtk_signal_connect (object,
		      signal_name,
		      GTK_SIGNAL_FUNC (handle_traverse_symlink_toggle), skin);
  return TRUE;
}
static void handle_find_inverted_toggle(GtkToggleButton* button, 
                                       GdamSongDatabaseMakerSkin* skin)
{
  int is_active = gtk_toggle_button_get_active (button) ? 1 : 0;
  skin->find_inverted = is_active;
}
static gboolean gdam_song_database_maker_find_inverted_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkToggleButton *button = GTK_TOGGLE_BUTTON (object);
  int is_active;
  g_return_val_if_fail (GTK_IS_TOGGLE_BUTTON (object), TRUE);
  is_active = gtk_toggle_button_get_active (button) ? 1 : 0;
  GDAM_SONG_DATABASE_MAKER_SKIN (skin)->find_inverted = is_active;
  gtk_signal_connect (object,
		      signal_name,
		      GTK_SIGNAL_FUNC (handle_find_inverted_toggle), skin);
  return TRUE;
}

static void do_run_find(GdamSongDatabaseMakerSkin* skin)
{
  gdam_song_database_maker_run_find (skin);
}

static gboolean gdam_song_database_maker_locate_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkEntry *entry = GTK_ENTRY (object);
  g_return_val_if_fail (entry != NULL, TRUE);
  GDAM_SONG_DATABASE_MAKER_SKIN (skin)->locate_entry = entry;
  return TRUE;
}

static void do_run_locate(GdamSongDatabaseMakerSkin* skin)
{
  gdam_song_database_maker_run_locate (skin);
}

static gboolean gdam_song_database_maker_arbitrary_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkEntry *entry = GTK_ENTRY (object);
  g_return_val_if_fail (entry != NULL, TRUE);
  GDAM_SONG_DATABASE_MAKER_SKIN (skin)->arbitrary_entry = entry;
  return TRUE;
}

static gboolean
got_path (GtkObject *object, const char *filename)
{
  GdamSongDatabaseMakerSkin *skin = GDAM_SONG_DATABASE_MAKER_SKIN (object);
  if (skin->find_starting_path)
    gtk_entry_set_text (skin->find_starting_path, filename);
  gdam_history_set_entry ("SongDBMaker", "LastStartingPath", filename);
  return TRUE;  /* destroy the dialog box */
}

static void 
do_path_dialog (GdamSongDatabaseMakerSkin* skin)
{
  const char *path;
  path = gdam_history_get_entry ("SongDBMaker", "LastStartingPath");
  gdam_skin_make_dialog (GTK_OBJECT (skin), got_path, path);
}

static void do_run_arbitrary(GdamSongDatabaseMakerSkin* skin)
{
  gdam_song_database_maker_run_arbitrary (skin);
}

static GdamSkinGenericConnect connect_infos[] = {
	{ "gdam_song_database_maker_find_is_name",
          gdam_song_database_maker_find_is_name_handler		},
	{ "gdam_song_database_maker_expression_entry",
          gdam_song_database_maker_expression_entry_handler	},
	{ "gdam_song_database_maker_find_path_entry",
          gdam_song_database_maker_find_path_entry_handler	},
	{ "gdam_song_database_maker_find_inverted",
          gdam_song_database_maker_find_inverted_handler	},
	{ "gdam_song_database_maker_traverse_mount_point",
          gdam_song_database_maker_traverse_mount_point_handler	},
	{ "gdam_song_database_maker_traverse_symlink",
          gdam_song_database_maker_traverse_symlink_handler	},
	{ "gdam_song_database_maker_locate_entry",
          gdam_song_database_maker_locate_entry_handler		},
	{ "gdam_song_database_maker_arbitrary_entry",
          gdam_song_database_maker_arbitrary_entry_handler	},
};

static GdamSkinSimpleConnect simple_connect_infos[] = {
	{ "gdam_song_database_maker_startpath_dialog",
           (GdamSkinFunc)do_path_dialog				},
	{ "gdam_song_database_maker_run_arbitrary",
           (GdamSkinFunc)do_run_arbitrary			},
	{ "gdam_song_database_maker_run_locate",
           (GdamSkinFunc)do_run_locate				},
	{ "gdam_song_database_maker_run_find",
           (GdamSkinFunc)do_run_find  				},
};

static void 
set_songs_found (GtkObject* object, GdamArg*   arg)
{
  GdamSongDatabaseMakerSkin *skin = GDAM_SONG_DATABASE_MAKER_SKIN (object);
  skin->songs_found = GDAM_ARG_INT (arg);
}

static void 
get_songs_found (GtkObject* object, GdamArg*   arg)
{
  GdamSongDatabaseMakerSkin *skin = GDAM_SONG_DATABASE_MAKER_SKIN (object);
  GDAM_ARG_INT (arg) = skin->songs_found ;
}

static void 
set_songs_processed (GtkObject* object, GdamArg*   arg)
{
  GdamSongDatabaseMakerSkin *skin = GDAM_SONG_DATABASE_MAKER_SKIN (object);
  skin->songs_processed = GDAM_ARG_INT (arg);
}

static void 
get_songs_processed (GtkObject* object, GdamArg*   arg)
{
  GdamSongDatabaseMakerSkin *skin = GDAM_SONG_DATABASE_MAKER_SKIN (object);
  GDAM_ARG_INT (arg) = skin->songs_processed ;
}

static void gdam_song_database_maker_skin_init(GdamSongDatabaseMakerSkin* skin)
{
  skin->pid_table = g_hash_table_new (NULL, NULL);
}
static void gdam_song_database_maker_skin_class_init(GdamSkinClass* skin_class)
{
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
  gdam_skin_class_register_simple_connections 
    				      (skin_class,
				       G_N_ELEMENTS (simple_connect_infos),
				       simple_connect_infos);
  gdam_arg_register (GTK_OBJECT_CLASS (skin_class),
		     "songs_found",
		     GDAM_ARG_TYPE_INT,
		     set_songs_found,
		     get_songs_found,
		     NULL, "0,", 0);
  gdam_arg_register (GTK_OBJECT_CLASS (skin_class),
		     "songs_processed",
		     GDAM_ARG_TYPE_INT,
		     set_songs_processed,
		     get_songs_processed,
		     NULL, "0,", 0);
}

GtkType gdam_song_database_maker_skin_get_type()
{
  static GtkType song_database_maker_skin_type = 0;
  if (!song_database_maker_skin_type)
    {
      static const GtkTypeInfo song_database_maker_skin_info = {
	"GdamSongDatabaseMakerSkin",
	sizeof (GdamSongDatabaseMakerSkin),
	sizeof (GdamSongDatabaseMakerSkinClass),
	(GtkClassInitFunc) gdam_song_database_maker_skin_class_init,
	(GtkObjectInitFunc) gdam_song_database_maker_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_SKIN;
      song_database_maker_skin_type = gtk_type_unique (parent,
						       &song_database_maker_skin_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return song_database_maker_skin_type;
}

static gboolean 
has_extension (const char *filename, const char *extension)
{
  const char *f_extension = strrchr (filename, '.');
  if (f_extension != NULL && extension != NULL &&
      (strcasecmp (f_extension, extension) == 0))
    return TRUE;
  return FALSE;
}


static void build_song_entry(GdamSongDatabase* database, const char* filename)
{
  GdamSongInfo *song_info = gdam_song_database_find (database, filename);
  if (song_info != NULL)
    {
      /* Don't requery the file... */
      return;
    }
  if (has_extension (filename, ".mp3"))
    {
      GdamMp3Info *mp3_info = gdam_mp3_info_new (filename);
      if (mp3_info == NULL)
	{
	  g_warning ("'%s' was not an mp3 file", filename);
	  return;
	}
      //g_message("building song entry for %s", filename);
      song_info = gdam_song_info_new (filename, 0.0, 0.0);
      song_info->length = mp3_info->length;
      gdam_mp3_info_destroy (mp3_info);
    }
  else if (has_extension (filename, ".ogg"))
    {
      GString *out_string = NULL;
      gdouble length = -1;
      char *output = g_new (char, 1024);
      char *command = g_strdup_printf ("ogginfo '%s'", filename);
      FILE *cmd_file;
      
      cmd_file = popen (command, "r");

      while (fgets(output, sizeof(output), cmd_file) != NULL)
	{
	  if (!out_string)
	    out_string = g_string_new (output);
	  else
	    out_string = g_string_append (out_string, output);
	}

      if (pclose (cmd_file) != 0)
	{
	  g_warning ("error running '%s' please check that ogginfo is"
	             " in your path", command);
	}
      else if (out_string == NULL || out_string->str == NULL)
	{
	  g_warning ("'%s' returned no output ", command);
	}
      else 
	{
	  char **lines = g_strsplit (out_string->str, "\n", 32);
	  int i;
	  for (i = 0; lines[i] != NULL; i++)
	    {
	      char **nv = g_strsplit (lines[i], "=", 3);
	      if (nv[0] && nv[1] && strcmp (nv[0], "length") == 0)
		  length = atof (nv[1]);
	      if (nv[0] && nv[1] && strcmp (nv[0], "header_integrity") == 0)
		if (strcmp (nv[1], "pass") != 0)
		  g_warning ("file %s has bad header", filename);
	      if (nv[0] && nv[1] && strcmp (nv[0], "stream_integrity") == 0)
		if (strcmp (nv[1], "pass") != 0)
		  g_warning ("file %s has stream errors", filename);
	      g_strfreev (nv);
	    }
	  g_strfreev (lines);
	}

      song_info = gdam_song_info_new (filename, 0.0, 0.0);
      if (length > 0)
	song_info->length = length;
    }
  else if (has_extension (filename, ".flac"))
    {
      GString *out_string = NULL;
      gdouble length = -1;
      char *output = g_new (char, 1024);
      char *command = g_strdup_printf ("metaflac '%s'", filename);
      FILE *cmd_file;
      
      cmd_file = popen (command, "r");

      while (fgets(output, sizeof(output), cmd_file) != NULL)
	{
	  if (!out_string)
	    out_string = g_string_new (output);
	  else
	    out_string = g_string_append (out_string, output);
	}

      if (pclose (cmd_file) != 0)
	{
	  g_warning ("error running '%s' please check that metaflac is"
	             " in your path", command);
	}
      else if (out_string == NULL || out_string->str == NULL)
	{
	  g_warning ("'%s' returned no output ", command);
	}
      else 
	{
	  char **lines = g_strsplit (out_string->str, "\n", 32);
	  int i;
	  int samples = 0;
	  int sample_rate = 44100;
	  for (i = 0; lines[i] != NULL; i++)
	    {
	      char **nv = g_strsplit (lines[i], ":", 3);
	      //g_message ("handling line %s", lines[i]);
	      if (nv[0] && nv[1] && strstr (nv[0], "sample_rate"))
		{
		  char *rest = nv[1];
	          char **bits;
		  while (rest[0] && !isdigit (rest[0]))
		      rest++;
		  bits = g_strsplit (rest, " ", 3);
		  sample_rate = atoi (bits[0]);
		  g_message ("got sample rate %d", sample_rate);
		  g_strfreev (bits);
		}
	      if (nv[0] && nv[1] && strstr (nv[0], "total samples"))
		{
		  char *rest = nv[1];
	          char **bits;
		  while (rest[0] && !isdigit (rest[0]))
		      rest++;
		  bits = g_strsplit (rest, " ", 3);
		  samples = atoi (bits[0]);
		  g_message ("got total samples %d", samples);
		  g_strfreev (bits);
		}
	    }
	  g_strfreev (lines);
	  length = (gdouble)samples / sample_rate;
	  g_message ("got length %f from %d samples rate %d for song %s", 
		     length, samples, sample_rate, filename);
	}

      song_info = gdam_song_info_new (filename, 0.0, 0.0);
      if (length > 0)
	song_info->length = length;
    }
  else if (has_extension (filename, ".wav") ||
           has_extension (filename, ".aif") ||
           has_extension (filename, ".aiff"))
    {
      GString *out_string = NULL;
      gdouble length = -1;
      char *output = g_new (char, 1024);
      char *command = g_strdup_printf ("sfinfo '%s'", filename);
      FILE *cmd_file;

      cmd_file = popen (command, "r");

      while (fgets(output, sizeof(output), cmd_file) != NULL)
	{
	  if (!out_string)
	    out_string = g_string_new (output);
	  else
	    out_string = g_string_append (out_string, output);
	}

      if (pclose (cmd_file) != 0)
	{
	  g_warning ("error running '%s' please check that sfinfo is"
	             " in your path", command);
	}
      else if (out_string == NULL || out_string->str == NULL)
	{
	  g_warning ("'%s' returned no output ", command);
	}
      else 
	{
	  char **lines = g_strsplit (out_string->str, "\n", 32);
	  int i;
	  for (i = 0; lines[i] != NULL; i++)
	    {
	      char **nv = g_strsplit (lines[i], " ", 12);
	      if (nv[0] && strcmp (nv[0], "Duration") == 0)
		{
		  int i = 1;
		  char *cur = nv[i];
		  while (nv[i] != NULL && strcmp (nv[i], "") == 0)
		    i++;
		  if (nv[i])
		    length = atof (nv[i]);
		}
	      g_strfreev (nv);
	    }
	  g_strfreev (lines);
	}

      song_info = gdam_song_info_new (filename, 0.0, 0.0);
      if (length > 0)
	song_info->length = length;
    }
  else
    {
      song_info = gdam_song_info_new (filename, 0.0, 0.0);
    }
  /*  gboolean gdam_song_info_add_property(GdamSongInfo*     song_info,
     const char*       key,
     const char*       value); */
  g_return_if_fail (song_info != NULL);
  gdam_song_database_add (database, song_info);
}

static gboolean process_one_song(gpointer user_data)
{
  GdamSongDatabaseMakerSkin *maker = user_data;
  char *fname;
  GdamSongDatabase *database = gdam_song_database_system ();
  if (maker->first_pending_file == NULL)
    {
      maker->pending_thread = 0;
      gdam_song_database_thaw (database);
      return FALSE;
    }
  fname = maker->first_pending_file->data;
  //g_message("process_one_song: %s", fname);
  {
    GSList *to_free = maker->first_pending_file;
    maker->first_pending_file = to_free->next;
    if (to_free->next == NULL)
      maker->last_pending_file = NULL;
    g_slist_free_1 (to_free);
  }
  g_return_val_if_fail (database != NULL, FALSE);
  if (gdam_song_database_find (database, fname))
    {
      g_free (fname);
    }
  else
    {
      build_song_entry (database, fname);
    }
  if (maker->first_pending_file == NULL)
    {
      maker->pending_thread = 0;
      gdam_song_database_thaw (database);
      //gtk_signal_emit_by_name(GTK_OBJECT(database), "changed");
    }
  maker->songs_processed++;
  gdam_arg_set (GTK_OBJECT (maker), "songs_processed", maker->songs_processed);
  return maker->first_pending_file != NULL;
}

void  gdam_song_database_maker_add_song
                              (GdamSongDatabaseMakerSkin* skin,
			       const char*                filename)
{
  GSList *list = g_slist_prepend (NULL, g_strdup (filename));
  GdamSongDatabase *database = gdam_song_database_system ();
  if (skin->first_pending_file == NULL)
    skin->first_pending_file = list;
  else
    skin->last_pending_file->next = list;
  skin->last_pending_file = list;
  if (skin->pending_thread == 0)
    {
      gdam_song_database_freeze (database);
      skin->pending_thread = g_idle_add (process_one_song, skin);
    }
  skin->songs_found++;
  gdam_arg_set (GTK_OBJECT (skin), "songs_found", skin->songs_found);
}

typedef struct _FileListThreadData FileListThreadData;
struct _FileListThreadData {
	int			pid;
	GIOChannel*		io_channel;
	GdamSongDatabaseMakerSkin* skin;
	GdamBuffer		line_buffer;
	char			separator;
};

static void add_song_with_name_from_buffer
                              (GdamSongDatabaseMakerSkin* skin,
			       GdamBuffer*                buffer,
			       int                        index)
{
  char *str = alloca (index + 1);
  gdam_buffer_read (buffer, str, index + 1);
  str[index] = '\0';
  //g_message("add song %s", str);
  gdam_song_database_maker_add_song (skin, str);
}

static void cleanup_thread_data(FileListThreadData* thread_data)
{
  int status;
  GdamSongDatabaseMakerSkin *skin = thread_data->skin;
  gdam_buffer_destruct (&thread_data->line_buffer);
  kill (thread_data->pid, SIGKILL);
  g_hash_table_remove (skin->pid_table, GINT_TO_POINTER (thread_data->pid));
  if (waitpid (thread_data->pid, &status, 0) < 0)
    {
      g_warning ("waitpid: pid %d: %s", thread_data->pid, strerror (errno));
    }
  gtk_object_unref (GTK_OBJECT (skin));
  g_free (thread_data);
}
static int do_fork(char** command, int* fd_out)
{
  int fds[2];
  int pid;
  if (pipe (fds) < 0)
    {
      g_warning ("do_fork: couldn't create pipe: %s", strerror (errno));
      return 0;
    }
  while ((pid = fork ()) < 0)
    {
      if (errno == EAGAIN)
	{
	  sleep (5);
	  continue;
	}
      if (errno == EINTR)
	{
	  continue;
	}
      g_warning ("fork failed: %s", strerror (errno));
      return 0;
    }
  if (pid == 0)
    {
      close (fds[0]);
      dup2 (fds[1], STDOUT_FILENO);
      execvp (command[0], command);
      _exit (127);
    }
  close (fds[1]);
  *fd_out = fds[0];
  return pid;
}

static gboolean process_filename_list(GIOChannel   *source,
				      GIOCondition  condition,
			              gpointer      data)
{
  GIOError io_error;
  char buf[4096];
  int num_read;
  int index;
  FileListThreadData *thread_data = data;
  GdamBuffer *buffer;
  GdamSongDatabaseMakerSkin *skin = thread_data->skin;
  io_error = g_io_channel_read (source, buf, sizeof (buf), &num_read);
  if (io_error != G_IO_ERROR_NONE)
    {
      g_warning ("ioerror");
      cleanup_thread_data (thread_data);
      return FALSE;
    }
  //g_message("got %d bytes for external song database making command", num_read);
  buffer = &thread_data->line_buffer;
  gdam_buffer_append (buffer, buf, num_read);
  while ((index = gdam_buffer_index_of (buffer,
					thread_data->separator, 0)) >= 0)
    {
      add_song_with_name_from_buffer (skin, buffer, index);
    }
  if (num_read <= 0)
    {
      cleanup_thread_data (thread_data);
      return FALSE;
    }
  //g_io_channel_unref(thread_data->io_channel);
  return TRUE;
}

guint gdam_song_database_maker_run_command
                              (GdamSongDatabaseMakerSkin* skin,
                               char**                     command,
			       gboolean                   make_update_window,
			       char                       separator)
{
  FileListThreadData *thread_data;
  int pid, fd;

  gdam_arg_set (GTK_OBJECT (skin), "songs_found", 0);
  gdam_arg_set (GTK_OBJECT (skin), "songs_processed", 0);

  thread_data = g_new (FileListThreadData, 1);
  pid = do_fork (command, &fd);
  //g_message("ran command: %s %s", command[0], command[1]);
  if (pid == 0)
    {
      g_warning ("couldn't run command");
      return 0;
    }
  g_return_val_if_fail (fd >= 0, 0);
  gtk_object_ref (GTK_OBJECT (skin));
  thread_data->skin = skin;
  thread_data->io_channel = g_io_channel_unix_new (fd);
  thread_data->pid = pid;
  thread_data->separator = separator;
  gdam_buffer_construct (&thread_data->line_buffer);
  g_io_add_watch (thread_data->io_channel,
		  G_IO_IN | G_IO_HUP, process_filename_list, thread_data);
  g_hash_table_insert (skin->pid_table, GINT_TO_POINTER (pid), thread_data);
  return pid;
}

guint gdam_song_database_maker_run_find(GdamSongDatabaseMakerSkin* skin)
{
  char **names = {NULL};
  char *start_path = NULL;
  char *name = "*.mp3";
  char *command[40];
  char *expanded_path = NULL;
  char *path;
  int i;
  guint rv;

  if (skin->find_starting_path != NULL)
    {
      start_path = gtk_entry_get_text (skin->find_starting_path);
      expanded_path = gdam_tilde_expand (start_path);
    }
  if (start_path == NULL)
    {
      start_path = "/";
    }
  if (expanded_path)
    path = expanded_path;
  else
    path = start_path;

  if (skin->find_name_or_regex != NULL)
    name = gtk_entry_get_text (skin->find_name_or_regex);

  command[0] = "find";
  command[1] = path;
  i = 2;
  if (skin->find_traverse_symlinks)
    command[i++] = "-follow";
  if (!skin->find_traverse_mount_points)
    command[i++] = "-xdev";
  if (skin->find_inverted)
    command[i++] = "!";
  command[i++] = (skin->find_is_name ? "-name" : "-regex");
  if (skin->find_is_name && strstr (name, ","))
    {
      int j = 0;
      names = g_strsplit (name, ",", 10);
      while (names[j] != NULL)
	{
	  if (j != 0)
	    {
	      command[i++] = "-or";
	      command[i++] = "-name";
	    }
	  command[i++] = names[j];
	  j++;
	}
    }
  else
    command[i++] = name;
  command[i] = NULL;
  rv =  gdam_song_database_maker_run_command (skin, command, TRUE, '\n');
  g_strfreev (names);
  return rv;
}

guint gdam_song_database_maker_run_locate(GdamSongDatabaseMakerSkin* skin)
{
  char *command[3];
  char *name = ".mp3";
  if (skin->locate_entry != NULL)
    name = gtk_entry_get_text (skin->locate_entry);
  command[0] = "locate";
  command[1] = name;
  command[2] = NULL;
  return gdam_song_database_maker_run_command (skin, command, TRUE, '\n');
}

guint gdam_song_database_maker_run_arbitrary(GdamSongDatabaseMakerSkin* skin)
{
  char *command[4];
  char *name;
  if (skin->locate_entry == NULL)
    {
      g_warning ("run arbitrary command: no entry box");
      return 0;
    }
  name = gtk_entry_get_text (skin->arbitrary_entry);
  g_message ("running arbitrary command: %s", name);
  command[0] = "/bin/sh";
  command[1] = "-c";
  command[2] = name;
  command[3] = NULL;
  return gdam_song_database_maker_run_command (skin, command, TRUE, '\n');
}

void  gdam_song_database_maker_kill(GdamSongDatabaseMakerSkin* skin, guint id)
{
  FileListThreadData *thread_data;
  thread_data = g_hash_table_lookup (skin->pid_table, GUINT_TO_POINTER (id));
  if (thread_data != NULL)
    cleanup_thread_data (thread_data);
}

