/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamspinskin.h"
#include "../../component/plugins/gdamspin.h"
#include <gdam/component/gdamutil.h>
#include <glade/glade.h>
#include "../gdamechobaseskin.h"
//#include "../gdamfactory.h"
#include "../gdamskin.h"
#include "../gdamskinutil.h"
#include <gdam/common/gdamplugin.h>
#include <gtk/gtklabel.h>


static void gdam_spin_skin_class_init(GdamSpinSkinClass* klass);

/*    ____ _                 ____  _          __  __ 
 *   / ___| | __ _ ___ ___  / ___|| |_ _   _ / _|/ _|
 *  | |   | |/ _` / __/ __| \___ \| __| | | | |_| |_ 
 *  | |___| | (_| \__ \__ \  ___) | |_| |_| |  _|  _|
 *   \____|_|\__,_|___/___/ |____/ \__|\__,_|_| |_|  
 */                                                 


static void gdam_spin_skin_init(GdamSpinSkin* skin)
{
}

GtkType gdam_spin_skin_get_type()
{
  static GtkType spin_skin_type = 0;
  if (!spin_skin_type)
    {
      static const GtkTypeInfo spin_skin_info = {
	"GdamSpinSkin",
	sizeof (GdamSpinSkin),
	sizeof (GdamSpinSkinClass),
	(GtkClassInitFunc) gdam_spin_skin_class_init,
	(GtkObjectInitFunc) gdam_spin_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      spin_skin_type = gtk_type_unique (GDAM_TYPE_ECHO_BASE_SKIN,
					&spin_skin_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return spin_skin_type;
}

static void on_spin_bf_set(gpointer	info_ptr,
			   GdamArg*	arg,
			   GtkObject*	object)
{
  GtkLabel *label = GTK_LABEL (info_ptr);
  GdamSpin *spin = GDAM_SPIN (object);
  char buf[254];
  double sbf, bf;
  bf = GDAM_ARG_DOUBLE (arg);
  if (bf == 0)
    sbf = bf;
  else if (bf < 1)
    sbf = -1.0 / bf;
  else
    sbf = bf;

  if (sbf < 0)
    sprintf (buf, "1/%2.0f", -sbf);
  else if (sbf == 0)
    sprintf (buf, "0");
  else
    sprintf (buf, "%3.0f", sbf);
  gtk_label_set_text (label, buf);
}

static gboolean gdam_spin_skin_sub_bpm_factor_label_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkLabel *label = GTK_LABEL (object);
  gdam_arg_connect (GTK_OBJECT (skin->model),
		    "beat_factor", on_spin_bf_set, label);
  return TRUE;
}

static GdamSkinGenericConnect connect_infos[] = {
	{ "gdam_spin_skin_sub_bpm_factor_label",
	  gdam_spin_skin_sub_bpm_factor_label_handler		},
};

static void gdam_spin_skin_class_init(GdamSpinSkinClass* klass)
{
  GdamSkinClass *skin_class = GDAM_SKIN_CLASS (klass);
  skin_class->model_type = GDAM_TYPE_SPIN;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
}
