/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamwalkerskin.h"
#include <math.h>
#include "../../component/plugins/gdamproducerwalker.h"
#include "../../component/gdambuffermanager.h"
//#include "../gdamfactory.h"
#include "../gdamskinutil.h"
#include <gdam/common/gdamplugin.h>
#include <gtk/gtkdnd.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <glade/glade.h>
#include <stdlib.h>
#include <stdio.h>
#include <gdam/common/gdamplugin.h>


static gboolean gdam_walker_build_new_model(GdamSkin*      skin,
                                                   GdamChannel*   channel,
						   int            num_options,
						   char**         options);

static void gdam_walker_skin_class_init(GdamWalkerSkinClass* klass);
/*    ____ _                 ____  _          __  __ 
 *   / ___| | __ _ ___ ___  / ___|| |_ _   _ / _|/ _|
 *  | |   | |/ _` / __/ __| \___ \| __| | | | |_| |_ 
 *  | |___| | (_| \__ \__ \  ___) | |_| |_| |  _|  _|
 *   \____|_|\__,_|___/___/ |____/ \__|\__,_|_| |_|  
 */                                                 


static void gdam_walker_skin_init(GdamWalkerSkin* skin)
{
}

GtkType gdam_walker_skin_get_type()
{
  static GtkType walker_skin_type = 0;
  if (!walker_skin_type)
    {
      static const GtkTypeInfo walker_skin_info = {
	"GdamWalkerSkin",
	sizeof (GdamWalkerSkin),
	sizeof (GdamWalkerSkinClass),
	(GtkClassInitFunc) gdam_walker_skin_class_init,
	(GtkObjectInitFunc) gdam_walker_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      walker_skin_type = gtk_type_unique (GDAM_TYPE_SKIN, &walker_skin_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return walker_skin_type;
}

enum {
	TARGET_STRING,
};

static GtkTargetEntry walker_target_table[] = {
	{ "gdam-buffer-id-text", 0, TARGET_STRING },
	{ "STRING", 0, TARGET_STRING },
	{ "text/plain", 0, TARGET_STRING }
};

#define NUM_WALKER_TARGETS	 (sizeof(walker_target_table)/sizeof(walker_target_table[0]))

static void on_walker_data_received(
		GtkObject*        object,
		GdkDragContext*   dc,
		gint              x,
		gint              y,
		GtkSelectionData* selection_data, 
		guint             info,
		guint             time,
		gpointer          data)
{
  GdamProducerWalker *walker = GDAM_PRODUCER_WALKER (data);
  GdkAtom type = selection_data->type;
  gdam_debug ("walker producer, data recieved");
  if (type == gdk_atom_intern ("gdam-buffer-id-text", TRUE)
      && (selection_data->length >= 0) && (selection_data->format == 8))
    {
      int buffer_id = (int) atof ((char *) (selection_data->data));
      gdam_debug ("walker producer, id %d dropped", buffer_id);
      gdam_producer_walker_set_buffer_id (walker, buffer_id);
      return;
    }
  g_warning ("type was of unknown atom (in producer-walker)");
  return;
}

static void do_import(GdamWalkerSkin* skin)
{
  GdamModel *model = GDAM_SKIN (skin)->model;
  GdamProducerWalker *walker = GDAM_PRODUCER_WALKER (model);
  int buffer_id;

  buffer_id = gdam_buffer_manager_obtain_id (model);
  if (buffer_id == 0)
    {
      g_message ("no sample has been chosen -- cannot add");
      return;
    }
  gdam_producer_walker_set_buffer_id (walker, buffer_id);
  gdam_buffer_manager_unref (buffer_id, model);

}

#if 0
/*
 *             _
 *   ___   ___| |_ __ ___   _____
 *  / _ \ / __| __/ _` \ \ / / _ \
 * | (_) | (__| || (_| |\ V /  __/
 *  \___/ \___|\__\__,_| \_/ \___|
 * 
 */

static void octave_adj_changed	        (GtkAdjustment* 	adj,
					 GdamSkin*              skin)
{
  GdamModel *model = skin->model;
  GdamProducerWalker *walker = GDAM_PRODUCER_WALKER (model);
  gdouble rate = 440.0 * pow (2.0, adj->value);
  gdam_debug ("setting octave: %f", adj->value);
  gdam_producer_walker_set_frequency (walker, rate);
}
static void walker_octave_changed   (GdamProducerWalker* 	walker,
					GtkAdjustment* 	        adj)
{
  gdouble octave = log (walker->hertz / 440.0) / M_LN2;
  gtk_adjustment_set_value (adj, octave);
}

static gboolean gdam_walker_octave_adjustment_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkAdjustment *adjustment;
  adjustment = util_get_widget_adjustment (GTK_WIDGET (object));
  gtk_signal_connect (GTK_OBJECT (adjustment),
		      "value_changed",
		      GTK_SIGNAL_FUNC (octave_adj_changed), skin);
  gtk_signal_connect_while_alive
    (GTK_OBJECT (skin->model),
     "rate_changed",
     GTK_SIGNAL_FUNC (walker_octave_changed),
     adjustment, GTK_OBJECT (adjustment));
  gtk_adjustment_value_changed (adjustment);
  return TRUE;
}

#endif


static inline GdamProducerWalker* get_walker(GdamWalkerSkin* skin)
{
  return GDAM_PRODUCER_WALKER (GDAM_SKIN (skin)->model);
}

static gboolean gdam_walker_frame_drag_data_received_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_drag_dest_set (GTK_WIDGET (object),
		     GTK_DEST_DEFAULT_ALL,
		     walker_target_table, 3, GDK_ACTION_COPY);
  gtk_signal_connect (object,
		      "drag_data_received",
		      GTK_SIGNAL_FUNC (on_walker_data_received), skin->model);
  return TRUE;
}

static gboolean gdam_walker_import_buffer_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name,
			     GTK_SIGNAL_FUNC (do_import), GTK_OBJECT (skin));
  return TRUE;
}

static gboolean gdam_walker_build_new_model(GdamSkin*      skin,
                                                   GdamChannel*   channel,
						   int            num_options,
						   char**         options)
{
  GdamModel *model;
  model = GDAM_MODEL (gtk_type_new (GDAM_TYPE_PRODUCER_WALKER));
  model->channel = channel;
  skin->model = model;
  return TRUE;
}

static GdamSkinGenericConnect connect_infos[] = {
	{ "gdam_walker_frame_drag_data_received",
          gdam_walker_frame_drag_data_received_handler	},
 	//{ "gdam_walker_octave_adjustment",
          //gdam_walker_octave_adjustment_handler		},
 	{ "gdam_walker_import_buffer",
          gdam_walker_import_buffer_handler			},
};


static void gdam_walker_skin_class_init(GdamWalkerSkinClass* klass)
{
  GdamSkinClass *skin_class = GDAM_SKIN_CLASS (klass);
  skin_class->build_new_model = gdam_walker_build_new_model;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
}
