/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamwavetableskin.h"
#include <math.h>
#include "../../component/plugins/gdamproducerwavetable.h"
#include "../../component/gdambuffermanager.h"
//#include "../gdamfactory.h"
#include "../gdamskinutil.h"
#include <gdam/common/gdamplugin.h>
#include <gtk/gtkdnd.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkentry.h>
#include <glade/glade.h>
#include <stdlib.h>
#include <stdio.h>
#include <gdam/common/gdamplugin.h>


static gboolean gdam_wavetable_build_new_model(GdamSkin*      skin,
                                                   GdamChannel*   channel,
						   int            num_options,
						   char**         options);

static void gdam_wavetable_skin_class_init(GdamWavetableSkinClass* klass);
/*    ____ _                 ____  _          __  __ 
 *   / ___| | __ _ ___ ___  / ___|| |_ _   _ / _|/ _|
 *  | |   | |/ _` / __/ __| \___ \| __| | | | |_| |_ 
 *  | |___| | (_| \__ \__ \  ___) | |_| |_| |  _|  _|
 *   \____|_|\__,_|___/___/ |____/ \__|\__,_|_| |_|  
 */                                                 


static void gdam_wavetable_skin_init(GdamWavetableSkin* skin)
{
}

GtkType gdam_wavetable_skin_get_type()
{
  static GtkType wavetable_skin_type = 0;
  if (!wavetable_skin_type)
    {
      static const GtkTypeInfo wavetable_skin_info = {
	"GdamWavetableSkin",
	sizeof (GdamWavetableSkin),
	sizeof (GdamWavetableSkinClass),
	(GtkClassInitFunc) gdam_wavetable_skin_class_init,
	(GtkObjectInitFunc) gdam_wavetable_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      wavetable_skin_type = gtk_type_unique (GDAM_TYPE_SKIN,
					     &wavetable_skin_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return wavetable_skin_type;
}

enum {
	TARGET_STRING,
};

static GtkTargetEntry wavetable_target_table[] = {
	{ "gdam-buffer-id-text", 0, TARGET_STRING },
	{ "STRING", 0, TARGET_STRING },
	{ "text/plain", 0, TARGET_STRING }
};

#define NUM_WAVETABLE_TARGETS	 (sizeof(wavetable_target_table)/sizeof(wavetable_target_table[0]))

static void on_wavetable_data_received(
		GtkObject*        object,
		GdkDragContext*   dc,
		gint              x,
		gint              y,
		GtkSelectionData* selection_data, 
		guint             info,
		guint             time,
		gpointer          data)
{
  GdamProducerWavetable *wavetable = GDAM_PRODUCER_WAVETABLE (data);
  GdkAtom type = selection_data->type;
  gdam_debug ("wavetable producer, data recieved");
  if (type == gdk_atom_intern ("gdam-buffer-id-text", TRUE)
      && (selection_data->length >= 0) && (selection_data->format == 8))
    {
      int buffer_id = (int) atof ((char *) (selection_data->data));
      gdam_debug ("wavetable producer, id %d dropped", buffer_id);
      gdam_producer_wavetable_set_buffer_id (wavetable, buffer_id);
      return;
    }
  g_warning ("type was of unknown atom (in producer-wavetable)");
  return;
}

static void do_import(GdamWavetableSkin* skin)
{
  GdamModel *model = GDAM_SKIN (skin)->model;
  GdamProducerWavetable *wavetable = GDAM_PRODUCER_WAVETABLE (model);
  int buffer_id;

  buffer_id = gdam_buffer_manager_obtain_id (model);
  if (buffer_id == 0)
    {
      g_message ("no sample has been chosen -- cannot add");
      return;
    }
  gdam_producer_wavetable_set_buffer_id (wavetable, buffer_id);
  gdam_buffer_manager_unref (buffer_id, model);

}

static void update_buffer_entry(GtkEntry* entry, int buffer_id)
{
  char buf[63];
  sprintf (buf, "%d", buffer_id);
  gdam_debug ("Wavetable, updating entry.");
  gtk_entry_set_text (entry, buf);
}

/*
 *   __
 *  / _|_ __ ___  __ _ _   _  ___ _ __   ___ _   _
 * | |_| '__/ _ \/ _` | | | |/ _ \ '_ \ / __| | | |
 * |  _| | |  __/ (_| | |_| |  __/ | | | (__| |_| |
 * |_| |_|  \___|\__, |\__,_|\___|_| |_|\___|\__, |
 *                  |_|                      |___/
 */

static void frequency_adj_changed	(GtkAdjustment* 	adj,
					 GdamSkin*              skin)
{
  GdamModel *model = skin->model;
  GdamProducerWavetable *wavetable = GDAM_PRODUCER_WAVETABLE (model);
  gdam_debug ("setting frequency: %f", adj->value);
  gdam_producer_wavetable_set_frequency (wavetable, adj->value);
}
static void wavetable_rate_changed     (GdamProducerWavetable* 	wavetable,
					GtkAdjustment* 	        adj)
{
  gtk_adjustment_set_value (adj, wavetable->hertz);
}

static gboolean gdam_wavetable_frequency_adjustment_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkAdjustment *adjustment;
  adjustment = util_get_widget_adjustment (GTK_WIDGET (object));
  gtk_signal_connect (GTK_OBJECT (adjustment),
		      "value_changed",
		      GTK_SIGNAL_FUNC (frequency_adj_changed), skin);
  gtk_signal_connect_while_alive
    (GTK_OBJECT (skin->model),
     "rate_changed",
     GTK_SIGNAL_FUNC (wavetable_rate_changed),
     adjustment, GTK_OBJECT (adjustment));
  gtk_adjustment_value_changed (adjustment);
  gdam_delayed_adjustment_init (adjustment);
  return TRUE;
}

/*
 *             _
 *   ___   ___| |_ __ ___   _____
 *  / _ \ / __| __/ _` \ \ / / _ \
 * | (_) | (__| || (_| |\ V /  __/
 *  \___/ \___|\__\__,_| \_/ \___|
 * 
 */

static void octave_adj_changed	        (GtkAdjustment* 	adj,
					 GdamSkin*              skin)
{
  GdamModel *model = skin->model;
  GdamProducerWavetable *wavetable = GDAM_PRODUCER_WAVETABLE (model);
  gdouble rate = 440.0 * pow (2.0, adj->value);
  gdam_debug ("setting octave: %f", adj->value);
  gdam_producer_wavetable_set_frequency (wavetable, rate);
}
static void wavetable_octave_changed   (GdamProducerWavetable* 	wavetable,
					GtkAdjustment* 	        adj)
{
  gdouble octave = log (wavetable->hertz / 440.0) / M_LN2;
  gtk_adjustment_set_value (adj, octave);
}

static gboolean gdam_wavetable_octave_adjustment_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkAdjustment *adjustment;
  adjustment = util_get_widget_adjustment (GTK_WIDGET (object));
  gtk_signal_connect (GTK_OBJECT (adjustment),
		      "value_changed",
		      GTK_SIGNAL_FUNC (octave_adj_changed), skin);
  gtk_signal_connect_while_alive
    (GTK_OBJECT (skin->model),
     "rate_changed",
     GTK_SIGNAL_FUNC (wavetable_octave_changed),
     adjustment, GTK_OBJECT (adjustment));
  gtk_adjustment_value_changed (adjustment);
  return TRUE;
}


static void buffer_entry_changed	(GtkEntry* 		entry, 
					GdamProducerWavetable* 	wavetable)
{
  int buffer_id = (int) atof (gtk_entry_get_text (entry));
  gdam_debug ("wavetable producer, id %d entered", buffer_id);
  gdam_producer_wavetable_set_buffer_id (wavetable, buffer_id);
}

static inline GdamProducerWavetable* get_wavetable(GdamWavetableSkin* skin)
{
  return GDAM_PRODUCER_WAVETABLE (GDAM_SKIN (skin)->model);
}

static gboolean gdam_wavetable_frame_drag_data_received_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_drag_dest_set (GTK_WIDGET (object),
		     GTK_DEST_DEFAULT_ALL,
		     wavetable_target_table, 3, GDK_ACTION_COPY);
  gtk_signal_connect (object,
		      "drag_data_received",
		      GTK_SIGNAL_FUNC (on_wavetable_data_received),
		      skin->model);
  return TRUE;
}

static gboolean gdam_wavetable_import_buffer_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name,
			     GTK_SIGNAL_FUNC (do_import), GTK_OBJECT (skin));
  return TRUE;
}

static gboolean gdam_wavetable_buffer_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamProducerWavetable *wavetable;
  wavetable = GDAM_PRODUCER_WAVETABLE (skin->model);
  gtk_signal_connect_object (GTK_OBJECT (wavetable),
			     "set_buffer_id",
			     GTK_SIGNAL_FUNC (update_buffer_entry), object);
  gtk_signal_connect (object,
		      "changed",
		      GTK_SIGNAL_FUNC (buffer_entry_changed), skin->model);
  return TRUE;
}

static gboolean gdam_wavetable_build_new_model(GdamSkin*      skin,
                                                   GdamChannel*   channel,
						   int            num_options,
						   char**         options)
{
  GdamModel *model;
  model = GDAM_MODEL (gtk_type_new (GDAM_TYPE_PRODUCER_WAVETABLE));
  model->channel = channel;
  skin->model = model;
  return TRUE;
}

static GdamSkinGenericConnect connect_infos[] = {
	{ "gdam_wavetable_frame_drag_data_received",
          gdam_wavetable_frame_drag_data_received_handler	},
 	{ "gdam_wavetable_frequency_adjustment",
          gdam_wavetable_frequency_adjustment_handler		},
 	{ "gdam_wavetable_octave_adjustment",
          gdam_wavetable_octave_adjustment_handler		},
 	{ "gdam_wavetable_import_buffer",
          gdam_wavetable_import_buffer_handler			},
	{ "gdam_wavetable_buffer_entry",
          gdam_wavetable_buffer_entry_handler			}	};


static void gdam_wavetable_skin_class_init(GdamWavetableSkinClass* klass)
{
  GdamSkinClass *skin_class = GDAM_SKIN_CLASS (klass);
  skin_class->build_new_model = gdam_wavetable_build_new_model;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
}
